#ifndef BBOX_
#define BBOX_

//======================================================================
//
// Bbox header
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.6 $
//
//======================================================================

// Bbox constructor needs access to members of Point
#ifndef POINT_
#include "Point.H"
#endif

//===========================================================  SetMinMax
//
// Sets the mn and mx arguments to the minimum and maximum of the other
// two arguments
//
static inline void
SetMinMax( COORD &mn, COORD &mx, COORD const val1, COORD const val2 )
{
    if ( val1 <= val2 ) {
        mn = val1 ;   mx = val2 ;
    } else {
        mn = val2 ;   mx = val1 ;
    }
}                                                           // SetMinMax

//========================================================  AdjustMinMax
//
// Adjusts mn and mx arguments depending on how they compare to the
// third argument. REQUIRES mn <= mx.
//
static inline void
AdjustMinMax( COORD &mn, COORD &mx, COORD const val )
{
    if ( val < mn ) {
        mn = val ;
    } else if ( val > mx ) {
        mx = val ;
    }
}                                                        // AdjustMinMax

//========================================================  AdjustMinMax
//
// Adjusts mn and mx arguments depending on how they compare to another
// pair of min/max arguments. REQUIRES mn <= mx for both pairs.
//
static inline void
AdjustMinMax( COORD &mn, COORD &mx, COORD const zmn, COORD const zmx )
{
    if ( zmn < mn ) mn = zmn ;
    if ( zmx > mx ) mx = zmx ;
}                                                        // AdjustMinMax

//================================================================  Bbox
//
// Bounding box
//
class Bbox {

    friend ostream& operator<<( ostream &os, Bbox const &bbox ) ;

public:
    COORD llx, lly, urx, ury ;

    // ctors
    // default copy ctor and dtor ok
    Bbox() { }
    Bbox( Point const &p ) { llx = urx = p.x ; lly = ury = p.y ; }
    Bbox( Point const &pa, Point const &pb ) {
        SetMinMax( llx, urx, pa.x, pb.x ) ;
        SetMinMax( lly, ury, pa.y, pb.y ) ;
    }

    // operators
    bool operator== ( Bbox const &a ) const {
        return llx == a.llx && lly == a.lly
            && urx == a.urx && ury == a.ury ;
    }

    // other methods

    // Initialize to bbox of a point
    //
    void Set( Point const &p ) { llx = urx = p.x ; lly = ury = p.y ; }

    // Initialize to bbox of a line segment
    //
    void Set( Point const &p, Point const &q ) {
        SetMinMax( llx, urx, p.x, q.x ) ;
        SetMinMax( lly, ury, p.y, q.y ) ;
    }

    // Adjust llx or urx of bbox to include value. MUST have llx <= urx
    // before calling this method.
    //
    void AdjustX( COORD const val ) {
        AdjustMinMax( llx, urx, val ) ;
    }

    // Adjust lly or ury of bbox to include value. MUST have lly <= ury
    // before calling this method.
    //
    void AdjustY( COORD const val ) {
        AdjustMinMax( lly, ury, val ) ;
    }

    // Adjust bbox to include point. bbox MUST be initialized before
    // calling this method to ensure llx <= urx and lly <= ury
    //
    void Adjust( Point const &p ) {
        AdjustX( p.x ) ;   AdjustY( p.y ) ;
    }

    // Adjust bbox to include argument; both MUST be initialized before
    // calling this method to ensure llx <= urx and lly <= ury
    //
    void Adjust( Bbox const &bbox ) {
        AdjustMinMax( llx, urx, bbox.llx, bbox.urx ) ;
        AdjustMinMax( lly, ury, bbox.lly, bbox.ury ) ;
    }

    // Test if x-coordinate is inside (or on boundary)
    bool InsideX( COORD const x ) const {
        return llx <= x && x <= urx ;
    }

    // Test if x-coordinate is strictly inside
    bool StrictInsideX( COORD const x ) const {
        return llx < x && x < urx ;
    }

    // Test if y-coordinate is inside (or on boundary)
    bool InsideY( COORD const y ) const {
        return lly <= y && y <= ury ;
    }

    // Test if y-coordinate is strictly inside
    bool StrictInsideY( COORD const y ) const {
        return lly < y && y < ury ;
    }

    // Test if point is inside (or on boundary)
    bool Inside( Point const &p ) const {
        return InsideX( p.x ) && InsideY( p.y ) ;
    }

    // Test if point is strictly inside
    bool StrictInside( Point const &p ) const {
        return StrictInsideX( p.x ) && StrictInsideY( p.y ) ;
    }

    // Test if line ax + by + c = 0 intersects bbox. MUST have
    // a != 0 || b != 0
    //
    bool Intersect( COORD a, COORD b, COORD c ) const {

        if ( ApproxEq( 0.0, a ) ) {   // line is y = -c/b
            COORD v = -c / b ;
            return lly <= v && v <= ury ;
        }
        if ( ApproxEq( 0.0, b ) ) {   // line is x = -c/a
            COORD v = -c / a ;
            return llx <= v && v <= urx ;
        }

#define SIDE( X, Y ) (a * X + b * Y + c)

        if ( a < 0.0 ) {
            a = -a, b = -b, c = -c ;
        }
        if ( b > 0.0 ) {    // negative slope
            if ( SIDE( llx, lly ) > 0.0 ) {
                return false ;
            }
            return SIDE( urx, ury ) < 0.0 ? false : true ;
        }
        // b < 0; positive slope
        if ( SIDE( llx, ury ) > 0.0 ) {
            return false ;
        }
        return SIDE( urx, lly ) < 0.0 ? false : true ;
#undef SIDE
    }
} ;                                                              // Bbox
#endif  // BBOX_
