#ifndef BITMAP_
#define BITMAP_

//======================================================================
//
// Bitmap
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.3 $
//
//======================================================================

// Need setw()
#ifndef __IOMANIP__
#include <iomanip>
#endif
using namespace std ;

// Need memset
#if !(defined(_MEMORY_H) || defined(_memory_h) || defined(__memory_h__))
extern "C" {
#include <memory.h>
}
#endif

// Need sqrt()
#if !defined(_MATH_H) && !defined(__math_h)
extern "C" {
#include <math.h>
}
#endif

#ifndef GLOBALDEFS_
#include "globaldefs.H"
#endif

//--------------------------------------------------------------  Ipoint
//
// Point with integer coordinates
//
struct Ipoint {
    INT32  x, y ;

    // ctor
    Ipoint( ) {}
    Ipoint( INT32 xx, INT32 yy ) : x( xx ), y( yy ) { }

    // operators
    bool
    operator==( const Ipoint &pt ) const {
        return x == pt.x && y == pt.y ;
    }
} ;

// 8 principal directions
enum EDir8 {
    eNorth, eSouth, eEast, eWest,
    eNorthEast, eNorthWest, eSouthEast, eSouthWest
} ;

//--------------------------------------------------------------  Bitmap
//
// bitmap aligned according to the template arguments as follows:
//                  T        aBits
//                  =====    =====
// 8-bit  aligned:  UINT8    3
// 16-bit aligned:  UINT16   4
// 32-bit aligned:  UINT32   5
// 64-bit aligned:  UINT64   6
//
// Any other T/aBits combination is erroneous.
//
// The alignment unit will be called a 'cell' in comments below.
//
// The bits are ordered LSB-->MSB in y-major order (x increasing
// fastest); note that this bit order is somewhat unusual but is used
// in the X11 bitmap format (which is byte aligned).
//
template< class T, INT32 aBits > class Bitmap {
public:

    // alignBits: 3, 4, 5, or 6 for 8-, 16-, 32- or 64- alignment
    // alignSize: 2^alignBits
    // alignMask: alignSize - 1
    // fullMask:  ~0 (mask with all bits set)
    //
    static INT32 const
        alignBits = aBits,
        alignSize = 1u << alignBits ;
    static T const
        alignMask,  // = alignSize - 1
        fullMask ;  //  = ~static_cast< T >( 0 )

    INT32 mWd, mHt ;
    T     *mData ;

    // ctors
    Bitmap( INT32 const wd, INT32 const ht ) ;

    // dtor
    // NOTE: Character bitmaps are never freed when used with TK canvas
    // bitmap items since TK does not yet provide a Tk_UndefineBitmap()
    // call.  Large pens are created and freed as necessary
    //
    ~Bitmap() {
        delete mData ;
        mData = NIL ; mWd = mHt = 0 ;
    }

    // other methods

    // zero the data member to prevent bitmap data from being deleted
    // when the object is deleted (used when tk assumes ownership of
    // the bitmap data)
    //
    void Zero( ) { mData = NIL ; }

    // return size of entire bitmap in cells
    INT32 Size( ) const { return mHt * RowSize() ; }

    // return cell offset of column from row beginning
    INT32 Cell( INT32 const col ) const { return col >> alignBits ; }

    // return size of row in cells
    INT32 RowSize( ) const { return Cell( mWd + alignMask ) ; }

    // return bit position corresponding to column
    INT32 BitPos( INT32 const col ) const { return col & alignMask ; }

    // return cell mask corresponding to column
    T BitMask( INT32 const col ) const { return 1u << BitPos( col ) ; }

    // return pointer to cell containing (row,col)
    T *Cellp( INT32 const row, INT32 const col ) {
        return mData + row * RowSize() + Cell( col ) ;
    }

    // turn bit at (row,col) on or off according as black is true or not
    //
    void BitOnOff( INT32 const row,
                   INT32 const col,
                   bool  const black = true ) {
        if ( black ) {
            *Cellp( row, col ) |= BitMask( col ) ;
        } else {
            *Cellp( row, col ) &= ~BitMask( col ) ;
        }
    }

    // Paste the argument bitmap at position (row, col) which must fit
    // in its entirety in current bitmap (otherwise a fatal error is
    // generated)
    //
    // Destination bits corresponding to 1 bits of source are set if
    // 'blacken' is true, cleared otherwise. If more general
    // rasterops are needed, pass in a function pointer.
    //
    void Paste( Bitmap const &src, INT32 const row, INT32 const col,
                bool const black = true ) ;

    // Draws straight line in one of 8 principal directions of starting
    // at (x,y) of given length in given direction using the given
    // bitmap as a pen.
    //
    // The line is black or white according as the argument 'black' is
    // true or not.
    //
    // If 'pen' is NIL, a single pixel pen of the appropriate color is
    // used.
    //
    // NOTE: x is the column and y the row.
    //
    void Line8( Bitmap const *pen,
                Ipoint const &pt,
                INT32  const aLen,
                EDir8  const dir,
                bool   const black = true ) ;

    // Draw a single dot
    inline void Dot( Bitmap const *const pen,
                     INT32  const x,
                     INT32  const y,
                     bool   const black = true ) ;

    // For testing: dumps bitmap as ASCII
    void Dump() ;

    // Draws straight line from p1 to p2 using the given bitmap as a pen
    //
    // The line is black or white according as the argument 'black' is
    // true or not.
    //
    // If 'pen' is NIL, a single pixel pen of the appropriate color is
    // used.
    //
    // NOTE: x is the column and y the row.
    //
    void Line( Bitmap const *const pen,
               Ipoint const &p1,
               Ipoint const &p2,
               bool   const black = true ) ;

    static Bitmap *
    MakePen( INT32 const diam ) ;

private:
    // never copy bitmaps (because that creates multiple pointers to the
    // data and may result in dangling pointers)
    //
    Bitmap( Bitmap const & ) {
        Fatal( "(Bitmap): Cannot copy Bitmaps" ) ;
    }

    // never assign bitmaps (same reason as above)
    const Bitmap &
    operator=( Bitmap const &a ) {
        Fatal( "(Bitmap::=): Cannot assign Bitmaps" ) ;
        return a ;  // keep compiler happy
    }

} ;                                                            // Bitmap

typedef Bitmap< UINT8, 3 > BitmapX11 ;

//-----------------------------------------------------------------  Dot
//
// Draw a single dot
//
template< class T, INT32 aBits> inline void Bitmap<T, aBits>::
Dot( Bitmap const *const pen,
     INT32  const x,
     INT32  const y,
     bool   const black )
{
    if ( NIL == pen ) {
        BitOnOff( y, x, black ) ;
    } else {
        Paste( *pen, y, x, black ) ;
    }
}                                                                 // Dot

//--------------------------------------------------------------  Bitmap
//
template< class T, INT32 aBits> Bitmap<T, aBits>::
Bitmap( INT32 const wd, INT32 const ht ) {
    // sanity checks
    if ( wd <= 0 || ht <= 0 || wd > 20000 || ht > 20000 ) {
        Fatal( "(Bitmap): Bad bitmap dimensions" ) ;
    }

    // This can commented out after testing phase is over; it is useful
    // for catching errors like: Bitmap<UINT8,7>
    //
    static UINT8 const sz[ 8 ] = { 3, 4, 0, 5, 0, 0, 0, 6 } ;
    INT32 size = sizeof( T ) - 1 ;
    if ( size < 0 || size >= 8 || 0 == sz[ size ] ) {
        Fatal( "(Bitmap): Bad template arguments" ) ;
    }

    mWd = wd ;
    mHt = ht ;
    INT32 n = Size() ;
    mData = new T[ n ] ;
    memset( mData, 0, n * sizeof( T ) ) ;    // zero allocated memory
}                                                              // Bitmap

//---------------------------------------------------------------  Paste
//
// Paste the argument bitmap at position (row, col) which must fit in
// its entirety in current bitmap (otherwise a fatal error is generated)
//
// Destination bits corresponding to 1 bits of source are set if 'black'
// is true, cleared otherwise
//
// Note that the bits are ordered from LSB to MSB as the x-coordinate
// increases, which is somewhat unusual for bitmaps
//
template< class T, INT32 aBits> void Bitmap<T, aBits>::
Paste( Bitmap const &src,
       INT32  const row,
       INT32  const col,
       bool   const black )
{
    // nFB -- no. of unaffected low order bits in first destination cell
    //        of each row
    // nLB -- no. of affected (potentially) low order bits in last
    //        destination cell of each row (mod alignSize)
    // nFull -- no. of full (unfragmented) cells in a destination row
    //
    INT32
        sWd      = src.mWd,
        sHt      = src.mHt,
        dRowSize = RowSize(),
        nFB      = BitPos( col ),
        nLB      = BitPos( col + sWd ),
        nFull ;

    // sanity checks
    if ( row < 0 || col < 0
         || (row + sHt > mHt) || (col + sWd > mWd) ) {
        cerr << "(Paste): Bad bitmap dimensions" << endl ;
        exit( 1 ) ;
    }

    // sp     -- source pointer
    // dBegin -- first cell of each destination row
    // dEnd   -- end of destination area
    // dFull  -- end of full (unfragmented) destination cells in a row
    //
    T  *sp     = src.mData,
       *dBegin = Cellp( row, col ),
       *dEnd   = dBegin + (dRowSize * sHt),
       *dFull, *dp, cell, cell1, mask ;

    // check if more than 1 destination cell is affected in each row
    //
    if ( sWd <= alignSize - nFB ) {
        // Only 1 destination unit affected in each row
        mask = fullMask >> (alignSize - sWd) ;
        sp = src.mData ;

        for ( dp = dBegin ; dp < dEnd ; dp += dRowSize ) {
            cell = (mask & *sp++) << nFB ;
            if ( black ) {
                *dp |=  cell ;
            } else {
                *dp &=  ~cell ;
            }
        }
        return ;
    }

    mask = fullMask >> (alignSize - nLB) ;

    // Two or more destination cells affected; check for initial
    // fragmentation
    //
    if ( 0 == nFB ) {        // no initial cell fragmentation

        // handle unfragmented cells first
        nFull = Cell( sWd - nLB ) ;

        for ( ; dBegin < dEnd ; dBegin += dRowSize ) {
            dFull = dBegin + nFull ;
            for ( dp = dBegin ; dp < dFull ; ) {
                if ( black ) {
                    *dp++ |=  *sp++ ;
                } else {
                    *dp++ &=  ~*sp++ ;
                }
            }
            // now handle last fragmented cell of each row, if any
            if ( nLB ) {
                cell = mask & *sp++ ;
                if ( black ) {
                    *dp++ |=  cell ;
                } else {
                    *dp++ &=  ~cell ;
                }
            }
        }
        return ;
    }

    // Here, we have initial fragmentation and 2 or more destination
    // cells are affected.
    //
    INT32 rshift = alignSize - nFB ;
    nFull  = Cell( sWd - rshift - nLB ) ;

    for ( ; dBegin < dEnd ; dBegin += dRowSize ) {

        // first destination cell
        dp = dBegin ;
        cell = *sp++ ;
        if ( black ) {
            *dp++ |=  cell << nFB ;
        } else {
            *dp++ &=  ~(cell << nFB) ;
        }
        cell >>= rshift ;

        // next, unfragmented destination cells
        dFull = dp + nFull ;
        for ( ; dp < dFull ; cell = cell1 >> rshift ) {
            cell1 = *sp++ ;
            cell |= (cell1 << nFB) ;
            if ( black ) {
                *dp++ |=  cell ;
            } else {
                *dp++ &=  ~cell ;
            }
        }

        // finally, last destination cell (if any)
        if ( nLB ) {
            // if the last destination cell has a contribution from the
            // next source cell, get it
            //
            if ( nLB > nFB ) {
                cell |=  (*sp++ << nFB) ;
                cell &=  mask ;
            }

            if ( black ) {
                *dp |=  cell ;
            } else {
                *dp &=  ~cell ;
            }
        }
    }        // for
}                                                               // Paste

//---------------------------------------------------------------  Line8
//
// Draws straight line in one of 8 principal directions of starting
// at (x,y) of given length in given direction using the given pen
// bitmap as a pen.
//
// The line is black or white according as the argument 'black' is true
// or not.
//
// If 'pen' is NIL, a single pixel pen of the appropriate color is used.
//
// NOTE: -- x is the column and y the row.
//       -- Caller MUST ensure that the pen is entirely within the
//          bitmap (no clipping is allowed).
//
template< class T, INT32 aBits> void Bitmap<T, aBits>::
Line8( Bitmap const *pen,
       Ipoint const &pt,
       INT32  const aLen,
       EDir8  const dir,
       bool   const black )
{
    INT32 x = pt.x, y = pt.y, len = aLen ;

    if ( 0 == len ) {
        Fatal( "(Line8): len is zero" ) ;
    }

    if ( NIL == pen ) {    // single pixel pen
        switch ( dir ) {
        case eNorth:
            for ( ; len-- ; y++ ) {
                BitOnOff( y, x, black ) ;
            }
            break ;
        case eSouth:
            for ( ; len-- ; --y ) {
                BitOnOff( y, x, black ) ;
            }
            break ;
        case eEast:
            for ( ; len-- ; x++ ) {
                BitOnOff( y, x, black ) ;
            }
            break ;
        case eWest:
            for ( ; len-- ; --x ) {
                BitOnOff( y, x, black ) ;
            }
            break ;
        case eNorthEast:
            for ( ; len-- ; x++, y++ ) {
                BitOnOff( y, x, black ) ;
            }
            break ;
        case eSouthEast:
            for ( ; len-- ; x++, --y ) {
                BitOnOff( y, x, black ) ;
            }
            break ;
        case eNorthWest:
            for ( ; len-- ; --x, y++ ) {
                BitOnOff( y, x, black ) ;
            }
            break ;
        case eSouthWest:
            for ( ; len-- ; --x, --y ) {
                BitOnOff( y, x, black ) ;
            }
            break ;
        default:
            Fatal( "(Line8): Bad dir" ) ;
        }

    } else {    // bitmap pen

        // move to pen origin
        x -= (pen->mWd - 1) >> 1 ;
        y -= (pen->mHt - 1) >> 1 ;
        switch ( dir ) {
        case eNorth:
            for ( ; len-- ; y++ ) {
                Paste( *pen, y, x, black ) ;
            }
            break ;
        case eSouth:
            for ( ; len-- ; --y ) {
                Paste( *pen, y, x, black ) ;
            }
            break ;
        case eEast:
            for ( ; len-- ; x++ ) {
                Paste( *pen, y, x, black ) ;
            }
            break ;
        case eWest:
            for ( ; len-- ; --x ) {
                Paste( *pen, y, x, black ) ;
            }
            break ;
        case eNorthEast:
            for ( ; len-- ; x++, y++ ) {
                Paste( *pen, y, x, black ) ;
            }
            break ;
        case eSouthEast:
            for ( ; len-- ; x++, --y ) {
                Paste( *pen, y, x, black ) ;
            }
            break ;
        case eNorthWest:
            for ( ; len-- ; --x, y++ ) {
                Paste( *pen, y, x, black ) ;
            }
            break ;
        case eSouthWest:
            for ( ; len-- ; --x, --y ) {
                Paste( *pen, y, x, black ) ;
            }
            break ;
        default:
            Fatal( "(Line8): Bad dir" ) ;
        }
    }
}                                                               // Line8

//----------------------------------------------------------------  Line
//
// Draws straight line from p1 to p2 using the given bitmap as a pen
//
// The line is black or white according as the argument 'black' is
// true or not.
//
// If 'pen' is NIL, a single pixel pen of the appropriate color is
// used.
//
// NOTE: x is the column and y the row.
//
template< class T, INT32 aBits> void Bitmap<T, aBits>::
Line( Bitmap const *pen,
      Ipoint const &p1,
      Ipoint const &p2,
      bool   const black )
{
    INT32 x  = p1.x,    y  = p1.y,
          x2 = p2.x,    y2 = p2.y,
          dx = x2 - x,  dy = y2 - y ;

    // Eliminate special cases
    if ( 0 == dy ) {      // horizontal
        if ( dx >= 0 ) {
            Line8( pen, p1, dx + 1, eEast, black ) ;  return ;
        }
        Line8( pen, p1, (-dx + 1), eWest, black ) ;  return ;
    }
    if ( 0 == dx ) {      // vertical
        if ( dy >= 0 ) {
            Line8( pen, p1, dy + 1, eNorth, black ) ;  return ;
        }
        Line8( pen, p1, (-dy + 1), eSouth, black ) ;  return ;
    }
    if ( dx == dy ) {     // diagonal (NE,SW)
        if ( dx >= 0 ) {
            Line8( pen, p1, dx + 1, eNorthEast, black ) ;  return ;
        }
        Line8( pen, p1, (-dx + 1), eSouthWest, black ) ;  return ;
    }
    if ( -dx == dy ) {    // diagonal (SE,NW)
        if ( dx >= 0 ) {
            Line8( pen, p1, dx + 1, eSouthEast, black ) ;  return ;
        }
        Line8( pen, p1, (-dx + 1), eNorthWest, black ) ;  return ;
    }

    // Line is not along the 8 principal directions

    INT32  step ;         // row or column increment
    INT32  disc ;         // discriminant
    INT32  inc1, inc2 ;   // increments for disc

    step = (dy * dx < 0) ? -1 : 1 ;

    if ( dx < 0 )
        dx = -dx ;
    if ( dy < 0 )
        dy = -dy ;

    if (dy > dx) {
        inc1 = dx << 1 ;
        inc2 = (dx - dy) << 1 ;
        disc = inc1 - dy ;
        if ( y > y2 ) {
            for ( ; y >= y2 ; y-- ) {
                Dot( pen, x, y, black ) ;
                if ( disc < 0 ) {
                    disc += inc1 ;
                } else {
                    disc += inc2 ;   x -= step ;
                }
            }
        } else {
            for ( ; y <= y2 ; y++ ) {
                Dot( pen, x, y, black ) ;
                if ( disc < 0 ) {
                    disc += inc1 ;
                } else {
                    disc += inc2 ;   x += step ;
                }
            }
        }  // if

    } else {                   // dy <= dx

        inc1 = dy << 1 ;
        inc2 = (dy - dx) << 1 ;
        disc = inc1 - dx ;
        if ( x > x2 ) {
            for ( ; x >= x2 ; x-- ) {
                Dot( pen, x, y, black ) ;
                if ( disc < 0 ) {
                    disc += inc1 ;
                } else {
                    disc += inc2 ;   y -= step ;
                }
            }
        } else {
            for ( ; x <= x2 ; x++ ) {
                Dot( pen, x, y, black ) ;
                if ( disc < 0 ) {
                    disc += inc1 ;
                } else {
                    disc += inc2 ;   y += step ;
                }
            }
        }  // if
    }
}                                                                // Line

// ---------------------------------------------------------------  Dump
//
// For testing: dumps bitmap as ASCII
//
template< class T, INT32 aBits> void Bitmap<T, aBits>::
Dump()
{

    INT32  rdn, cdn,      // no. of digits in row/col no.
           rows = mHt,
           cols = mWd,
           rw   = 10,     // default width of row number field
           t1, t2 ;

    // find no. of digits for row/col coordinates
    for( rdn = 1, t1 = rows - 1 ; t1 > 9 ; t1 /= 10, rdn++ )
        ;
    for ( cdn = t2 = 1, t1 = cols - 1
        ; t1 > 9
        ; t1 /= 10, t2 *= 10, cdn++
        ) ;

    /* display column numbers */
    for ( ; cdn ; cdn--, t2 /= 10 ) {
        for ( t1 = rdn ; t1 ; t1-- )
            cout << ' ' ;
        for ( t1 = 0 ; t1 < cols ; t1++ )
            cout << (t1 / t2) % 10 ;
        cout << endl ;
    }

    switch ( rdn ) {
    case 1:  rw = 1 ; break ;
    case 2:  rw = 2 ; break ;
    case 3:  rw = 3 ; break ;
    case 4:  rw = 4 ; break ;
    default: break ;
    }

    // display each row with row no.
    T   *bp = mData,
        mask ;
    for ( t1 = 0 ; t1 < rows ; t1++ ) {
        cout << setw( rw ) << t1 ;
        mask = 1 ;
        for ( t2 = 0 ; t2 < cols ; t2++ ) {
            if ( mask & *bp )
                cout << '*' ;
            else
                cout << '.' ;
            mask <<= 1 ;
            if ( 0 == mask ) {
                mask = 1 ; bp++ ;
            }
        }
        // if cols is a multiple of 8, bp has been already incremented
        // inside the loop; otherwise, we increment here.
        //
        if ( 1 != mask ) {
           bp++ ;
        }
        cout << endl ;
    }
}                                                                // Dump

/*-------------------------------------------------------------  MakePen
 *
 * Creates pen of given diameter
 */
template< class T, INT32 aBits >
Bitmap<T, aBits> *Bitmap<T, aBits>::
MakePen( INT32 diam )
{
    // keep diameter in reasonable range
    if ( diam < 2 || diam > 150 ) {
        Fatal( "(MakePen): Bad pen diameter\n" ) ;
    }

    Bitmap *pen = new Bitmap( diam, diam ) ;

    // we reduce the radius slightly to produce lighter pens
    double
        r   = static_cast< double >( diam ) / 2.0,   // true radius
        rad = r - 0.4,                               // adjusted radius
        rad2 = rad * rad,
        cx = r, cy = r,                              // center
        y = cy - 0.5 ;
    INT32
        r1  = 0,                  // cur bottom row
        r2  = diam - 1,           // cur top row
        cnt = r2 >> 1,            // no. of iterations
        c1  = cnt,                // left extremity
        c2  = diam >> 1,          // right extremity
        len = (c2 - c1) + 1 ;     // length of line

    // left extremities of current bottom and top rows
    Ipoint pt1( c1, r1 ), pt2( c1, r2 ) ;
    for ( ; cnt-- ; ) {
        pen->Line8( NIL, pt1, len, eEast ) ;
        pen->Line8( NIL, pt2, len, eEast ) ;
        y -= 1.0 ;
        c2 = static_cast< INT32 >(
            0.5 + cx + sqrt( rad2 - y * y ) ) - 1 ;
        c1 = (diam - c2) - 1 ;
        pt1.x = pt2.x = c1 ;
        len = (c2 - c1) + 1 ;
        pt1.y++, pt2.y-- ;
    }

    // draw the middle line(s)
    if ( diam & 1 ) {
        pt1.x = 0 ; pt1.y = diam >> 1 ;
        pen->Line8( NIL, pt1, diam, eEast ) ;
    } else {
        pt1.x = 0 ; pt1.y = diam >> 1 ;
        pen->Line8( NIL, pt1, diam, eEast ) ;
        --pt1.y ;
        pen->Line8( NIL, pt1, diam, eEast ) ;
    }

    return pen ;
}                                                             // MakePen

#endif  // BITMAP_
