//======================================================================
//
// Debugging facilities
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.7 $
//
//======================================================================

// Need sscanf()
extern "C" {
#include <stdio.h>
}

#include "Debug.H"

// Need setw()
#ifndef __IOMANIP__
#include <iomanip>
#endif

using namespace std ;

// Need NIL
#ifndef GLOBALDEFS_
#include "globaldefs.H"
#endif

//====================  DEFINES  =======================================

// Argumnet must be a single lvalue (cannot use ++ since that is not
// defined for enumerations!)
//
#define DBGINC( X ) X = (dbgEnum)(X + 1)

//====================  GLOBALS  =======================================

// Keep this in alphabetic order and in sync with dbgEnum values
// in Debug.H
//
char *
Debug::mFlagNames[ Debug::eLast ] = {
   "Conic", "StrokeFont",
} ;

//====================  FUNCTIONS  =====================================

//-----------------------------------------------------------  BasicInit
//
// One-time initialization of module
//
void
Debug::BasicInit( )
{
    for ( int i = 0 ; i < eLast ; i++ ) {
        mFlags[ i ]     = false ;
        mFileNames[ i ] = NIL ;
        mFileIndex[ i ] = eLast ;
    }
    mNextSlot = (dbgEnum)0 ;
    mInitFile = NIL ;
}                                                           // BasicInit

//----------------------------------------------------------------  Init
//
// One-time initialization of module from data file
//
void
Debug::Init( char const *const f )
{
    int len ;  // scratch
    if ( 0 == f || 0 == (len = strlen( f )) ) {
        cerr << "ERROR(Debug::Init)-- Bad argument" << endl ; abort() ;
    }
    mInitFile = new char[ len + 1 ] ;
    strcpy( mInitFile, f ) ;
    ifstream ifs( f ) ;
    if ( ifs.fail() ) {
        cerr << "ERROR(Debug::Init)-- Bad file '" << f << "'" << endl ;
        abort() ;
    }

    int  lnum = 1 ;             // line number
    char line[ maxLine ] ;      // input line
    char curFlag[ maxLine ] ;  // name of flag ...
    int  flagValue ;            // ... and its value (0 or 1)
    char file[ maxLine ] ;      // name of output file
    bool seen[ eLast ] ;        // remember which flags were seen
    int  i1, i2, k ;

    for ( int i = 0 ; i < eLast ; i++ ) {
        seen[ i ] = false ;
    }

    // Read each line
    ifs.getline( line, maxLine ) ;
    for ( ; ; lnum++, ifs.getline( line, maxLine ) ) {
        if ( ifs.eof() )
            break ;
        if ( ifs.fail() ) {
            cerr << "ERROR(Debug::Init)-- getline() failed" << endl ;
            abort() ;
        }

        // ignore comment lines and empty lines
        if ( '#' == line[ 0 ] || 0 == line[ 0 ] )
            continue ;
        if ( 0 == (k = sscanf( line, "%s %d %s", curFlag, &flagValue,
                          file )) )
            continue ;

        // Check if at least flag name and value are present
        if ( 1 == k ) {
            cerr << "ERROR(Debug::Init)-- Missing flag value on line "
                 << lnum << endl ;
            abort() ;
        }

        // Check if name is valid and is not duplicate
        for ( i1 = 0 ; i1 < eLast ; i1++ ) {
            if ( 0 == strcmp( curFlag, mFlagNames[ i1 ] ) )
                break ;  // found it
        }
        if ( eLast == i1 ) {
            cerr << "ERROR(Debug::Init)-- Bad flag name on line "
                 << lnum << endl ;
            abort() ;
        }
        if ( seen[ i1 ] ) {
            cerr << "ERROR(Debug::Init)-- Duplicate flag name on line "
                 << lnum << endl ;
            abort() ;
        }
        seen[ i1 ] = true ;

        // Check if value is valid
        if ( 0 != flagValue && 1 != flagValue ) {
            cerr << "ERROR(Debug::Init)-- Bad flag value " << flagValue
                 << " on line " << lnum << endl ;
            abort() ;
        }
        mFlags[ i1 ] = flagValue ? true : false ;

        // If a flag is false, the filename is ignored
        if ( flagValue && 3 == k ) {
            // Got filename, so check if it has been seen before ; if
            // not, insert in table and open it
            //
            for ( i2 = 0 ; i2 < mNextSlot ; i2++ ) {
                if ( 0 == strcmp( file, mFileNames[ i2 ] ) )
                    break ;
            }
            mFileIndex[ i1 ] = (dbgEnum)i2 ;
            if ( mNextSlot == i2 ) {    // new name
                if ( mNextSlot == eLast ) {    // should never happen
                    cerr << "ERROR(Debug::Init)-- Too many files (line "
                         << lnum << ')' << endl ;
                    abort() ;
                }
                mFileNames[ mNextSlot ]
                    = new char[ strlen( file ) + 1 ] ;
                DBGINC( mNextSlot ) ;
                strcpy( mFileNames[ i2 ], file ) ;
                mOfile[ i2 ].open( mFileNames[ i2 ] ) ;
                if ( mOfile[ i2 ].fail() ) {    // should never happen
                    cerr << "ERROR(Debug::Init)-- Failed to open '"
                         << mFileNames[ i2 ] << "' for output" << endl ;
                    abort() ;
                }
            }
        }
    }  // for
}                                                             /* Init */

//--------------------------------------------------------------  ~Debug
//
// Destructor
//
Debug::~Debug( )
{
    int i ;

    if ( NIL == mInitFile ) {
        return ;
    }
    delete [] mInitFile ;

    for ( i = 0 ; i < mNextSlot ; i++ ) {
        delete [] mFileNames[ i ] ;
    }
}                                                           /* ~Debug */

//---------------------------------------------------------------  Debug
//
// Constructor
//
Debug::Debug( char const *const f )
{
    BasicInit( ) ;
    Init( f ) ;
}                                                            /* Debug */

//----------------------------------------------------------------  Dump
//
// For debugging Debug module
//
void
Debug::Dump( )
{
    int i, j,
        max = 12 ;  // minimum field width for flag names

    // Find the maximum length of a flag name
    for ( i = 0 ; i < eLast ; i++ ) {
        j = strlen( mFlagNames[ i ] ) ;
        if ( j > max )
            max = j ;
    }

    cout << "mInitFile = " << mInitFile << endl ;
    cout << "mNextSlot = " << mNextSlot << endl ;
    cout << setw( max ) << "Flag names" << setw( 8 ) << "Values"
         << setw( 9 ) << "Indices" << "  " << setw( 5 ) << "Files"
         << endl ;
    cout << setw( max ) << "----------" << setw( 8 ) << "------"
         << setw( 9 ) << "-------" << "  " << setw( 5 ) << "-----"
         << endl ;
    for ( i = 0 ; i < eLast ; i++ ) {
        cout << setw( max ) << mFlagNames[ i ]
             << setw( 8 )   << mFlags[ i ]
             << setw( 9 )   << mFileIndex[ i ]
             << "  "
             << (mFlags[ i ] ? (eLast == mFileIndex[ i ] ? "stdout"
                               : mFileNames[ mFileIndex[ i ] ])
                 : "")
             << endl ;
    }
}                                                             /* Dump */
