//======================================================================
//
// Geometry of conics
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.9 $
//
//======================================================================

#ifdef DEBUG
#ifndef DEBUG_
#include "Debug.H"
#endif
#endif

#ifndef GEOMETRY_
#include "Geometry.H"
#endif

//====================  DEFINES  =======================================

#ifdef SUNOS
#define LOG2( X ) log2( X )
#else
#define LOG2( X ) (log( X ) / log( 2.0 ))
#endif

//====================  FUNCTIONS  =====================================

// The following routines compute lots of things related to an ABC-conic
// passing through A and B with C the apex of the defining triangle
// (which will be a bounding triangle iff the bulge is nonnegative).
//
// 'bulge' must be in the range (-1:inf).
//   <= -1.0  ==> INVALID VALUE
//   < 0      ==> elliptic arc _OUTSIDE_ the triangle
//   0        ==> straight line,
//   (0:1)    ==> elliptic arc
//   1        ==> parabolic arc
//   > 1      ==> hyperbolic arc
//
// 'unitBulge' is true iff abs(bulge-1.0) < tiny. In this case, the
// equations are simpler.

// Types with no data; used simply to instantiate different variants of
// template functions FindExtrema() below.

// for finding x-extrema
struct ConicX {
    COORD get1( Point const &p ) const { return p.x ; }
    COORD get2( Point const &p ) const { return p.y ; }
    COORD operator()( ConicData const &data, COORD const t ) const {
        return (data.ft2 * t + data.ft1) * t + data.ft0 ;
    }
    void AdjustBbox( Bbox &bbox, COORD const f ) { bbox.AdjustX( f ) ; }
    char const *operator()( ) const { return "x" ; } // for debug output
};

// for finding y-extrema
struct ConicY {
    COORD get1( Point const &p ) const { return p.y ; }
    COORD get2( Point const &p ) const { return p.x ; }
    COORD operator()( const ConicData &data, COORD const t ) const {
        return (data.gt2 * t + data.gt1) * t + data.gt0 ;
    }
    void AdjustBbox( Bbox &bbox, COORD const f ) { bbox.AdjustY( f ) ; }
    char const *operator()( ) const { return "y" ; } // for debug output
};

//---------------------------------------------------------  FindExtrema
//
// Finds the extrema of a conic in the x- or y-direction (according as
// first argument is ConicX or ConicY. The extrema are folded in
// to bbox. Assumes coefficients have already been computed
// and isGood is false (for a good conic, the bbox is simply the bbox of
// AB and there is nothing to do).
//
// Note that there may be no extrema in the desired direction, 1
// extremum, or, if the bulge is negative, 2 extrema.
//
template< class T > static void
FindExtrema( T            xory,
             Point const  &pA,
             Point const  &pB,
             Point const  &pC,
             COORD        bulge,
             ConicData    &data,
             Bbox         &bbox )
{
#ifdef DEBUG
#define DBGFLAG dbg.Flag( Debug::eConic )
#define DBGFILE dbg.File( Debug::eConic )
    if ( DBGFLAG ) {
        DBGFILE << "Into FindExtrema" << "( " << xory() << " )"
                << endl;
    }
#endif

    COORD
        zA  = xory.get1( pA ),
        zB  = xory.get1( pB ),
        zC  = xory.get1( pC ),
        zBA = zB - zA,
        zAC = zA - zC,
        aa, bb, cc,             // coefficients of quadratic in t
        den, dis,               // denominator and discriminant
        t, f ;                  // temporaries

    // Check if quadratic is really linear: first case
    if ( data.unitBulge ) {
        // aa is 0 and not be used; single solution
        bb = zBA + (2.0 * zAC) ;
        if ( ApproxEq( bb, 0.0 ) )
            Fatal( "(FindExtrema): Bad conic" ) ;
        t = zAC / bb ;

        // No solutions if t is outside (0,1)
        if ( t < 0.0 || 1.0 < t ) {
#ifdef DEBUG
            if ( DBGFLAG ) {
                DBGFILE << "  [No solution]" << endl ;
            }
#endif
            return ;
        }

        f = xory( data, t ) ;    // denominator is 1
        xory.AdjustBbox( bbox, f ) ;

#ifdef DEBUG
        if ( DBGFLAG ) {
            DBGFILE << "  [Single solution]: t = " << t << ", "
                    << xory() << " = " << f
                    << endl ;
        }
#endif
        return ;
    }

#define SET \
    f = xory( data, t ) / ((t - 1.0) * data.zt2 * t + 1.0) ; \
    xory.AdjustBbox( bbox, f )

    // Check if quadratic is really linear: second case
    if ( ApproxEq( zBA, 0.0 ) ) {
        // aa is 0 and not be used; single solution
        bb = zBA + (2.0 * bulge * zAC) ;
        if ( ApproxEq( bb, 0.0 ) )
            Fatal( "(FindExtrema): Bad conic" ) ;
        t = bulge * zAC / bb ;

        // No solutions if t is outside (0,1)
        if ( t < 0.0 || 1.0 < t ) {
#ifdef DEBUG
            if ( DBGFLAG ) {
                DBGFILE << "  [No solution]" << endl ;
            }
#endif
            return ;
        }

        SET ;

#ifdef DEBUG
        if ( DBGFLAG ) {
            DBGFILE << "  [Single solution]: t = " << t << ", "
                    << xory() << " = " << f
                    << endl ;
        }
#endif
        return ;
    }

    aa  = zBA * (bulge - 1.0) ;
    bb  = zBA + (2.0 * bulge * zAC) ;
    cc  = -bulge * zAC ;
    den = 2.0 * aa ;
    dis = bb * bb - 4.0 * aa * cc ;

    // Should never happen but ...
    if ( ApproxEq( aa, 0.0 ) ) {
        Fatal( "(FindExtrema): Degenerate conic" ) ;
    }

    // Check if quadratic has coincident solutions
    if ( ApproxEq( dis, 0.0 ) ) {
        t = -bb / den ;

        // No solutions if t is outside (0,1)
        if ( t < 0.0 || 1.0 < t ) {
#ifdef DEBUG
            if ( DBGFLAG ) {
                DBGFILE << "  [No solution]" << endl ;
            }
#endif
            return ;
        }

        SET ;

#ifdef DEBUG
        if ( DBGFLAG ) {
            DBGFILE << "  [Coincident solutions]: t = " << t << ", "
                    << xory() << " = " << f
                    << endl ;
        }
#endif
        return ;
    }

    // Should never happen, but ...
    if ( dis < 0.0 ) {
        Fatal( "FindExtrema: No extrema found" ) ;
    }

    // Note that there could be 2 solutions if bulge is negative
    dis = sqrt( dis ) ;
    t = (-bb + dis) / den ;
    if ( 0.0 < t && t < 1.0 ) {

        SET ;

#ifdef DEBUG
        if ( DBGFLAG ) {
            DBGFILE << "  [first solution]: t = " << t << ", "
                    << xory() << " = " << f
                    << endl ;
        }
#endif
    }

    t = (-bb - dis) / den ;
    if ( 0.0 < t && t < 1.0 ) {

        SET ;

#ifdef DEBUG
        if ( DBGFLAG ) {
            DBGFILE << "  [second solution]: t = " << t << ", "
                    << xory() << " = " << f
                    << endl ;
        }
#endif
    }
#undef SET
}                                                         // FindExtrema

// Now instantiate FindExtrema
template void
FindExtrema( ConicX xory,
             Point const &pA,
             Point const &pB,
             Point const &pC,
             COORD bulge,
             ConicData &data,
             Bbox &bbox ) ;

template void
FindExtrema( ConicY xory,
             Point const &pA,
             Point const &pB,
             Point const &pC,
             COORD bulge,
             ConicData &data,
             Bbox &bbox ) ;

//---------------------------------------------------------------  Coeff
//
// Compute and return coefficients of parametric equation. No sanity
// checks here, so make sure the data is valid
//
static void
Coeff( ConicData   &data,
       Point const &pA,
       Point const &pB,
       Point const &pC,
       COORD       bulge )
{
    COORD xA = pA.x, yA = pA.y,
          xB = pB.x, yB = pB.y,
          xC = pC.x, yC = pC.y ;

    data.zt0 = 1.0 ;
    if ( data.unitBulge ) {
        data.zt2 = 0.0 ;    // not really needed but ...
        data.zt1 = 0.0 ;    // ditto

        data.ft2 = xA + xB - 2.0 * xC ;
        data.ft1 = 2.0 * (xC - xA) ;
        data.ft0 = xA ;

        data.gt2 = yA + yB - 2.0 * yC ;
        data.gt1 = 2.0 * (yC - yA) ;
        data.gt0 = yA ;
    } else {
        data.zt2 = 2.0 * (1.0 - bulge) ;
        data.zt1 = -data.zt2 ;

        data.ft2 = xA + xB - 2.0 * bulge * xC ;
        data.ft1 = 2.0 * (bulge * xC - xA) ;
        data.ft0 = xA ;

        data.gt2 = yA + yB - 2.0 * bulge * yC ;
        data.gt1 = 2.0 * (bulge * yC - yA) ;
        data.gt0 = yA ;
    }

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE
            << "Leaving Coeff(), bulge = " << bulge << '\n'
            << "    pts = " << pA << pB << pC << '\n'
            << "    zt2 = "   << data.zt2 << ", zt1 = " << data.zt1
            << ", zt0 = " << data.zt0 << '\n'
            << "    ft2 = "   << data.ft2 << ", ft1 = " << data.ft1
            << ", ft0 = " << data.ft0 << '\n'
            << "    gt2 = "   << data.gt2 << ", gt1 = " << data.gt1
            << ", gt0 = " << data.gt0
            << endl ;
    }
#endif

}                                                               // Coeff

//------------------------------------------------------------- NumFlats
//
// Determine maximum dimension of bbox and use this to compute the
// number of flats needed to render a conic
//
static INT32
NumFlats( const Bbox &bb )
{
    static const int
        Ntable = 10,
        num[ Ntable ] = { 0, 2, 2, 2, 2, 3, 3, 3, 4, 4 } ;

    COORD wd = bb.urx - bb.llx,
          ht = bb.ury - bb.lly,
          fbig ;
    INT32 big, small ;

    if ( wd < ht ) {
        fbig  = ht ;
        small = Round( wd ) ;
    } else {
        fbig  = wd ;
        small = Round( ht ) ;
    }
    big = Round( fbig ) ;

#ifdef DEBUG
    if ( big < 1 || small < 1 ) {
        Fatal( "(NumFlats): flat conic" ) ;
    }
#endif

    // other possibilities:
    //   int( 0.5 + sqrt(big) )  -- faster but slightly coarser curves
    //   int( 0.5 + pow(big, 0.53) ) -- reasonable
    //
    big = (big < Ntable) ? num[ big ]
        : static_cast<INT32>( 0.5 + sqrt( fbig ) + LOG2( fbig ) ) ;
    return 6 + (big < small ? big : small) ;
}                                                            // NumFlats

//-----------------------------------------------------------  DoFlatten
//
// Returns the flattened segments including the first endpoint BUT NOT
// THE LAST. Thus, the number of points inserted at 'flats' will be
// EXACTLY nFlats.
//
static void
DoFlatten( ConicData    &data,
           Point const  &pA,
           Point        *flats,
           INT32        nFlats )
{
    *flats = pA ;  // first endpoint
    if ( 1 == nFlats ) {
        return ;
    }

    register COORD const eps = 1.0 / nFlats ;
    register Point *p = flats + 1 ;
    register Point *q = flats + nFlats ;

#ifdef DEBUG
    if ( dbg.Flag( Debug::eConicFlatten ) ) {
        dbg.File( Debug::eConicFlatten ) << "Flatten()"
            << ", nFlats = " << nFlats
            << ", eps = " << eps << endl ;
    }
#endif

    // The coordinates are given by x(t) = f(t)/z(t), y(t) = g(t)/z(t)
    // Each of f, g, and z is computed by forward differencing.
    //
    COORD const
        eps2 = eps * eps,
        ddf  = 2.0 * data.ft2 * eps2,             // level-2 increments
        ddg  = 2.0 * data.gt2 * eps2 ;
    COORD
        df  = eps * (data.ft2 * eps + data.ft1),  // level-1 increments
        dg  = eps * (data.gt2 * eps + data.gt1),
        f   = pA.x,                               // f(0)
        g   = pA.y ;                              // g(0)

    // Since nFlats > 1, must have p < q here
    if ( data.unitBulge ) {  // No need to compute denominator
        for ( ; ; df += ddf, dg += ddg ) {
            f   += df ;  g    += dg ;
            p->x = f ;   p->y =  g ;
            if ( ++p >= q ) {
                break ;
            }
        }
    } else {
        COORD const
            ddz = 2.0 * data.zt2 * eps2 ;   // level-2 z-increment
        COORD                               // level-1 z-increment
            dz  = eps * (data.zt2 * eps + data.zt1),
            z   = 1.0 ;                     // z(0)

        for ( ; ; df += ddf, dg += ddg, dz += ddz ) {
            f += df ;  g += dg ;  z += dz ;
            p->x = f / z ;
            p->y = g / z ;
            if ( ++p >= q ) {
                break ;
            }
        }
    }

#ifdef DEBUG
    if ( dbg.Flag( Debug::eConicFlatten ) ) {
        for ( p = flats, q = flats + nFlats ; p < q ; p++ ) {
            dbg.File( Debug::eConicFlatten ) << *p << endl ;
        }
    }
#endif
}                                                           // DoFlatten

//---------------------------------------------------------------  Conic
//
// Returns coefficients of parametric equation of conic, exact bbox etc.
//
// bbox should contain bbox of AB when this routine is called; it will
// be further refined to the exact bbox.
//
// If the conic is completely flat, isFlat is set to true and
// coefficients are not computed (bbox is not modified).
//
void Geometry::
Conic(  Point const &pA,
        Point const &pB,
        Point const &pC,
        COORD const bulge,
        Bbox        &bbox,
        ConicData   &data )
{
#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "Into Geometry::Conic()" << endl ;
    }
#endif

    // Need the scope resolution since Geometry::Fatal() exists
    if ( bulge <= -1.0 ) {
        ::Fatal( "(Geometry::Conic): Bad bulge" ) ;
    }
    if ( bulge < bulgeMin || bulgeMax < bulge ) {
        ::Fatal( "(Geometry::Conic): unreasonable bulge" ) ;
    }

    // If bulge is unity, faster code is possible
    //
    COORD u, b ;
    if ( ApproxEq( bulge, 1.0 ) ) {
        data.unitBulge = true ;
        b = 1.0 ;
        u = 0.5 ;
    } else {
        data.unitBulge = false ;
        b = bulge ;
        u = b / (1.0 + b) ;
    }

    if ( b >= 0.0 ) {
        data.isGoodX = bbox.InsideX( pC.x ) ;
        data.isGoodY = bbox.InsideY( pC.y ) ;
    } else {
        data.isGoodX = data.isGoodY = false ;
    }

    Point pM = Point( (pA + pB) / 2.0 ) ;         // chord midpoint
    Point pP = Point( pM + u * (pC - pM) ) ;      // shoulder point

    // If conic completely flat, nothing else needs to be computed.
    // Note that in this case, we treat the conic as simply a straight
    // line from A to B. This means that in cases where A, B, and C are
    // collinear and B is not within the line segment AB, the conic
    // will not show the expected "spike" outside AB.
    //
    if ( Dist( pA, pB, pP ) <= G::tolFlatConic ) {
        data.isFlat = true ;
        data.isGoodX = data.isGoodY = true ;
        return ;
    }
    data.isFlat = false ;

    // Get coefficients of parametric equation (needed to compute exact
    // bbox.)
    Coeff( data, pA, pB, pC, b ) ;

    // Find exact bbox
    if ( ! data.isGoodX )     // x-extrema
        FindExtrema( ConicX(), pA, pB, pC, b, data, bbox ) ;
    if ( ! data.isGoodY )     // y-extrema
        FindExtrema( ConicY(), pA, pB, pC, b, data, bbox ) ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "Computed bbox = " << bbox << endl ;
    }
#endif

    return ;
}                                                               // Conic

//-------------------------------------------------------------  Flatten
//
// Returns flattened polyline for a conic in data.flats
//
// 'bbox' should be the exact (tight) bbox of the conic and the
// various members of 'data' should be valid.
//
// No special treatment for flat conics, so they should be eliminated
// by caller.
//
void Geometry::
Flatten( Point const &pA,
         Point const &pB,
         Point const &pC,
         COORD const bulge,
         Bbox const  &bbox,
         ConicData   &data )
{
    INT32 nFlats ;
    Point *flatPts ;

    COORD u = data.unitBulge ? 0.5 : (bulge / (1.0 + bulge)) ;

    // No need to subdivide if bulge is positive
    if ( bulge >= 0.0 ) {
        nFlats = 1 + NumFlats( bbox ) ;
        // we can reduce the number of flats a little if the curve is
        // not parabolic
        if ( !data.unitBulge ) {
            nFlats
                = Round( nFlats * (u > 0.5 ? (1.5 - u) : (0.5 + u)) ) ;
        }
        flatPts  = new Point[ nFlats + 1 ] ;

        DoFlatten( data, pA, flatPts, nFlats ) ;
        // terminal extremity
        flatPts[ nFlats ] = pB ;
        data.flats = new PolyLine( flatPts, nFlats + 1, true ) ;
#ifdef DEBUG
        if ( DBGFLAG ) {
            DBGFILE << "Leaving Geometry::Flatten (no subdivision)"
                    << endl ;
        }
#endif
        return ;
    }

    // Negative bulge: need to subdivide once.
    // Find apexes of AP-piece and PB-piece
    Point pM = Point( (pA + pB) / 2.0 ) ;         // chord midpoint
    Point pP = Point( pM + u * (pC - pM) ) ;      // shoulder point
    Point pD = Point( pA + u * (pC - pA) ) ;
    Point pE = Point( pB + u * (pC - pB) ) ;

    // Compute approximate bbox for each piece. NOTE: If you want to
    // call FindExtrema() to find tighter bboxes, you should eliminate
    // the two calls to Adjust() below. Otherwise, the bbox will be too
    // generous and FindExtrema() will do nothing.
    //
    Bbox bx1( pA, pP ), bx2( pP, pB ) ;
    bx1.Adjust( pD ) ;
    bx2.Adjust( pE ) ;

    ConicData data1 ;
    COORD bulge1 = sqrt( (1.0 + bulge) / 2.0 ) ;
    data1.unitBulge = false ;

    INT32 nFlats1 = NumFlats( bx1 ),
          nFlats2 = NumFlats( bx2 ),
          nPts    = 1 + nFlats1 + nFlats2 ;

    flatPts  = new Point[ nPts ] ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "Negative bulge (first)"
                << ", nFlats1 = " << nFlats1
                << ", nFlats2 = " << nFlats2
                << ", nPts = " << nPts << endl ;
    }
#endif

    // first piece
    Coeff( data1, pA, pP, pD, bulge1 ) ;
    DoFlatten( data1, pA, flatPts, nFlats1 ) ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "Negative bulge (second)" << endl ;
    }
#endif

    // second piece
    Coeff( data1, pP, pB, pE, bulge1 ) ;
    DoFlatten( data1, pP, flatPts + nFlats1, nFlats2 ) ;

    flatPts[ nPts - 1 ] = pB ;  // last endpoint
    data.flats = new PolyLine( flatPts, nPts, true ) ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "Leaving Geometry::Flatten (subdivided)" << endl ;
    }
#undef DBGFLAG
#undef DBGFILE
#endif
    return ;
}                                                             // Flatten
