//======================================================================
//
// Basic geometry routines
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.8 $
//
//======================================================================

#ifdef DEBUG
#ifndef DEBUG_
#include "Debug.H"
#endif
#endif

#ifndef GEOMETRY_
#include "Geometry.H"
#endif

//====================  FUNCTIONS  =====================================

//----------------------------------------------------------  operator<<
//
// Output bbox info
//
ostream&
operator<<( ostream &os, Bbox const &bbox )
{
    return os << "[ "
              << bbox.llx << ", " << bbox.lly << ", "
              << bbox.urx << ", " << bbox.ury
              << " ]" ;
}                                                          // operator<<

//---------------------------------------------------------------  Fatal
//
void Geometry::
Fatal( char  const *const msg,
       Point const &pA,
       Point const &pB )
{
    cerr << msg << " A=" << pA << ", B=" << pB << endl ;
    exit( 1 ) ;
}                                                               // Fatal

//------------------------------------------------------------  TestSide
//
// Returns a discriminant which is >, =, or < 0 according as the point
// 'p' is on one side of the line AB, on AB or the other side. A little
// less expensive than 'dist()' since no division or sqrt() is needed.
// The order of A and B does not matter.
//
COORD Geometry::
TestSide( Point const &pA,
          Point const &pB,
          Point const &p )
{
    Point pAB = pB ;
    pAB -= pA ;

#ifdef DEBUG
    COORD lenSq = pAB.LenSq() ;
    if ( ApproxZero( lenSq ) ) {
        Fatal( "(Geometry::TestSide): Degenerate line", pA, pB ) ;
    }
#endif

    COORD f = pAB % (p - pA) ;

    // order A and B lexicographically so the result does not depend
    // on the order of arguments
    //
    return ((pA.y > pB.y) || (pA.y == pB.y && pA.x > pB.x)) ? -f : f ;

}                                                            // TestSide

//----------------------------------------------------------------  Dist
//
// Returns the UNSIGNED distance of the point 'p' to the line AB.
// A little more expensive than 'testSide()' since it involves an extra
// division and sqrt().
//
COORD Geometry::
Dist( Point const &pA,
      Point const &pB,
      Point const &p )
{
    Point pAB = pB ;
    pAB -= pA ;
    COORD lenSq = pAB.LenSq() ;

#ifdef DEBUG
    if ( ApproxZero( lenSq ) ) {
        Fatal( "(Geometry::dist): Degenerate line", pA, pB ) ;
    }
#endif

    // The distance is given by abs( (pB-pA)x(p-pA)/norm(pB-pA) )
    COORD d = fabs( pAB % (p - pA) ) ;
    return (d == 0.0) ? 0.0 : (d / sqrt( lenSq )) ;
}                                                                // Dist

//-----------------------------------------------------------  Intersect
//
// Returns true if line segments intersect and the intersection point is
// returned in pI; otherwise false is returned.
//
bool Geometry::
Intersect( Point const &pA,
           Point const &pB,
           Point const &pC,
           Point const &pD,
           Point       &pI )
{
    // vector AB
    Point pAB = pB ;
    pAB -= pA ;

    // vector CD
    Point pCD = pD ;
    pCD -= pC ;

    // We check the sign of:
    //    (pB-pA)x(pC-pA)    (pB-pA)x(pD-pA)
    //    (pD-pC)x(pA-pC)    (pD-pC)x(pB-pC)
    Point pAC = pC ;
    pAC -= pA ;
    COORD dC = pAB % pAC,         // =0 means ABC are collinear
          dD = pAB % (pD - pA),   // =0 means ABD are collinear
          dA = -(pCD % pAC),      // =0 means ACD are collinear
          dB = pCD % (pB - pC) ;   // =0 means BCD are collinear

    // A bbox check is a easy way to detect non-intersecting cases but
    // for our use (finding the boundary of a flattened wide line), we
    // expect the segments to intersect more often than not, so we skip
    // that check unless it becomes necessary

    bool collC, collD, straddleCD ;

    // Check if CD straddles AB
    collC = collD = straddleCD = false ;
    if ( 0.0 < dC ) {
        if ( 0.0 < dD )
            return false ;       // C,D on same side of AB
        if ( 0.0 > dD )
            straddleCD = true ;
        else
            collD = true ;
    } else if ( 0.0 > dC ) {
        if ( 0.0 > dD )
            return false ;       // C,D on same side of AB
        if ( 0.0 < dD )
            straddleCD = true ;
        else
            collD = true ;
    } else {    // dC == 0, so C is collinear with AB
        collC = true ;
    }

    bool collA, collB, straddleAB ;

    // Check if AB straddles CD
    collA = collB = straddleAB = false ;
    if ( 0.0 < dA ) {
        if ( 0.0 < dB )
            return false ;       // A,B on same side of CD
        if ( 0.0 > dB )
            straddleAB = true ;
        else
            collB = true ;
    } else if ( 0.0 > dA ) {
        if ( 0.0 > dB )
            return false ;       // A,B on same side of CD
        if ( 0.0 < dB )
            straddleAB = true ;
        else
            collB = true ;
    } else {    // dA == 0, so A is collinear with AB
        collA = true ;
    }

    // Check if we have an intersection
    if ( straddleAB && straddleCD ) {
        COORD den = pAB % pCD ;         // denominator
        if ( !ApproxZero( den ) ) {    // we can divide
            COORD ba = pB % pA, cd = pC % pD ;
            // This is:
            //     pI = (cd * pAB - ba * pCD) / den ;
            // we write it using the assignment operators to avoid
            // excessive temporary generation
            //
            pAB *= cd ; pCD *= ba ; pI = pAB ; pI -= pCD ; pI /= den ;
            return true ;
        }
        // Here, the denominator is too small to divide, so we return
        // the midpoint of the intersection of bboxes (since the line
        // segments straddle one another, the bboxes MUST intersect).
        //
        Bbox bboxAB( pA, pB ), bboxCD( pC, pD ) ;
        Point minPt, maxPt ;

        // maximum of the minima
        minPt.x = bboxAB.llx <= bboxCD.llx ? bboxCD.llx : bboxAB.llx ;
        minPt.y = bboxAB.lly <= bboxCD.lly ? bboxCD.lly : bboxAB.lly ;
        // minimum of the maxima
        maxPt.x = bboxAB.urx <= bboxCD.urx ? bboxAB.urx : bboxCD.urx ;
        maxPt.y = bboxAB.ury <= bboxCD.ury ? bboxAB.ury : bboxCD.ury ;

        pI = (minPt + maxPt) / 2.0 ;
        return true ;
    }

    // Check if PT1 or PT2 is on PT3-PT4
#define CHECK( FLAG1, FLAG2, PT1, PT2, PT3, PT4 ) \
    if ( FLAG1 || FLAG2 ) { \
        Bbox bbox( PT3, PT4 ) ; \
        if ( FLAG1 && bbox.Inside( PT1 ) ) { \
            pI = PT1 ; return true ; \
        } \
        if ( FLAG2 && bbox.Inside( PT2 ) ) { \
            pI = PT2 ; return true ; \
        } \
        return false ; \
    }

    // Check if C or D is on AB
    CHECK( collC, collD, pC, pD, pA, pB ) ;

    // Check if A or B is on CD
    CHECK( collA, collB, pA, pB, pC, pD ) ;

    return false ;
}                                                           // Intersect

//----------------------------------------------------------------  Fmod
//
// Takes an arbitrary value x, and reduces it to the range
// 0 <= x < u and returns it. Intended for reducing angles to common
// ranges such as 0..90, 0..180, 0..360 etc.
//
COORD Geometry::
Fmod( COORD const ang, COORD const u )
{
    if ( 0.0 == u ) {
        ::Fatal( "(Geometry::Fmod): zero divisor" ) ;
    }
    COORD f = fmod( ang, u ) ;

    if ( ApproxZero( f ) ) {
        return 0.0 ;
    }

    return (ang < 0.0) ? (u + f) : f ;
}                                                                // Fmod

//----------------------------------------------------------------  Imod
//
// Takes an arbitrary integral value x, and reduces it to the range
// 0 <= x < u and returns it. Intended for reducing angles to common
// ranges such as 0..90, 0..180, 0..360 etc. MUST have u > 0.
//
INT32 Geometry::
Imod( INT32 const ang, INT32 const u )
{
    if ( 0 == u ) {
        ::Fatal( "(Geometry::Imod): zero divisor" ) ;
    }
    if ( ang >= u ) {
        return ang % u ;
    }
    if ( ang < 0 ) {
        return u - (-ang % u) ;
    }
    return ang ;
}                                                                // Imod
