#ifndef GEOMETRY_
#define GEOMETRY_

//======================================================================
//
// Basic geometry
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.9 $
//
//======================================================================

// Need hypot(), M_PI etc.
#if !defined(_MATH_H) && !defined(__math_h)
extern "C" {
#include <math.h>
}
#endif

#ifndef BBOX_
#include "Bbox.H"
#endif

//============================================================= PolyLine
//
// A polyline is a sequence of connected straight line segments (useful
// for dealing with flattened curves).
//
// In some contexts, we find it useful for the point array (mPts) to be
// owned elsewhere, so mOwn tells if we own it here or not.
//
// This class is used in ConicData and so is used by StokeFont. It
// should be kept simple; complex methods that are not needed by
// StrokeFont should be placed in derived classes.
//
class PolyLine {

    friend ostream& operator<<( ostream &os, const PolyLine &p ) ;

public:

    // data

    bool  mOwn ;    // true iff point array is owned here
    Point *mPts ;   // array of points
    INT32 mCnt ;    // number of elements in array (at least 2)

    // ctors

    PolyLine( ) : mOwn( false ), mPts( NIL ), mCnt( 0 ) { }

    // ownership of point array transferred iff aOwn is true
    PolyLine( Point *aPts, INT32 aCnt, bool aOwn ) {
        Init( aPts, aCnt, aOwn ) ;
    }

    // dtor
    ~PolyLine( ) {
        if ( mOwn ) {
            delete mPts ;
        }
        mPts = NIL ;
        mCnt = 0 ;
    }

    // other methods

    // This code is not placed in the constructor since we want to be
    // able to initialize an object that has been previously created.
    // Ownership of point array transferred iff aOwn is true
    //
    void Init( Point *aPts, INT32 aCnt, bool aOwn ) {
        mPts = aPts ; mCnt = aCnt ; mOwn = aOwn ;
        if ( 1 >= aCnt ) {
            Fatal( "(PolyLine::Init): bad count" ) ;
        }
    }

    // sanity check
    bool Valid() const { return mCnt > 1 && NIL != mPts ; }


    // The number of segments in the polyline is actually mCnt-1. The
    // segments are:
    //     A == flatPts[0] --> flatPts[1] --> ...
    //          --> flatPts[mCnt-1] == B
} ;                                                          // PolyLine

//===========================================================  ConicData
//
// Info about a conic; used to avoid copying this stuff
//
struct ConicData {

    // Vertices of defining triangle are located outside this struct

    // unitBulge:
    //     true iff bulge is close to 1.0
    // isGoodX:
    //     true if the x-coordinates along the conic are bounded by xA
    //     and xB, i.e. bulge >= 0 and min(xA,xB) <= xC <= max(xA,xB)
    // isGoodY:
    //     similar to isGoodX but for the y-coordinates.
    // isFlat is true iff conic is almost straight line; in this case
    //        both isGood{X,Y} are also set to 'true' and we treat it
    //        as just a straight segment from A to B even though this is
    //        not strictly accurate (e.g. in cases when C is outside the
    //        AB segment.)
    //
    bool  unitBulge, isGoodX, isGoodY, isFlat ;

    // Coefficients of the parametric equation needed for computing
    // intersections of conic with other graphic objects. See comments
    // in GeomConic.H for more info.
    //
    // For flattening when the bulge is < 0, we need to subdivide the
    // conic into two pieces and render them separately to get good
    // smoothness but we do not need to store the coefficients of the
    // pieces since we cache the flattened segments.
    //
    COORD zt2, zt1, zt0,   // denominator
          ft2, ft1, ft0,   // numerator x
          gt2, gt1, gt0 ;  // numerator y

    // Flattened polyline; computed on an as-needed basis. Note that
    // this may actually point to a derived class object as well. When
    // used from the StrokeFont class, it will point to a PolyLine; in
    // other contexts we may need cumulative length of the polyline etc.
    // so a derived class object may be used.
    //
    PolyLine *flats ;

    // only default ctor needed

    // ctor
    ConicData() : flats( NIL ) { }

    // dtor
    ~ConicData() {
        delete flats ;   flats = NIL ;
        // "delete 0" does nothing
    }
} ;                                                         // ConicData

//============================================================  Geometry
//
// Contains useful geometry routines and data all of which are static
// Should not be instantiated.
//
class Geometry {
public:

    // for equality: values that differ by less than this are
    // considered equal. Not made const so that it can be initialized
    // from a tcl variable. Something around 1e-6 is reasonable.
    //
    static COORD tiny ;

    static const COORD
        d2r,              //  = M_PI / 180.0 (degrees to radians)
        r2d ;             //  = 180.0 / M_PI (radians to degrees)

    static const COORD    // keep conic bulge in a reasonable range
        bulgeMin,         // = -0.99
        bulgeMax ;        // = 1000

    // Takes an arbitrary value x, and reduces it to the range
    // 0 <= x < u and returns it. Intended for reducing angles to common
    // ranges such as 0..90, 0..180, 0..360 etc.
    //
    static COORD
    Fmod( COORD const ang, COORD const u = 360.0 ) ;

    // Integer version of Fmod (since C does not guarantee the sign of
    // a%b when negative values are involved)
    //
    static INT32
    Imod( INT32 const ang, INT32 const u = 360 ) ;

    // Returns a discriminant which is >, =, or < 0 according as the
    // point 'p' is on one side of the line AB, on it or the other side.
    // A little less expensive than 'Dist()' since no division or sqrt()
    // is needed.
    //
    static COORD
    TestSide( Point const &pA, Point const &pB,
              Point const &p ) ;

    // Returns the (UNSIGNED) length of line segment AB.
    //
    static COORD
    Len( Point const &pA, Point const &pB ) {
        return hypot( static_cast<double>( pA.x - pB.x ),
                      static_cast<double>( pA.y - pB.y ) ) ;
    }

    // Returns the UNSIGNED distance of the point 'p' to the line AB.
    // A little more expensive than 'TestSide()' since it involves an
    // extra division and sqrt().
    //
    static COORD
    Dist( Point const &pA, Point const &pB,
          Point const &p ) ;

    // Returns true if line segments AB & CD intersect, false otherwise.
    // In the former case, intersection point is returned in pI
    //
    static bool
    Intersect( Point const &pA, Point const &pB,
               Point const &pC, Point const &pD,
               Point &pI
    ) ;

    static void
    Conic(  Point const &pA,
            Point const &pB,
            Point const &pC,
            COORD const bulge,
            Bbox        &bbox,
            ConicData   &data ) ;

    static void
    Flatten( Point const &pA,
             Point const &pB,
             Point const &pC,
             COORD const bulge,
             Bbox  const &bbox,
             ConicData   &data ) ;

private:
    Geometry() { }  // cannot derive or instantiate

    // Print fatal error message and exit
    static void Fatal( char const *const msg,
                       Point const &pA,
                       Point const &pB ) ;
} ;                                                          // Geometry

#endif  // GEOMETRY_
