#=======================================================================
#
# 'Pencil' stroke font
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.
#  
# Copyright (C 1999 CurveSoft, Inc. All rights reserved.
#
# $Revision: 1.13 $
#
#=======================================================================

# Stroke font 'Pencil'
#

# Font files are read by a separate tcl interpreter, so we only need to
# worry about namespace conflicts among fonts.
#
# The following globals names are in use:
#
#   mode -- value should be set to "display", "ps", or "scaler" (before
#           font file is evaluated) according as intended use is for
#           character proofing on a tk canvas, output of font as a
#           PS Type 3 font, or scaling characters to produce bitmaps.
#           Mode specific tcl scripts, viz. sfdisplay.tcl,
#           sfps.tcl and sfscaler.tcl are evaluated; these may be
#           shared across fonts.
#
#
#   N -- array containing all font-wide info
#
#   aName -- name of array containing font-wide info (used in mode
#            specific scripts so they can be shared by across fonts).
#
#   tmp -- temporary
#
#   S_, E_, M_, L_, C_,
#   moveto,  xmoveto,  ymoveto,  rmoveto,
#   lineto,  xlineto,  ylineto,  rxlineto,  rylineto,  rlineto
#   curveto, xcurveto, ycurveto, rxcurveto, rycurveto, rcurveto
#
#   S_ gc wx wy llx lly urx ury nSeg nStr
#       gc -- glyph code
#       wx -- advance width (x)
#       wy -- advance width (y)
#       llx, lly, urx, ury -- bbox (lower-left and upper-right)
#       nSeg -- no. of segments
#       nStr -- no. of strokes
#   E_ (takes no parameters)
#   M_, L_, C_ -- used for error checking in the various moveto, lineto
#       and curveto procs.
#
#   WrapupSf -- mode specific wrapup proc
#   $N(prefix)@?$N(suffix) -- individual character definitions, where ?
#       is the glyph code in lower case hex (1-8 chars).

# All font-wide values are in the array $aName. Different fonts should
# use different array names for this purpose.

#
set aName N

# Em-size: usually height+depth. This value is used to scale the font
# so that an n-point font will have an em-square that is n-points on the
# side. Thus, if d is a coordinate in font space, for an n-point font
# it will become
#    d' = (d * n * r) / (72 * m)
# where r is the device resolution (dpi) and m is the em-size
#
# It is usually a good idea to keep this large enough that all necessary
# coordinates can be integers
#
set N(emSize) 4096

# Various useful fractions of an em
set N(em05) [expr int(0.05 * $N(emSize))]
set N(em08) [expr int(0.08 * $N(emSize))]
set N(em10) [expr int(0.10 * $N(emSize))]
set N(em15) [expr int(0.15 * $N(emSize))]
set N(em20) [expr int(0.20 * $N(emSize))]
set N(em25) [expr int(0.25 * $N(emSize))]
set N(em30) [expr int(0.30 * $N(emSize))]
set N(em35) [expr int(0.35 * $N(emSize))]
set N(em40) [expr int(0.40 * $N(emSize))]
set N(em45) [expr int(0.45 * $N(emSize))]
set N(em50) [expr int(0.50 * $N(emSize))]
set N(em55) [expr int(0.55 * $N(emSize))]
set N(em60) [expr int(0.60 * $N(emSize))]
set N(em65) [expr int(0.65 * $N(emSize))]
set N(em70) [expr int(0.70 * $N(emSize))]


# font bbox; need not be tight, can be quite generous (used to check
# all point coordinates)
#
set tmp [expr 4.0 * $N(emSize)]
set N(llx) [expr -$tmp]
set N(lly) [expr -$tmp]
set N(urx) $tmp
set N(ury) $tmp

# Heights: These values are used to ensure that heights of certain
# character features are uniform. The "Over" values are overshoots if
# positive (e.g. ascender, x-height etc.) and undershoots if negative
# (e.g. baseLine, descender etc.)
#
# base  --  base line
# x     --  x height
# X     --  cap height
# asc   --  ascender height
# fig   --  figure height (i.e. digits 0, 1, etc.)
# dsc   --  descender height
#
# acc -- height of accent above another height
# xAcc -- lowercase accent height
# XAcc -- uppercase accent height
# math -- center line for math symbols
#
# They should all be integers; most are specified as multiples of em/24
#
set tmp                 [expr $N(emSize) / 24]
set N(overShoot)        [expr round($N(emSize) * 0.0125)]
set N(ht_base)          0
set N(ht_baseOver)      -$N(overShoot)
set N(ht_x)             [expr 9 * $tmp]
set N(ht_xOver)         [expr $N(ht_x)+$N(overShoot)]
set N(ht_X)             [expr 14 * $tmp]
set N(ht_XOver)         [expr $N(ht_X)+$N(overShoot)]
set N(ht_asc)           [expr 15 * $tmp]
set N(ht_ascOver)       [expr $N(ht_asc)+$N(overShoot)]
set N(ht_fig)           [expr 14 * $tmp]
set N(ht_figOver)       [expr $N(ht_fig)+$N(overShoot)]
set N(ht_dsc)           [expr -5 * $tmp]
set N(ht_dscOver)       [expr $N(ht_dsc)-$N(overShoot)]

set N(acc)           [expr int(0.1 * $N(emSize))]
set N(ht_xAcc)       [expr $N(ht_x) + $N(acc)]
set N(ht_XAcc)       [expr $N(ht_X) + $N(acc)]
set N(ht_math)       [expr $N(ht_X) / 2]

# various other parameters controlling shapes
#
# NOTE: Be _very_ careful when modifying these value; some character
#    widths and bboxes are calculated based on them and others are not,
#    so making large changes to these values could result in points
#    falling outside the bbox. Most should NOT be set to 0.
#

# half and full length of a typical serif
#
set N(serif) [expr round($N(emSize) * 0.07)]
set N(2serif) [expr 2 * $N(serif)]

# width of figures (digits)
#
set N(figWid) $N(em40)
set N(figAWid) $N(em50)

# radius of dot on i, j, dieresis, etc.
#
set N(dotRad) [expr round($N(emSize) * 0.015)]

# radius of ring (aring, Aring, etc.)
#
set N(ringRad) [expr round($N(emSize) * 0.06)]

# widths of various accents
#
# (If the width of acute, grave, circumflex, or dieresis is larger than
# a full length serif, it will need the bbox adjusted in the
# corresponding accented forms of 'i')
#
set tmp           [expr round($N(emSize) * 0.15)]
set N(wdGrave)    [expr round(0.9 * $tmp)]
set N(wdAcute)    $N(wdGrave)
set N(wdCircum)   [expr round(1.3 * $tmp)]
set N(wdTilde)    [expr round(1.3 * $tmp)]
set N(wdMacron)   [expr round(0.9 * $tmp)]
set N(wdBreve)    [expr round(1.3 * $tmp)]
set N(wdDot)      [expr 2 * $N(dotRad)]
set N(wdDieresis) [expr round(1.3 * $tmp)]
set N(wdRing)     [expr 2 * $N(ringRad)]
set N(wdCedilla)  [expr round(0.9 * $tmp)]


# heights of various accents
#
set tmp           [expr round(0.1 * $N(emSize))]
set N(htGrave)    [expr $tmp]
set N(htAcute)    $N(htGrave)
set N(htCircum)   [expr $tmp]
set N(htTilde)    [expr $tmp]
# set N(htMacron)   1
set N(htBreve)    [expr $tmp]
set N(htDot)      [expr 2 * $N(dotRad)]
set N(htDieresis) $N(htDot)
set N(htRing)     $N(wdRing)
set N(htCedilla)  [expr round(0.2 * $N(emSize))]

set N(htComma)    [expr $tmp]

# Flag to do screen display, write PostScript Type 3 font to file, or
# return info to font scaler.
#
# This definition is just for testing and should be removed.
#
# The calling environment should set it to one of 3 values:
#     display, ps, scaler
#
#set mode ps
#set mode scaler
#set mode display

# current point
set N(curx) 0
set N(cury) 0

# font name
set N(fontName) "Pencil"

# Header file for PostScript Type 3 font
set N(psHead) "FontHeader.ps"

# file name for output of PostScript Type 3 font
set N(psFile) "Pencil.ps"

# and id (used if mode is ps)
set N(fileId) ""

# prefix and suffix of char procs: each proc will be named
#     $N(prefix)@?$N(suffix)
# where ? is the glyphcode in lowercase hex
set N(prefix) "_"
set N(suffix) "_"

# read the mode specific and common definitions
#
if {![string compare $mode scaler]} {
    source sfscaler.tcl
} elseif {![string compare $mode display]} {
    source sfdisplay.tcl
} elseif {![string compare $mode ps]} {
    source sfps.tcl
} else {
    puts stderr "ERROR: Bad mode"; exit 1
}
source sfcommon.tcl

# Character definitions.
#
# Every char definition MUST execute the S_ before any of the moveto,
# lineto, or curveto procs and end with E_; moreover, the S_ proc must
# be executed directly by the char proc (i.e. it should not be placed in
# a subroutine). This is because in ps or display mode, the glyph code
# argument is checked against the code extracted from the call proc
# name which is obtained using the "info level" tcl command.
#
# Each proc name is formed by prepending '@' to the glyph-code in
# lowercase hex, then prefixed and suffixed by $N(prefix) and $N(suffix)
# respectively.
#
# Subroutines used by the character definitions (for accented characters
# for example) should omit the '@'
#
# NOTE: It is CRITICAL that there should be no procs matching the
#       pattern $N(prefix)@*$N(suffix) other than character
#       definitions: the code to output a Type 3 PS font will execute
#       all such procs
#
# NOTE: The bounding box MUST be large enough to accomodate all points;
#       otherwise a run-time abort occurs. Watch out for cumulative
#       errors from consecutive 'rmoveto' type commands which may take
#       a point outside the intended bbox.
#
# NOTE: The definitions below assume the baseline is at y=0
#
# NOTE: For efficient tcl execution, place no comments inside procs; put
#       them all at the beginning of each proc.
#

# Auxillary procs used by the char defs (e.g. components of accented
# chars etc.

#--------------------------------------------------------------  accents
# accent components of accented chars
#
# NOTE: changes here may require corresponding changes to the width,
#       bbox, number of strokes/segments in all composite characters
#       using the accent.

# dot for i, j, period, semi-colon, colon etc. Arguments are coords of
# center and radius
proc $N(prefix)dot$N(suffix) {x y r} {
    set w [expr $r + $r]; moveto [expr $x + $r] [expr $y + $r]
    rylineto -$w; rxlineto -$w; rylineto $w; rxlineto $w
}

# grave; args are lower-left point, height and width
proc $N(prefix)grave$N(suffix) {x y h w} {
    moveto $x $y; rlineto -$w $h
}

# acute; args are lower-right point, height and width
proc $N(prefix)acute$N(suffix) {x y h w} {
    moveto $x $y; rlineto $w $h
}

# circum; args are lower-left point, height and width
proc $N(prefix)circum$N(suffix) {x y h w} {
    set w2 [expr $w / 2]
    moveto $x $y; rlineto $w2 $h; rlineto $w2 -$h
}

# tilde; args are lower-left point, height, and width
proc $N(prefix)tilde$N(suffix) {x y h w} {
    set w3 [expr $w / 3]
    set w6 [expr $w / 6]
    set d1  [expr $w / 12]
    set d2  [expr $w6 - $d1]
    set h2 [expr round($h / 2.0)]
    moveto $x $y; rycurveto $w3 $h 1.0; rcurveto $w6 -$h2 $d1 0 0.8
    rcurveto $w6 -$h2 $d2 -$h2 0.8; rxcurveto $w3 $h 1.0
}

# macron; args are left endpoint and width
proc $N(prefix)macron$N(suffix) {x y w} {
    moveto $x $y; rxlineto $w
}

# breve; args are upper-left point, height and width
proc $N(prefix)breve$N(suffix) {x y h w} {
    set w2 [expr $w / 2]
    moveto $x $y; rycurveto $w2 -$h 1.5; rxcurveto $w2 $h 1.5
}

# dieresis; args are left center point, dot-radius and distance to
# right center point
proc $N(prefix)dieresis$N(suffix) {x y r d} {
    global N
    $N(prefix)dot$N(suffix) $x $y $r
    $N(prefix)dot$N(suffix) [expr $x + $d] $y $r
}

# ring; args are center and radius
proc $N(prefix)ring$N(suffix) {x y r} {
    moveto [expr $x - $r] $y; rycurveto $r $r 0.7
     rxcurveto $r -$r 0.7; rycurveto -$r -$r 0.7; rxcurveto -$r $r 0.7
}

# cedilla; args are top point, height and width
proc $N(prefix)cedilla$N(suffix) {x y h w} {
    set w4 [expr $w / 4]
    set w34 [expr 3 * $w4]
    set h3 [expr $h / 3]
    set h23 [expr 2 * $h3]
    moveto $x $y
    rlineto -$w4 -$h3; rxcurveto $w34 -$h3 0.8; rycurveto -$w -$h3 0.8
}

# Naming convention for local variables (note that many characters do
# not yet conform to these conventions):
#    aw  -- advance width
#    wd  -- bbox xmax (right)
#    ht  -- bbox ymax (top)
#    h?  -- heights
#    xm  -- x midpoint of bbox
#    ym  -- y midpoint of bbox
#    x?  -- significant x positions
#    y?  -- significant y positions
#    s   -- half serif
#    s2  -- full serif
#    q?, k? -- temporaries

#----------------------------------------------------------------  bases
# base components of accented chars
#
# NOTE: changes here may require corresponding changes to the width,
#       bbox, number of strokes/segments in all composite characters
#       using the base component.

# A base
#   q -- height of bar as a fraction of total height
proc $N(prefix)A$N(suffix) {wid} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set h1 $N(ht_XOver)

    set q 0.3
    set xm [expr $wid / 2]
    set y1 [expr $q * $h1]
    if $s {
        set x2 [expr $wid - $s]
        set k [expr $q * ($xm - $s)]
        set xl [expr $s + $k];        set xr [expr $x2 - $k]

        moveto 0 0; rxlineto $s2
        moveto $wid 0; rxlineto -$s2
        moveto $s 0; lineto $xm $h1; lineto $x2 0
        moveto $xl $y1; xlineto $xr
    } {
        set xl [expr $q * $xm]; set xr [expr $wid - $xl]
        moveto 0 0; lineto $xm $h1; lineto $wid 0
        moveto $xl $y1; xlineto $xr
    }
}

# C base
proc $N(prefix)C$N(suffix) {wid} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set h1 $N(ht_XOver)
    set h2 $N(ht_baseOver)
    set h3 $N(ht_X)
    set q1 $N(em05)
    set q2 $N(em10)

    set xm [expr $wid / 2]
    set ym [expr ($h1 + $h2) / 2]
    set x1 [expr $wid - $q2]
    if $s {
        moveto $wid [expr $h3 - $q1]; rylineto -$s2
        set y1 [expr $h3 - $q1 - int(0.75 * $s2)]
    } {
        set y1 [expr $h3 - $q2]
    }
    moveto $wid $y1; ycurveto $xm $h1; xcurveto 0 $ym
    ycurveto $xm $h2; curveto $wid $q2 $x1 $h2 1.3
}

# D base
proc $N(prefix)D$N(suffix) {wid x} {
    global N
    set s $N(serif)
    set h $N(ht_X)

    set xm [expr $wid / 2]
    set x1 [expr $x + $s]
    set x2 [expr $x + $xm]
    set x3 [expr $x + $wid]

    set y1 [expr round($h * 0.5)]

    moveto $x1 0; ylineto $h
    moveto $x 0; xlineto $x2; xcurveto $x3 $y1
    ycurveto $x2 $h; xlineto $x
}

# E base
proc $N(prefix)E$N(suffix) {wid} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set h $N(ht_X)

    set x1 [expr $wid - $s]
    set x2 [expr int(0.8 * $x1)]
    set ym [expr $h / 2]

    moveto $s 0; ylineto $h
    moveto $s $ym; xlineto $x2;
    if $s {
        moveto 0 0; xlineto $x1; lineto $wid $s
        moveto 0 $h; xlineto $x1; rylineto -$s
        moveto $x2 [expr $ym - $s]; rylineto $s2
    } {
        moveto 0 0; xlineto $x1
        moveto 0 $h; xlineto $x1
    }
}

# I base
proc $N(prefix)I$N(suffix) {x} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set h $N(ht_X)

    if $s {
        moveto $x 0; rxlineto $s2; moveto $x $h; rxlineto $s2
        moveto [expr $x + $s] 0; ylineto $h
    } {
        moveto $x 0; ylineto $h
    }
}

# N base
proc $N(prefix)N$N(suffix) {wid} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set h $N(ht_X)

    set x1 [expr $wid - $s]

    if $s {
        moveto 0 0; xlineto $s2
        moveto $s 0; ylineto $h
        moveto 0 $h; xlineto $s; lineto $x1 0; ylineto $h
        moveto $wid $h; rxlineto -$s2
    } {
        moveto 0 0; ylineto $h; lineto $wid 0; ylineto $h
    }
}

# O base
#
proc $N(prefix)O$N(suffix) {wid} {
    global N
    set em $N(emSize)
    set h $N(ht_XOver)
    set hbo $N(ht_baseOver)

    set w [expr int($wid / 2.0)]
    set m [expr ($h - $hbo) / 2]

    moveto $w $hbo; rxcurveto  $w $m; rycurveto -$w $m
    moveto $w $hbo; rxcurveto -$w $m; rycurveto  $w $m
}

# U base
#
proc $N(prefix)U$N(suffix) {wid} {
    global N
    set em $N(emSize)
    set h $N(ht_X)
    set s $N(serif)
    set s2 $N(2serif)
    set hbo $N(ht_baseOver)

    set w [expr $wid / 2]
    set m [expr ($h - $hbo) / 2]
    set q $N(em10)

    if $s {
        set x1 [expr $wid - $s]
        moveto 0 $h; xlineto $s2; moveto $wid $h; rxlineto -$s2
        moveto $s $h; ylineto $q; ycurveto $w $hbo
        xcurveto $x1 $q; ylineto $h
    } {
        moveto 0 $h; ylineto $q; ycurveto $w $hbo
        xcurveto $wid $q; ylineto $h
    }
}

# Y base
#
proc $N(prefix)Y$N(suffix) {wid} {
    global N
    set em $N(emSize)
    set h $N(ht_X)
    set s $N(serif)
    set s2 $N(2serif)

    set w [expr $wid / 2]
    set x1 [expr $wid - $s]
    set y1 [expr $h / 3]

    if $s {
        moveto 0 $h; xlineto $s2; moveto $wid $h; rxlineto -$s2
        moveto $s $h; lineto $w $y1; lineto $x1 $h
        moveto $w $y1; ylineto 0
        moveto [expr $w - $s] 0; rxlineto $s2
    } {
        moveto 0 $h; lineto $w $y1; lineto $x1 $h
        moveto $w $y1; ylineto 0
    }
}

# base a
#  aw -- advance width
#  wid -- bbox width
#  t -- width of tail of 'a'
#  m -- vertical midpoint of bbox
#  w -- horizontal midpoint of bbox excluding tail
#  q -- controls various internal features
#
proc $N(prefix)a$N(suffix) {wid t w} {
    global N
    set em $N(emSize)
    set hbo $N(ht_baseOver)
    set h $N(ht_xOver)

    set q  [expr round(0.13 * $em)]
    set q2 [expr $q / 2]
    set q3 [expr $q / 3]
    set m  [expr round(($h + $hbo) / 2.0)]
    set m2 [expr round($m / 2.0)]

    moveto    $q3 [expr $h - $q2]
    ycurveto  $w  $h  0.8;    rxcurveto $w  -$q 1.0
    if {0 != $t} {
        ylineto $q; ycurveto $wid 0 2.0
    } {
        ylineto 0
    }

    rmoveto -$t [expr $h-$q]
    ycurveto $w  $m  1.0;     rxcurveto -$w  -$m2 1.0
    ycurveto   $w  $hbo 1.0;  rxcurveto  $w  $q 0.5
}

# base c
#
proc $N(prefix)c$N(suffix) {wid w k} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set h $N(ht_xOver)
    set bo $N(ht_baseOver)

    set cx [expr int($w + (($wid - $w)/2))]
    set m [expr ($h - $bo) / 2]
    set q [expr round(0.075 * $em)]
    set p [expr $N(ht_x) - $q]

    moveto $wid $q; curveto $w $bo $cx $bo 1.5; xcurveto 0 $m
    ycurveto $w $h; xcurveto $wid $p 0.5; rlineto 0 -$s
}

# base e
proc $N(prefix)e$N(suffix) {wid} {
    global N
    set em $N(emSize)
    set h $N(ht_xOver)
    set bo $N(ht_baseOver)

    set w [expr int($wid / 2.0)]
    set cx [expr int($w + (($wid - $w)/2))]
    set m [expr ($h - $bo) / 2]
    set q [expr round(0.075 * $em)]

    moveto $wid $q; curveto $w $bo $cx $bo 1.5; xcurveto 0 $m
    ycurveto $w $h; xcurveto $wid $m; xlineto 0
}

# dotless i; argument is left end
proc $N(prefix)dotlessi$N(suffix) {x} {
    global N
    set s $N(serif)
    set h $N(ht_x)

    if $s {
        moveto $x $h; rxlineto $s; ylineto 0
        moveto $x 0; rxlineto $N(2serif)
    } {
        moveto $x 0; ylineto $h
    }
}

# n base
proc $N(prefix)n$N(suffix) {wid} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set h1 $N(ht_xOver)
    set h2 $N(ht_x)
    set q1 $N(em15)

    set q2 [expr round(0.03 * $em)]
    set w [expr $wid / 2 + $q2]
    set x1 [expr $wid - $s]
    set x2 [expr $w - $q1]
    set y1 [expr $h1 - $q1]

    if $s {
        set s2 $N(2serif)
        moveto 0 $h2; xlineto $s; ylineto 0
        moveto 0 0; rxlineto $s2; moveto $wid 0; rxlineto -$s2
        moveto $x1 0; ylineto $y1; ycurveto $w $h1 1.6
        curveto $s $y1 $x2 $h1 0.7
    } {
        moveto 0 0; ylineto $h2
        moveto $x1 0; ylineto $y1; ycurveto $w $h1 1.6
        curveto 0 $y1 $x2 $h1 0.7
    }
}

# o base
#
proc $N(prefix)o$N(suffix) {wid} {
    global N
    set h $N(ht_xOver)
    set bo $N(ht_baseOver)

    set w [expr int($wid / 2.0)]
    set m [expr ($h - $bo) / 2]

    moveto $w $bo; rxcurveto  $w $m; rycurveto -$w $m
    moveto $w $bo; rxcurveto -$w $m; rycurveto  $w $m
}

# u base
proc $N(prefix)u$N(suffix) {wid} {
    global N
    set em $N(emSize)
    set s $N(serif)

    set q1 $N(em15)
    set q2 [expr round(0.03 * $em)]
    set w [expr int($wid / 2.0) - $q2]
    set h1 $N(ht_baseOver)
    set h2 $N(ht_x)

    set x1 [expr $wid - $s]
    set x2 [expr $w + $q1]
    set y1 [expr $h1 + $q1]

    if $s {
        moveto $wid 0; rxlineto -$s; ylineto $h2; rxlineto -$s
        moveto 0 $h2; rxlineto $s; ylineto $y1; ycurveto $w $h1 1.6
        curveto $x1 $y1 $x2 $h1 0.7
    } {
        moveto $wid 0; ylineto $h2;
        moveto 0 $h2; ylineto $y1; ycurveto $w $h1 1.6
        curveto $x1 $y1 $x2 $h1 0.7
    }
}

# y base
proc $N(prefix)y$N(suffix) {wid} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)

    set w [expr $wid / 2.0]
    set h1 $N(ht_x)
    set h2 $N(ht_baseOver)
    set h3 $N(ht_dsc)


    if $s {
        set x1 [expr $wid - $s]
        moveto 0 $h1; rxlineto $s2;    moveto $wid $h1; rxlineto -$s2
        moveto $s $h1; lineto $w $h2
        moveto $x1 $h1; lineto $w $h2; curveto 0 $h3 $s2 $h3 1.5
    } {
        set x1 [expr int($em * 0.07)]
        moveto 0 $h1; lineto $w $h2
        moveto $wid $h1; lineto $w $h2; curveto 0 $h3 $x1 $h3 1.5
    }
}

#--------------------  components for various punctuation, math etc.

# comma; can be used in semi-colon, apostrophe, close double quotes etc.
# args are positon of dot center and width and height of comma
proc $N(prefix)comma$N(suffix) {x y wd ht} {
    global N
    set r $N(dotRad)

    $N(prefix)dot$N(suffix) $x $y $r
    set x1 [expr $x + $r];  set x2 [expr $x1 - $wd]
    set y2 [expr $y - $ht + $r]; set y3 [expr ($y + $y2) / 2]
    moveto $x1 $y; curveto $x2 $y2 $x1 $y3
}

# greater/less (wid > 0 gives greater, wid < 0 yields less
proc $N(prefix)><$N(suffix) {wid} {
    global N
    set hm $N(ht_math)

    set ht [expr 2 * $hm]


    if {$wid > 0} {
        moveto 0 $ht; lineto $wid $hm; lineto 0 0
    } {
        set wid [expr -$wid]
        moveto $wid $ht; lineto 0 $hm; lineto $wid 0
    }
}

#-------------------------------------------------------------  chardefs
# Actual char defs

# ' ' (blank)
proc $N(prefix)@20$N(suffix) {} {
    global N
    set em $N(emSize)
    set aw [expr 0.27 * $em]

    S_ 20 $aw 0   0 0 0 0   0 0
    E_
}

# ',' (comma)
proc $N(prefix)@2c$N(suffix) {} {
    global N
    set wd $N(wdDot)
    set ht $N(htComma)
    set hd $N(htDot)
    set r $N(dotRad)

    set aw [expr $N(em10) + $wd]
    set y [expr -($ht - $hd)]

    S_ 2c $aw 0   0 $y $wd $ht   5 2
    $N(prefix)comma$N(suffix) $r $r $wd $ht
    E_
}

# digits
#
# it is generally a good idea to use the same width for all digits,
# since that ensures that they will line up properly in tables etc.
#

# 0
proc $N(prefix)@30$N(suffix) {} {
    global N
    set aw $N(figAWid)
    set wd $N(figWid)
    set ht $N(ht_figOver)
    set h1 $N(ht_baseOver)

    set xm [expr $wd / 2]
    set ym [expr ($ht + $h1) / 2]

    S_ 30 $aw 0   0 $h1 $wd $ht   4 2
    moveto $xm $ht; xcurveto $wd $ym 0.6; ycurveto $xm $h1 0.6
    moveto $xm $ht; xcurveto   0 $ym 0.6; ycurveto $xm $h1 0.6
    E_
}

# 1
proc $N(prefix)@31$N(suffix) {} {
    global N
    set aw $N(figAWid)
    set wd $N(figWid)
    set q1 $N(em05)
    set s $N(serif)
    set s2 $N(2serif)
    set ht $N(ht_figOver)

    set xm [expr $wd / 2]
    set x1 [expr $xm - $q1]
    set y1 [expr $ht - 3 * $q1]

    if $s {
        if {$s >= $q1} {
            set xmin [expr $xm - $s]
        } {
            set xmin $x1
        }
        S_ 31 $aw 0   $xmin 0 $wd $ht   3 2
        moveto $xmin 0; rxlineto $s2
    } {
        S_ 31 $aw 0   $x1 0 $wd $ht   2 1
    }
    moveto $x1 $y1; lineto $xm $ht; ylineto 0
    E_
}

# 2
proc $N(prefix)@32$N(suffix) {} {
    global N
    set aw $N(figAWid)
    set wd $N(figWid)
    set s $N(serif)
    set ht $N(ht_figOver)

    set xm [expr ($wd - $s) / 2]
    set x1 [expr $wd - $s]
    set y1 [expr $ht - $N(em15)]
    set y2 [expr $y1 - $N(em10)]

    if $s {
        S_ 32 $aw 0   0 0 $wd $ht   5 1
        moveto 0 $y1; ycurveto $xm $ht; xcurveto $x1 $y1
        curveto 0 0 $x1 $y2; xlineto $x1; lineto $wd $s
    } {
        S_ 32 $aw 0   0 0 $wd $ht   4 1
        moveto 0 $y1; ycurveto $xm $ht; xcurveto $wd $y1
        curveto 0 0 $x1 $y2; xlineto $wd
    }
    E_
}

# 3
proc $N(prefix)@33$N(suffix) {} {
    global N
    set aw $N(figAWid)
    set wd $N(figWid)
    set s $N(serif)
    set ht $N(ht_fig)
    set h1 $N(ht_baseOver)

    set xm [expr $wd / 2]
    set y1 [expr $ht - $s]
    set y2 [expr $h1 + int( 0.6 * ($ht - $h1) )]
    set y3 [expr ($h1 + $y2) / 2]

    if $s {
        S_ 33 $aw 0   0 0 $wd $ht   6 1
        moveto 0 $y1; lineto $s $ht; xlineto $wd; lineto $xm $y2
        xcurveto $wd $y3; ycurveto $xm 0; xcurveto 0 $y3
    } {
        S_ 33 $aw 0   0 0 $wd $ht   5 1
        moveto 0 $ht; xlineto $wd; lineto $xm $y2
        xcurveto $wd $y3; ycurveto $xm 0; xcurveto 0 $y3
    }
    E_
}

# 4
proc $N(prefix)@34$N(suffix) {} {
    global N
    set aw $N(figAWid)
    set wd $N(figWid)
    set s $N(serif)
    set s2 $N(2serif)
    set ht $N(ht_figOver)

    set x1 [expr $wd * 3 / 4]
    set x2 [expr $x1 - $s]
    set y1 [expr $ht / 4]

    if $s {
        S_ 34 $aw 0   0 0 $wd $ht   4 2
        moveto $wd $y1; xlineto 0; lineto $x1 $ht; ylineto 0
        xmoveto $x2; rxlineto $s2
    } {
        S_ 34 $aw 0   0 0 $wd $ht   3 1
        moveto $wd $y1; xlineto 0; lineto $x1 $ht; ylineto 0
    }
    E_
}

# 5
proc $N(prefix)@35$N(suffix) {} {
    global N
    set em05 $N(em05)
    set em10 $N(em10)
    set aw $N(figAWid)
    set wd $N(figWid)
    set ht $N(ht_fig)
    set h1 $N(ht_baseOver)

    set xm [expr $wd / 2]
    set x1 $em05
    set x2 [expr $wd - $x1]
    set y1 [expr ($ht - $h1) / 3]
    set y2 [expr $y1 * 2 - $em05]
    set y3 [expr $y2 - $em05]
    set y4 [expr $y1 - $em10]

    S_ 35 $aw 0   0 $h1 $wd $ht   6 1
    moveto $x2 $ht; xlineto $x1; lineto 0 $y3
    curveto $xm $y2 [expr $xm - $em10] $y2
    xcurveto $wd $y1 0.8; ycurveto $xm $h1 1.1; xcurveto 0 $y4
    E_
}

# 6
proc $N(prefix)@36$N(suffix) {} {
    global N
    set em05 $N(em05)
    set em10 $N(em10)
    set aw $N(figAWid)
    set wd $N(figWid)
    set ht $N(ht_figOver)
    set h1 $N(ht_baseOver)

    set xm [expr $wd / 2]
    set x1 [expr $wd - $em05]
    set y1 [expr ($ht - $h1) / 3]
    set y2 [expr $y1 * 2 - $em05]
    set y3 [expr $y2 - $em10]
    set y4 [expr $ht - $em10]

    S_ 36 $aw 0   0 $h1 $wd $ht   6 1
    moveto $x1 $y4; ycurveto $xm $ht; xcurveto 0 $y1 1.2
    ycurveto $xm $h1; xcurveto $wd $y1; ycurveto $xm $y2; xcurveto 0 $y3
    E_
}

# 7
proc $N(prefix)@37$N(suffix) {} {
    global N
    set s $N(serif)
    set aw $N(figAWid)
    set wd $N(figWid)
    set ht $N(ht_fig)

    set x1 [expr $wd / 2]
    set y1 [expr $ht - $s]
    set y2 [expr $ht / 2]

    if $s {
        S_ 37 $aw 0   0 0 $wd $ht   3 1
        moveto 0 $y1; lineto $s $ht; xlineto $wd;
        curveto $x1 0 $x1 $y2
    } {
        S_ 37 $aw 0   0 0 $wd $ht   2 1
        moveto 0 $ht; xlineto $wd; curveto $x1 0 $x1 $y2
    }
    E_
}

# 8
proc $N(prefix)@38$N(suffix) {} {
    global N
    set em05 $N(em05)
    set s $N(serif)
    set aw $N(figAWid)
    set wd $N(figWid)
    set ht $N(ht_figOver)
    set h1 $N(ht_baseOver)

    set xm [expr $wd / 2]
    set x1 [expr $wd - $em05]
    set y1 [expr int(0.6 * $ht)]
    set y2 [expr $y1 / 2]
    set y3 [expr ($ht + $y1) / 2]

    S_ 38 $aw 0   0 $h1 $wd $ht   8 2
    moveto $xm $h1; xcurveto 0 $y2; ycurveto $xm $y1
    xcurveto $em05 $y3; ycurveto $xm $ht
    moveto $xm $h1; xcurveto $wd $y2; ycurveto $xm $y1
    xcurveto $x1 $y3; ycurveto $xm $ht
    E_
}

# 9
proc $N(prefix)@39$N(suffix) {} {
    global N
    set em05 $N(em05)
    set s $N(serif)
    set aw $N(figAWid)
    set wd $N(figWid)
    set ht $N(ht_figOver)
    set h1 $N(ht_baseOver)

    set xm [expr $wd / 2]
    set x1 [expr $xm - $em05]
    set y1 [expr int(0.4 * $ht)]
    set y2 [expr ($y1 + $ht) / 2]

    S_ 39 $aw 0   0 $h1 $wd $ht   5 1
    moveto $wd $y2
    ycurveto $xm $y1
    xcurveto 0 $y2
    ycurveto $xm $ht
    xcurveto $wd $y2
    curveto $x1 $h1 $wd [expr $y1 / 2]
    E_
}

# colon
proc $N(prefix)@3a$N(suffix) {} {
    global N
    set ht $N(em30)
    set wd $N(wdDot)
    set r $N(dotRad)

    set aw [expr $N(em10) + $wd]
    set y1 [expr $ht - $r]

    S_ 3a $aw 0   0 0 $wd $ht   8 2
    $N(prefix)dot$N(suffix) $r $r $r
    $N(prefix)dot$N(suffix) $r $y1 $r
    E_
}

# semi-colon
proc $N(prefix)@3b$N(suffix) {} {
    global N
    set ht $N(em30)
    set wd $N(wdDot)
    set r $N(dotRad)
    set hc $N(htComma)
    set hd $N(htDot)

    set aw [expr $N(em10) + $wd]
    set y1 [expr $ht - $r]
    set y2 [expr -($hc - $hd)]

    S_ 3b $aw 0   0 $y2 $wd $ht   9 3
    $N(prefix)comma$N(suffix) $r $r $wd $hc
    $N(prefix)dot$N(suffix) $r $y1 $r
    E_
}

# less: '<'
proc $N(prefix)@3c$N(suffix) {} {
    global N
    set em $N(emSize)
    set wd $N(em40)
    set aw $N(em50)
    set hm $N(ht_math)

    set ht [expr 2 * $hm]

    S_ 3c $aw 0   0 0 $wd $ht   2 1
    $N(prefix)><$N(suffix) -$wd
    E_
}

# equal: '='
proc $N(prefix)@3d$N(suffix) {} {
    global N
    set em $N(emSize)
    set wd $N(em40)
    set aw $N(em50)
    set q $N(em10)
    set hm $N(ht_math)

    set ht [expr $hm + $q]
    set y1 [expr $hm - $q]

    S_ 3d $aw 0   0 0 $wd $ht   2 2
    moveto 0 $y1; xlineto $wd; moveto 0 $ht; xlineto $wd
    E_
}

# greater: '>'
proc $N(prefix)@3e$N(suffix) {} {
    global N
    set em $N(emSize)
    set wd $N(em40)
    set aw $N(em50)
    set hm $N(ht_math)

    set ht [expr 2 * $hm]

    S_ 3e $aw 0   0 0 $wd $ht   2 1
    $N(prefix)><$N(suffix) $wd
    E_
}

# question: '?'
proc $N(prefix)@3f$N(suffix) {} {
    global N
    set em $N(emSize)
    set wd $N(em40)
    set aw $N(em50)
    set q $N(em15)
    set ht $N(ht_X)
    set r $N(dotRad)

    set xm [expr $wd / 2]
    set y1 [expr $ht - $q]
    set y2 [expr $y1 - $q]
    set y3 [expr $y2 - $q]

    S_ 3f $aw 0   0 0 $wd $ht   8 2
    moveto 0 $y1; ycurveto $xm $ht; xcurveto $wd $y1; ycurveto $xm $y2
    ylineto $y3
    $N(prefix)dot$N(suffix) $xm $r $r
    E_
}

# at: '@'
proc $N(prefix)@40$N(suffix) {} {
    global N
    set em $N(emSize)
    set wd $N(em45)
    set aw $N(em55)
    set k2 $N(em15)
    set k1 $N(em10)
    set y5 $N(em10)
    set k3 $N(em05)
    set ht $N(ht_X)

    set xm [expr $wd / 2]
    set x1 [expr $xm - $k1]
    set x2 [expr $xm + $k1]
    set x3 [expr ($x2 + $wd) / 2]

    set ym [expr $ht / 2]
    set y1 [expr $ym - $k2]
    set y2 [expr $ym + $k2]
    set y3 [expr $y2 - $k3]
    set y4 [expr $y1 + $k3]

    S_ 40 $aw 0   0 0 $wd $ht   11 2
    moveto $x2 $y2; ylineto $y4
    moveto $x2 $y3; ycurveto $xm $y2; xcurveto $x1 $ym; ycurveto $xm $y1
    xcurveto $x2 $y4
    ycurveto $x3 $y1; xcurveto $wd $ym; ycurveto $xm $ht
    xcurveto 0 $ym; ycurveto $xm 0; curveto $wd $y5 $x3 0
    E_
}

# A
proc $N(prefix)@41$N(suffix) {} {
    global N
    set ht $N(ht_XOver)
    set aw $N(em50)
    set wd $N(em40)
    set s $N(serif)

    if $s {
        S_ 41 $aw 0   0 0 $wd $ht   5 4
    } {
        S_ 41 $aw 0   0 0 $wd $ht   3 2
    }
    $N(prefix)A$N(suffix) $wd
    E_
}

# B
proc $N(prefix)@42$N(suffix) {} {
    global N
    set ht $N(ht_X)
    set s $N(serif)
    set aw $N(em60)
    set wd $N(em50)

    set w [expr $wd / 2]

    set x1 [expr round($wd * 0.6)]
    set x2 [expr round($wd * 0.5)]
    set x3 [expr round($wd * 0.8)]

    set y1 [expr round($ht * 0.3)]
    set y2 [expr round($ht * 0.6)]
    set y3 [expr round($ht * 0.8)]

    S_ 42 $aw 0   0 0 $wd $ht   8 3
    moveto $s 0; ylineto $ht
    moveto 0 0; xlineto $x1; xcurveto $wd $y1
    ycurveto $x2 $y2; xlineto $s
    moveto 0 $ht; xlineto $x2; xcurveto $x3 $y3
    ycurveto $x2 $y2
    E_
}

# C
proc $N(prefix)@43$N(suffix) {} {
    global N
    set h1 $N(ht_XOver)
    set h2 $N(ht_baseOver)
    set aw $N(em60)
    set wd $N(em50)

    S_ 43 $aw 0   0 $h2 $wd $h1   5 2
    $N(prefix)C$N(suffix) $wd
    E_
}

# D
proc $N(prefix)@44$N(suffix) {} {
    global N
    set h $N(ht_X)
    set aw $N(em60)
    set wd $N(em50)

    S_ 44 $aw 0   0 0 $wd $h   5 2
    $N(prefix)D$N(suffix) $wd 0
    E_
}

# E
proc $N(prefix)@45$N(suffix) {} {
    global N
    set h $N(ht_X)
    set aw $N(em60)
    set wd $N(em50)

    S_ 45 $aw 0   0 0 $wd $h   7 5
    $N(prefix)E$N(suffix) $wd
    E_
}

# F
proc $N(prefix)@46$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em50)
    set wd $N(em40)
    set q3 $N(em10)
    set q1 $N(em08)
    set h1 $N(ht_X)

    set q2 [expr $q1 / 2]
    set x1 [expr $wd - $q1]
    set x2 [expr $wd - $q2]
    set x3 [expr $wd - $q3]

    set y1 [expr $h1 / 2]
    set y2 [expr $h1 - $q1]

    S_ 46 $aw 0   0 0 $wd $h1   6 5
    moveto 0 0; xlineto $s2
    moveto $s 0; ylineto $h1
    moveto 0 $h1; xlineto $x2; lineto $wd $y2
    moveto $s $y1; xlineto $x3;
    rmoveto 0 -$s; rylineto $s2
    E_
}

# G
proc $N(prefix)@47$N(suffix) {} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em60)
    set wd $N(em50)
    set q $N(em10)
    set h1 $N(ht_XOver)
    set h2 $N(ht_baseOver)
    set h3 $N(ht_X)

    set w [expr $wd / 2]

    set x0 [expr $wd - $s]
    set x1 [expr $x0 - $q]
    set y1 [expr int(0.4 * $h1)]
    set y2 [expr $h1 - ($q / 2)]

    S_ 47 $aw 0   0 $h2 $wd $h1   7 3
    moveto $x0 $y2; rylineto -$s2
    ycurveto $w $h1; xcurveto 0 $y1
    ycurveto $w $h2; curveto $x0 $q $x1 $h2 1.3
    moveto $w $y1; xlineto $wd
    moveto $x0 $y1; ylineto $h2
    E_
}

# H
proc $N(prefix)@48$N(suffix) {} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em60)
    set wd $N(em50)
    set h1 $N(ht_X)

    set x1 [expr $wd - $s]
    set y1 [expr $h1 / 2]

    S_ 48 $aw 0   0 0 $wd $h1   7 7
    moveto 0 0; xlineto $s2;       moveto 0 $h1; xlineto $s2
    moveto $wd 0; rxlineto -$s2;  moveto $wd $h1; rxlineto -$s2
    moveto $s 0; ylineto $h1;      moveto $x1 0; ylineto $h1
    moveto $s $y1; xlineto $x1
    E_
}

# I
proc $N(prefix)@49$N(suffix) {} {
    global N
    set h $N(ht_X)
    set s $N(serif)
    set wd $N(2serif)
    set em10 $N(em10)

    set aw [expr $em10 + $wd]

    if $s {
        S_ 49 $aw 0   0 0 $wd $h   3 3
    } {
        S_ 49 $aw 0   0 0 $wd $h   1 1
    }
    $N(prefix)I$N(suffix) 0
    E_
}

# J
proc $N(prefix)@4a$N(suffix) {} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em50)
    set wd $N(em40)
    set q $N(em10)
    set h1 $N(ht_X)
    set h2 $N(ht_baseOver)

    set w [expr ($wd - $s) / 2]

    set x1 [expr $wd - $s]
    set y1 [expr $h2 + $q]

    S_ 4a $aw 0   0 $h2 $wd $h1   5 2
    moveto $wd $h1; rxlineto -$s2
    moveto $x1 $h1; ylineto $y1; ycurveto $w $h2
    xcurveto 0 $y1; rylineto $s
    E_
}

# K
proc $N(prefix)@4b$N(suffix) {} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em60)
    set wd $N(em50)
    set h1 $N(ht_X)

    set w [expr ($wd - $s) / 2]

    set k 0.3
    set x1 [expr $wd - $s]
    set x3 [expr $x1 - $s]
    set y1 [expr round($k * $h1)]
    set x2 [expr $s + $k * ($x3 - $s)]
    set y2 [expr $y1 + $k * ($h1 - $y1)]

    S_ 4b $aw 0   0 0 $wd $h1   7 7
    moveto 0 0; xlineto $s2
    moveto 0 $h1; xlineto $s2
    moveto $wd 0; rxlineto -$s2
    moveto $x1 $h1; rxlineto -$s2

    moveto $s 0; ylineto $h1
    moveto $s $y1; lineto $x3 $h1
    moveto $x2 $y2; lineto $x1 0
    E_
}

# L
proc $N(prefix)@4c$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em55)
    set wd $N(em45)
    set q $N(em08)
    set h1 $N(ht_X)

    set x1 [expr $wd - $s]

    S_ 4c $aw 0   0 0 $wd $h1   4 3
    moveto 0 0; xlineto $x1; lineto $wd $q
    moveto $s 0; ylineto $h1
    moveto 0 $h1; xlineto $s2
    E_
}

# M
#
proc $N(prefix)@4d$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_X)
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em60)
    set wd $N(em50)

    set w [expr $wd / 2]
    set y1 [expr int(0.2 * $em)]

    S_ 4d $aw 0   0 0 $wd $h   8 5
    moveto 0 0; xlineto $s2; moveto $wd 0; rxlineto -$s2;
    moveto 0 $h; xlineto $s; moveto $wd $h; rxlineto -$s;
    moveto $s 0; ylineto $h; lineto $w $y1; lineto [expr $wd - $s] $h
    ylineto 0
    E_
}

# N
proc $N(prefix)@4e$N(suffix) {} {
    global N
    set s $N(serif)
    set h $N(ht_X)
    set wd $N(em50)
    set aw $N(em60)

    if $s {
        S_ 4e $aw 0   0 0 $wd $h   6 4
    } {
        S_ 4e $aw 0   0 0 $wd $h   3 1
    }
    $N(prefix)N$N(suffix) $wd
    E_
}

# O
#
proc $N(prefix)@4f$N(suffix) {} {
    global N
    set wd $N(em50)
    set aw $N(em60)

    S_ 4f $aw 0   0 $N(ht_baseOver) $wd $N(ht_XOver)   4 2
    $N(prefix)O$N(suffix) $wd
    E_
}

# P
proc $N(prefix)@50$N(suffix) {} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set h $N(ht_X)
    set aw $N(em50)
    set wd $N(em40)

    set w [expr $wd / 2]
    set y1 [expr $h / 2]
    set y2 [expr $h * 3 / 4]

    S_ 50 $aw 0   0 0 $wd $h   6 3
    moveto 0 0; xlineto $s2; moveto $s 0; ylineto $h
    moveto $s $y1; xlineto $w; xcurveto $wd $y2
    ycurveto $w $h; xlineto 0

    E_
}

# Q
#
#   (x1,y1) -- left end of tail
#   (x2,y1) -- intersection of tail and O
#   (x3,y2), (x4,y2) -- apexes of first two conics of tail
#   (x5,y3) -- bottom of tail
#   (x6,y3) -- apex of penultimate conic of tail
#   b -- bulge of conics of O
#   t -- parameter value for start of tail
#
proc $N(prefix)@51$N(suffix) {} {
    global N
    set h $N(ht_XOver)
    set hbo $N(ht_baseOver)
    set aw $N(em60)
    set wd $N(em50)
    set em10 $N(em10)

    set w [expr int($wd / 2.0)]
    set m [expr ($h - $hbo) / 2]

    set b 0.9; set t 0.25; set t1 [expr 1.0 - $t]
    set g1 [expr $b * $t / $t1]; set g2 [expr $b * $t1 / $t]
    set d [expr $g1 + $g2 + 2.0 * $g1 * $g2]
    set u [expr $g2 / $d]; set v [expr $g1 / $d]

    set x1 [expr round($u * $w)]
    set x2 [expr $w + ($w - $x1)]
    set x3 [expr ($x1 + $w) / 2]
    set x4 [expr ($x2 + $w) / 2]
    set x5 [expr $x2 + round(0.7 * ($wd - $x2))]
    set x6 [expr ($x2 + $x5) / 2]

    set y1 [expr $hbo + round($v * ($m - $hbo))]
    set y2 [expr  $em10]
    set y3 [expr -$em10]

    S_ 51 $aw 0   0 $y3 $wd $h   8 3
    moveto $w $hbo; rxcurveto  $w $m $b; rycurveto -$w $m $b
    moveto $w $hbo; rxcurveto -$w $m $b; rycurveto  $w $m $b
    moveto $x1 $y1; curveto $w $y2 $x3 $y2
    curveto $x2 $y1 $x4 $y2
    curveto $x5 $y3 $x6 $y3 1.0
    xcurveto $wd 0 1.0
    E_
}

# R
proc $N(prefix)@52$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set h $N(ht_X)
    set aw $N(em55)
    set wd $N(em45)

    set w [expr $wd - $s]
    set k [expr $em * 0.07]
    set x1 [expr $w / 2]
    set x2 [expr $w - $k]
    set x3 [expr $x1 + 0.4 * ($w - $x1)]
    set x4 [expr $x1 + 0.2 * ($w - $x1)]

    set y1 [expr $h / 2]
    set y3 [expr $h / 4]
    set y2 [expr $y3 * 3]

    S_ 52 $aw 0   0 0 $wd $h   9 4
    moveto 0 0; xlineto $s2; moveto $s 0; ylineto $h
    moveto $s $y1; xlineto $x1; xcurveto $w $y2
    ycurveto $x1 $h; xlineto 0

    moveto $x1 $y1; curveto $x3 $y3 $x4 $y1 1.2
    curveto $w 0 $x2 0; xcurveto $wd $s
    E_
}

# S
proc $N(prefix)@53$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set h $N(ht_XOver)
    set hX $N(ht_X)
    set hbo $N(ht_baseOver)
    set aw $N(em55)
    set wd $N(em45)
    set q $N(em08)
    set em10 $N(em10)
    set em15 $N(em15)

    set x1 [expr $wd / 2]

    set yh [expr $h - $hbo]
    set y1 [expr int(0.2 * $yh)]
    set y2 [expr int(0.55 * $yh)]
    set y3 [expr ($y2 + $hbo) / 2]
    set y4 [expr ($y2 + $h) / 2]

    S_ 53 $aw 0   0 $hbo $wd $h   9 2
    moveto $wd $hX; rylineto -$s2
    ycurveto $x1 $h; xcurveto $s $y4
    curveto $x1 $y2 $s [expr $y4 - $em10] 1.2
    curveto $wd $y3 $wd [expr $y3 + $em15] 1.2
    ycurveto $x1 $hbo; curveto $s $s [expr $x1 - $q] $hbo
    ylineto 0
    moveto $s $s; curveto 0 $y1 $s [expr ($s + $y1) / 2]
    E_
}

# T
proc $N(prefix)@54$N(suffix) {} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set h $N(ht_X)
    set aw $N(em55)
    set wd $N(em45)

    set x1 [expr $wd / 2]
    set x2 [expr $s / 2]
    set y1 [expr $h - $s2]

    S_ 54 $aw 0   0 0 $wd $h   5 3
    moveto $wd $y1; lineto [expr $wd - $x2] $h; xlineto $x2
    lineto 0 $y1
    moveto $x1 0; ylineto $h; moveto [expr $x1 - $s] 0; rxlineto $s2
    E_
}

# U
#
proc $N(prefix)@55$N(suffix) {} {
    global N
    set s $N(serif)
    set h $N(ht_X)
    set wd $N(em40)
    set aw $N(em50)
    set hbo $N(ht_baseOver)

    if $s {
        S_ 55 $aw 0   0 $hbo $wd $h   6 3
    } {
        S_ 55 $aw 0   0 $hbo $wd $h   4 1
    }
    $N(prefix)U$N(suffix) $wd
    E_
}

# V
#
proc $N(prefix)@56$N(suffix) {} {
    global N
    set h $N(ht_X)
    set hbo $N(ht_baseOver)
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em50)
    set wd $N(em40)

    set w [expr $wd / 2]

    S_ 56 $aw 0   0 $hbo $wd $h   4 3
    moveto 0 $h; xlineto $s2; moveto $wd $h; rxlineto -$s2;
    moveto $s $h; lineto $w $hbo; lineto [expr $wd - $s] $h
    E_
}

# W
#
proc $N(prefix)@57$N(suffix) {} {
    global N
    set h $N(ht_X)
    set hbo $N(ht_baseOver)
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em70)
    set wd $N(em60)

    set w [expr $wd / 2]
    set x1 [expr $wd / 4]
    set x2 [expr 3 * $x1]

    S_ 57 $aw 0   0 $hbo $wd $h   7 4
    moveto 0 $h; xlineto $s2; moveto $wd $h; rxlineto -$s2;
    moveto [expr $w - $s] $h; rxlineto $s2
    moveto $s $h; lineto $x1 $hbo; lineto $w $h
    lineto $x2 $hbo; lineto [expr $wd - $s] $h
    E_
}

# X
#
proc $N(prefix)@58$N(suffix) {} {
    global N
    set h $N(ht_X)
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em60)
    set wd $N(em50)

    set w [expr $wd / 2]
    set x1 [expr $wd - $s]

    S_ 58 $aw 0   0 0 $wd $h   6 6
    moveto 0 0; xlineto $s2; moveto $wd 0; rxlineto -$s2;
    moveto 0 $h; xlineto $s2; moveto $wd $h; rxlineto -$s2;
    moveto $s $h; lineto $x1 0; moveto $s 0; lineto $x1 $h
    E_
}

# Y
#
proc $N(prefix)@59$N(suffix) {} {
    global N
    set h $N(ht_X)
    set s $N(serif)
    set wd $N(em40)
    set aw $N(em50)

    if $s {
        S_ 59 $aw 0   0 0 $wd $h   6 5
    } {
        S_ 59 $aw 0   0 0 $wd $h   3 2
    }
    $N(prefix)Y$N(suffix) $wd
    E_
}

# Z
#
proc $N(prefix)@5a$N(suffix) {} {
    global N
    set h $N(ht_X)
    set s $N(serif)
    set aw $N(em50)
    set wd $N(em40)

    set x1 [expr $wd - $s]
    set y1 [expr $h - $s]

    S_ 5a $aw 0   0 0 $wd $h   5 1
    moveto 0 $y1; lineto $s $h; xlineto $x1; lineto 0 0; xlineto $x1
    lineto $wd $s
    E_
}

# a
#  t -- width of tail of 'a'
#  m -- vertical midpoint of bbox
#  w -- horizontal midpoint of bbox excluding tail
#
proc $N(prefix)@61$N(suffix) {} {
    global N
    set aw $N(em50)
    set wd $N(em40)
    set h $N(ht_xOver)
    set bo $N(ht_baseOver)
    set s $N(serif)

    set w  [expr ($wd - $s) / 2]
    set m  [expr round(($h + $bo) / 2.0)]
    set m2 [expr round($m / 2.0)]
    if {0 != $s} {
        set n 8
    } {
        set n 7
    }

    S_ 61 $aw 0   0 $bo $wd $h   $n 2
    $N(prefix)a$N(suffix) $wd $s $w
    E_
}

# b (mirror image of d)
# m -- vertical midpoint of bowl
# w -- horizontal midpoint of bbox
#
proc $N(prefix)@62$N(suffix) {} {
    global N
    set s $N(serif)
    set aw $N(em50)
    set wd $N(em40)
    set q $N(em15)
    set h $N(ht_xOver)
    set t $N(ht_asc)
    set bo $N(ht_baseOver)

    set w [expr int($wd / 2.0)]
    set m [expr ($h - $bo) / 2]

    S_ 62 $aw 0   0 $bo $wd $t   7 2
    moveto 0 0; xlineto $s; ylineto $t; rlineto $s 0
    moveto $s $q; ycurveto $w $bo; xcurveto $wd $m
    ycurveto $w $h
    xcurveto $s [expr $N(ht_x) - $q]
    E_
}

# c
# m -- vertical midpoint of bowl
# w -- horizontal location of top and bottom
# s -- half serif length
#
proc $N(prefix)@63$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set aw $N(em45)
    set wd $N(em35)
    set bo $N(ht_baseOver)

    set k [expr round($em * 0.04)]
    set w [expr int($wd / 2.0) + $k]

    S_ 63 $aw 0   0 $bo $wd $N(ht_xOver)   5 1
    $N(prefix)c$N(suffix) $wd $w $k
    E_
}

# d (mirror image of b)
# m -- vertical midpoint of bowl
# w -- horizontal midpoint of bbox
# s -- half serif length
#
proc $N(prefix)@64$N(suffix) {} {
    global N
    set s $N(serif)
    set aw $N(em50)
    set wd $N(em40)
    set q $N(em15)
    set h $N(ht_xOver)
    set t $N(ht_asc)
    set bo $N(ht_baseOver)

    set w [expr int($wd / 2.0)]
    set m [expr ($h - $bo) / 2]
    set sx [expr $wd - $s]

    S_ 64 $aw 0   0 $bo $wd $t   7 2
    moveto $wd 0; xlineto $sx; ylineto $t; rlineto -$s 0
    moveto $sx $q; ycurveto $w $bo; xcurveto 0 $m
    ycurveto $w $h
    xcurveto $sx [expr $N(ht_x) - $q]
    E_
}

# e
proc $N(prefix)@65$N(suffix) {} {
    global N
    set aw $N(em45)
    set wd $N(em35)

    S_ 65 $aw 0   0 $N(ht_baseOver) $wd $N(ht_xOver)   5 1
    $N(prefix)e$N(suffix) $wd
    E_
}

# f
# b -- length of bar
proc $N(prefix)@66$N(suffix) {} {
    global N
    set em $N(emSize)
    set aw $N(em40)
    set wd $N(em30)
    set q $N(em05)
    set s $N(serif)
    set s2 $N(2serif)
    set h $N(ht_ascOver)
    set m $N(ht_x)


    set b [expr int(5.0 * $wd / 6.0)]
    set x1 [expr int(1.0 * $wd / 3.0)]
    set x2 [expr int(2.0 * $wd / 3.0)]
    set x3 [expr $x1 - $s]

    set y1 [expr $h - $q]
    set y2 [expr $y1 - $q]

    S_ 66 $aw 0   0 0 $wd $h   5 3
    moveto $x3 0; rlineto $s2 0; moveto 0 $m; rlineto $b 0;
    moveto $x1 0; ylineto $y2; ycurveto $x2 $h
    xcurveto $wd $y1
    E_
}

# g
proc $N(prefix)@67$N(suffix) {} {
    global N
    set s $N(serif)
    set aw $N(em50)
    set wd $N(em40)
    set q $N(em05)
    set h $N(ht_xOver)
    set b $N(ht_dscOver)
    set bo $N(ht_baseOver)

    set w [expr int($wd / 2.0)]
    set m [expr ($h - $bo) / 2]
    set x1 [expr $wd - $q]
    set x2 [expr $w - 2 * $q]
    set y1 [expr $h - 2 * $q]
    set y2 [expr $bo - $q]
    set y3 [expr $b + 2 * $q]
    set y4 [expr $bo + 3 * $q]

    S_ 67 $aw 0   0 $b $wd $h   8 2
    moveto $wd $h; lineto $x1 $y1; ylineto $y2
    ycurveto $w $b 1.5; curveto $q $y3 $x2 $b
    moveto $x1 $y1; ycurveto $w $h 0.6; xcurveto 0 $m
    ycurveto $w $bo; xcurveto $x1 $y4
    E_
}

# h
proc $N(prefix)@68$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em50)
    set wd $N(em40)
    set q1 $N(em15)
    set h1 $N(ht_xOver)
    set h2 $N(ht_asc)

    set q2 [expr round(0.03 * $em)]
    set w [expr int($wd / 2.0) + $q2]
    set x1 [expr $wd - $s]
    set x2 [expr $w - $q1]
    set y1 [expr $h1 - $q1]

    S_ 68 $aw 0   0 0 $wd $h2   7 4
    moveto 0 $h2; xlineto $s; ylineto 0
    moveto 0 0; rxlineto $s2; moveto $wd 0; rxlineto -$s2
    moveto $x1 0; ylineto $y1; ycurveto $w $h1 1.6
    curveto $s $y1 $x2 $h1 0.7
    E_
}

# i
proc $N(prefix)@69$N(suffix) {} {
    global N
    set em $N(emSize)
    set t $N(em10)
    set s $N(serif)
    set s2 $N(2serif)
    set r $N(dotRad)
    set h $N(ht_xAcc)

    set r2 [expr 2 * $r]
    set y1 [expr $h + $r]
    set ht [expr $y1 + $r]


    if $s {
        if {$s >= $r2} {
            set wd $s2; set x1 0
        } {
            set wd [expr $r2 + $s]; set x1 [expr $wd - $s2]
        }
        set x2 [expr $wd - $s - $r]; set aw [expr $t + $wd]
        S_ 69 $aw 0   0 0 $wd $ht   7 3
    } {
        set x1 $r; set x2 $r; set wd $r2
        set aw [expr $t + $wd]
        S_ 69 $aw 0   0 0 $wd $ht   5 2
    }
    $N(prefix)dotlessi$N(suffix) $x1
    $N(prefix)dot$N(suffix) $x2 $y1 $r
    E_
}

# j
#  (since there is no serif on the right, advance width needs to be
#  a little larger)
proc $N(prefix)@6a$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set r $N(dotRad)
    set q1 $N(em10)
    set h1 $N(ht_x)
    set h2 $N(ht_xAcc)
    set h3 $N(ht_dscOver)

    set wd [expr round(1.5 * $s)]
    set aw [expr $wd + round(0.13 * $em)]

    set h4 [expr $h3 + $q1]
    set ht [expr $h2 + $r + $r]

    set x1 [expr $wd - $s]

    S_ 6a $aw 0   0 $h3 $wd $ht   7 2
    moveto $x1 $h1; xlineto $wd; ylineto $h4; ycurveto 0 $h3 1.3
    $N(prefix)dot$N(suffix) [expr $wd - $r] [expr $h2 + $r] $r
    E_
}

# k
# q1 -- height of branch point above midpoint of x-height
# x1 -- branch point (x)
#
proc $N(prefix)@6b$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em55)
    set wd $N(em45)
    set h1 $N(ht_asc)
    set h2 $N(ht_x)

    set q1 [expr round(0.04 * $em)]

    set y1 [expr $h2 / 2]
    set y2 [expr $y1 + $q1]

    set x1 [expr round(0.35 * $wd)]
    set x2 [expr $wd - $s]
    set x3 [expr $wd - $s2]

    S_ 6b $aw 0   0 0 $wd $h1   8 6
    moveto 0 $h1; xlineto $s; ylineto 0; moveto 0 0; xlineto $s2
    moveto $wd 0; rxlineto -$s2; moveto $x2 $h2; rxlineto -$s2
    moveto $s $y1; lineto $x1 $y2; lineto $x3 $h2;
    moveto $x1 $y2; lineto $x2 0
    E_
}

# l
#
#
proc $N(prefix)@6c$N(suffix) {} {
    global N
    set em $N(emSize)
    set t $N(em10)
    set s $N(serif)
    set s2 $N(2serif)
    set h1 $N(ht_asc)

    set aw [expr $t + $s2]
    set wd $s2


    S_ 6c $aw 0   0 0 $wd $h1   3 2
    moveto $s 0; ylineto $h1; rxlineto -$s; moveto 0 0; rxlineto $s2;
    E_
}

# m
proc $N(prefix)@6d$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em70)
    set wd $N(em60)
    set q3 $N(em15)
    set h1 $N(ht_xOver)
    set h2 $N(ht_x)

    set q1 [expr round(0.12 * $em)]
    set q2 [expr round(0.03 * $em)]
    set w [expr $wd / 2]

    set x1 [expr $wd - $s]
    set x2 [expr $w - $s]
    set m1 [expr ($w + $s) / 2]
    set m2 [expr ($w + $x1) / 2]
    set x3 [expr $m1 - $q1]
    set x4 [expr $m1 + $q2]
    set x5 [expr $m2 - $q1]
    set x6 [expr $m2 + $q2]

    set y1 [expr $h1 - $q3]

    S_ 6d $aw 0   0 0 $wd $h1   11 6
    moveto 0 $h2; xlineto $s; ylineto 0
    moveto 0 0; rxlineto $s2; moveto $wd 0; rxlineto -$s2
    moveto $x2 0; rxlineto $s2
    moveto $x1 0; ylineto $y1; ycurveto $x6 $h1 1.6
    curveto $w $y1 $x5 $h1 0.7
    moveto $w 0; ylineto $y1; ycurveto $x4 $h1 1.6
    curveto $s $y1 $x3 $h1 0.7
    E_
}

# n
proc $N(prefix)@6e$N(suffix) {} {
    global N
    set em $N(emSize)
    set wd $N(em40)
    set aw $N(em50)
    set s $N(serif)
    set h $N(ht_xOver)

    if $s {
        S_ 6e $aw 0   0 0 $wd $h   7 4
    } {
        S_ 6e $aw 0   0 0 $wd $h   7 4
    }
    $N(prefix)n$N(suffix) $wd
    E_
}

# o
#
proc $N(prefix)@6f$N(suffix) {} {
    global N
    set aw $N(em45)
    set wd $N(em35)

    S_ 6f $aw 0   0 $N(ht_baseOver) $wd $N(ht_xOver)   4 2
    $N(prefix)o$N(suffix) $wd
    E_
}

# p (similar to b)
# m -- vertical midpoint of bowl
# w -- horizontal midpoint of bbox
# s -- half serif length
#
proc $N(prefix)@70$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em50)
    set wd $N(em40)
    set q $N(em15)
    set h1 $N(ht_xOver)
    set h2 $N(ht_x)
    set b $N(ht_dsc)
    set bo $N(ht_baseOver)

    set w [expr $wd / 2]
    set m [expr ($h1 - $bo) / 2]

    set y1 [expr $h2 - $q]

    S_ 70 $aw 0   0 $b $wd $h1   7 3
    moveto 0 $h2; xlineto $s; ylineto $b
    moveto 0 $b; xlineto $s2
    moveto $s $q; ycurveto $w $bo; xcurveto $wd $m
    ycurveto $w $h1
    xcurveto $s $y1
    E_
}

# q (similar to d)
#  m -- vertical midpoint of bowl
#  w -- horizontal midpoint of bbox
#  s -- half serif length
#
proc $N(prefix)@71$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em50)
    set wd $N(em40)
    set q $N(em15)
    set h1 $N(ht_xOver)
    set h2 $N(ht_x)
    set b $N(ht_dsc)
    set bo $N(ht_baseOver)

    set w [expr int($wd / 2.0)]
    set m [expr ($h1 - $bo) / 2]

    set x1 [expr $wd - $s]
    set y1 [expr $h2 - $q]

    S_ 71 $aw 0   0 $b $wd $h1   7 3
    moveto $wd $b; rxlineto -$s2
    moveto $x1 $b; ylineto $y1; ycurveto $wd $h2 0.8
    moveto $x1 $q; ycurveto $w $bo; xcurveto 0 $m
    ycurveto $w $h1; xcurveto $x1 $y1
    E_
}

# r (similar to n)
proc $N(prefix)@72$N(suffix) {} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em45)
    set wd $N(em35)
    set q1 $N(em15)
    set h1 $N(ht_xOver)
    set h2 $N(ht_x)

    set q2 [expr $q1/2]
    set w [expr ($wd / 2.0) + $s]

    set x1 [expr $wd - $s]
    set x2 [expr ($w + $s)/2]
    set x3 [expr ($w + $wd)/2]
    set y1 [expr $h1 - $q1]
    set y2 [expr $h1 - $q2]

    S_ 72 $aw 0   0 0 $wd $h1   5 3
    moveto 0 $h2; xlineto $s; ylineto 0
    moveto 0 0; rxlineto $s2
    moveto $s $y1; curveto $w $h1 $x2 $h1 1.6
    curveto $wd $y2 $x3 $h1 0.7
    E_
}

# s
proc $N(prefix)@73$N(suffix) {} {
    global N
    set s $N(serif)
    set aw $N(em45)
    set wd $N(em35)
    set q1 $N(em15)
    set h1 $N(ht_xOver)
    set h2 $N(ht_baseOver)

    set w [expr $wd / 2]
    set ss [expr 1.5 * $s]

    set x1 [expr $w - $q1]
    set x2 [expr $w + $q1]
    set x3 [expr $wd - $q1 / 3]
    set y1 [expr $h1 - $s]
    set y2 [expr $h2 + $s]
    set m [expr int(0.55 * $h1)]
    set y4 [expr ($h1 + $m) / 2]
    set y5 [expr ($h2 + $m) / 2]

    S_ 73 $aw 0   0 $h2 $wd $h1   8 3
    moveto 0 $h2; rylineto $ss
    moveto $x3 $h1; rylineto -$ss
    moveto $x3 $y1; curveto $w $h1 $x2 $h1 1.0
    xcurveto 0 $y4
    ycurveto $w $m
    xcurveto $wd $y5
    ycurveto $w 0
    curveto 0 $y2 $x1 $h2 1.0
    E_
}

# t
# b -- length of bar
proc $N(prefix)@74$N(suffix) {} {
    global N
    set s $N(serif)
    set aw $N(em40)
    set wd $N(em30)
    set q $N(em05)
    set h1 $N(ht_ascOver)
    set h2 $N(ht_xOver)
    set h3 $N(ht_baseOver)

    set b [expr int(5.0 * $wd / 6.0)]
    set x1 [expr int(1.0 * $wd / 3.0)]
    set x2 [expr int(2.0 * $wd / 3.0)]
    set x3 [expr $x1 - $s]

    set y1 [expr ($h1 + $h2) / 2]
    set y2 [expr $h3 + $q]
    set y3 [expr $y2 + $q]

    S_ 74 $aw 0   0 $h3 $wd $y1   4 2
    moveto 0 $h2; rxlineto $b
    moveto $x1 $y1; ylineto $y3; ycurveto $x2 $h3
    xcurveto $wd $y2
    E_
}

# u
proc $N(prefix)@75$N(suffix) {} {
    global N
    set s $N(serif)
    set wd $N(em40)
    set aw $N(em50)
    set h1 $N(ht_x)
    set h2 $N(ht_baseOver)

    if $s {
        S_ 75 $aw 0   0 $h2 $wd $h1   7 2
    } {
        S_ 75 $aw 0   0 $h2 $wd $h1   4 2
    }
    $N(prefix)u$N(suffix) $wd
    E_
}

# v
proc $N(prefix)@76$N(suffix) {} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em50)
    set wd $N(em40)
    set h1 $N(ht_baseOver)
    set h2 $N(ht_x)

    set w [expr $wd / 2.0]
    set x1 [expr $wd - $s]

    S_ 76 $aw 0   0 $h1 $wd $h2   4 3
    moveto $wd $h2; rxlineto -$s2; moveto 0 $h2; rxlineto $s2
    moveto $s $h2; lineto $w $h1; lineto $x1 $h2
    E_
}

# w
proc $N(prefix)@77$N(suffix) {} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em70)
    set wd $N(em60)
    set h1 $N(ht_baseOver)
    set h2 $N(ht_x)

    set w [expr $wd / 2.0]
    set x1 [expr $wd - $s]
    set x2 [expr $w - $s]
    set x3 [expr $w / 2]
    set x4 [expr $w + $x3]

    S_ 77 $aw 0   0 $h1 $wd $h2   7 4
    moveto $wd $h2; rxlineto -$s2; moveto 0 $h2; rxlineto $s2
    moveto $x2 $h2; rxlineto $s2
    moveto $s $h2; lineto $x3 $h1; lineto $w $h2
    lineto $x4 $h1; lineto $x1 $h2
    E_
}

# x
proc $N(prefix)@78$N(suffix) {} {
    global N
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em55)
    set wd $N(em45)
    set h1 $N(ht_x)

    set x1 [expr $wd - $s]

    S_ 78 $aw 0   0 0 $wd $h1   6 6
    moveto 0 0; rxlineto $s2;      moveto 0 $h1; rxlineto $s2;
    moveto $wd 0; rxlineto -$s2;  moveto $wd $h1; rxlineto -$s2
    moveto $s 0; lineto $x1 $h1;   moveto $s $h1; lineto $x1 0
    E_
}

# y
proc $N(prefix)@79$N(suffix) {} {
    global N
    set em $N(emSize)
    set aw $N(em55)
    set wd $N(em45)

    S_ 79 $aw 0   0 $N(ht_dsc) $wd $N(ht_x)   5 4
    $N(prefix)y$N(suffix) $wd
    E_
}

# z
proc $N(prefix)@7a$N(suffix) {} {
    global N
    set s $N(serif)
    set aw $N(em45)
    set wd $N(em35)
    set h1 $N(ht_x)

    set w [expr $wd / 2.0]
    set x1 [expr $wd - $s]
    set x2 [expr $wd - ($s / 2)]
    set y1 [expr $h1 - $s]

    S_ 7a $aw 0   0 0 $wd $h1   5 1
    moveto 0 $y1; lineto $s $h1; xlineto $x2; lineto 0 0; xlineto $x1
    lineto $wd $s
    E_
}

#----------  begin experimental samples  -------------------------------
#
# glyph codes from 0x80 to 0x8f are used for experimental samples
#

# Tamil -- thoo
proc $N(prefix)@80$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set hbo $N(ht_baseOver)
    set hx $N(ht_x)
    set hd $N(ht_dsc)

    set aw [expr round(1.1 * $em)]
    set wd $em

    set w [expr $wd * 0.2]
    set x1 [expr $wd - $s]
    set x2 [expr $x1 - $w]
    set x3 [expr $x2  - $s]
    set x4 [expr $x3 - $w]
    set x5 [expr $x4 - $w]
    set x6 [expr $x5 - $w]
    set x7 [expr $x3 / 2]

    set y1 [expr $hd / 2]
    set y2 [expr ($hbo + $hx) / 2]
    set y3 [expr $y2 / 2]

    S_ 80 $aw 0   0 $hd $wd $hx   13 3
    moveto $x1 $y1; ylineto $hx
    moveto $wd $hx; xlineto $x2; ylineto $y1; xlineto 0
    ycurveto $x7 $hd; xcurveto $x3 0; ycurveto $x5 $y2
    xcurveto $x6 $y3; ycurveto $x5 $hbo; xcurveto $x4 $y2
    ylineto $hx
    moveto [expr $x4 + $s] $hx; xlineto $x5; ylineto $y2
    E_
}

#----------  end experimental samples  ---------------------------------

# dotless i
proc $N(prefix)@90$N(suffix) {} {
    global N
    set em $N(emSize)
    set ht $N(ht_x)
    set s $N(serif)
    set s2 $N(2serif)

    set wd $s2
    set aw [expr $N(em10) + $wd]

    if $s {
        S_ 90 $aw 0   0 0 $wd $ht   3 2
    } {
        S_ 90 $aw 0   0 0 $wd $ht   1 1
    }
    $N(prefix)dotlessi$N(suffix) 0
    E_
}

# Agrave
proc $N(prefix)@c0$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set h $N(ht_XAcc)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htGrave)
    set wa $N(wdGrave)

    set t  [expr int(0.2 * $wa)]
    set x1  [expr ($wd / 2) + $t]
    set ht [expr $h + $ha]

    if $s {
        S_ c0 $aw 0   0 0 $wd $ht   6 5
    } {
        S_ c0 $aw 0   0 0 $wd $ht   4 3
    }
    $N(prefix)A$N(suffix) $wd
    $N(prefix)grave$N(suffix) $x1 $h $ha $wa
    E_
}

# Aacute
proc $N(prefix)@c1$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set h $N(ht_XAcc)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htAcute)
    set wa $N(wdAcute)

    set t  [expr int(0.2 * $wa)]
    set x1  [expr ($wd / 2) - $t]
    set ht [expr $h + $ha]

    if $s {
        S_ c1 $aw 0   0 0 $wd $ht   6 5
    } {
        S_ c1 $aw 0   0 0 $wd $ht   4 3
    }
    $N(prefix)A$N(suffix) $wd
    $N(prefix)acute$N(suffix) $x1 $h $ha $wa
    E_
}

# Acircum
proc $N(prefix)@c2$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set h $N(ht_XAcc)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htCircum)
    set wa $N(wdCircum)

    set x1  [expr ($wd - $wa) / 2]
    set ht [expr $h + $ha]

    if $s {
        S_ c2 $aw 0   0 0 $wd $ht   7 5
    } {
        S_ c2 $aw 0   0 0 $wd $ht   5 3
    }
    $N(prefix)A$N(suffix) $wd
    $N(prefix)circum$N(suffix) $x1 $h $ha $wa
    E_
}

# Atilde
proc $N(prefix)@c3$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set h $N(ht_XAcc)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htTilde)
    set wa $N(wdTilde)

    set x1  [expr ($wd - $wa) / 2]
    set ht [expr $h + $ha]

    if $s {
        S_ c3 $aw 0   0 0 $wd $ht   9 5
    } {
        S_ c3 $aw 0   0 0 $wd $ht   7 3
    }
    $N(prefix)A$N(suffix) $wd
    $N(prefix)tilde$N(suffix) $x1 $h $ha $wa
    E_
}

# Adieresis
proc $N(prefix)@c4$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set r $N(dotRad)
    set h $N(ht_XAcc)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htDieresis)
    set wa $N(wdDieresis)

    set x1 [expr ($wd - $wa) / 2]
    set y1 [expr $h + $r]
    set ht [expr $h + $ha]

    if $s {
        S_ c4 $aw 0   0 0 $wd $ht   13 6
    } {
        S_ c4 $aw 0   0 0 $wd $ht   11 4
    }
    $N(prefix)A$N(suffix) $wd
    $N(prefix)dieresis$N(suffix) $x1 $y1 $r $wa
    E_
}

# Aring
proc $N(prefix)@c5$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set r $N(ringRad)
    set h $N(ht_XAcc)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htRing)
    set wa $N(wdRing)

    set x1 [expr $wd / 2]
    set y1 [expr $h + $r]
    set ht [expr $h + $ha]

    if $s {
        S_ c5 $aw 0   0 0 $wd $ht   9 5
    } {
        S_ c5 $aw 0   0 0 $wd $ht   7 3
    }
    $N(prefix)A$N(suffix) $wd
    $N(prefix)ring$N(suffix) $x1 $y1 $r
    E_
}

# AE ligature (do later)
# proc $N(prefix)@c6$N(suffix) {} {
# }

# Ccedilla
proc $N(prefix)@c7$N(suffix) {} {
    global N
    set em $N(emSize)
    set wd $N(em50)
    set aw $N(em60)
    set s $N(serif)
    set h1 $N(ht_XOver)
    set h2 $N(ht_baseOver)

    set ha $N(htCedilla)
    set wa $N(wdCedilla)

    set xm [expr $wd / 2]
    set h3 [expr $h2 - $ha]

    if $s {
        S_ c7 $aw 0   0 $h3 $wd $h1   8 3
    } {
        S_ c7 $aw 0   0 $h3 $wd $h1   7 3
    }
    $N(prefix)C$N(suffix) $wd
    $N(prefix)cedilla$N(suffix) $xm $h2 $ha $wa
    E_
}

# Egrave
proc $N(prefix)@c8$N(suffix) {} {
    global N
    set em $N(emSize)
    set wd $N(em50)
    set aw $N(em60)
    set s $N(serif)
    set h $N(ht_XAcc)

    set ha $N(htGrave)
    set wa $N(wdGrave)

    set t  [expr int(0.2 * $wa)]
    set w  [expr ($wd / 2) + $t]
    set ht [expr $h + $ha]

    if $s {
        S_ c8 $aw 0   0 0 $wd $ht   8 6
    } {
        S_ c8 $aw 0   0 0 $wd $ht   5 5
    }
    $N(prefix)E$N(suffix) $wd
    $N(prefix)grave$N(suffix) $w $h $ha $wa
    E_
}

# Eacute
proc $N(prefix)@c9$N(suffix) {} {
    global N
    set em $N(emSize)
    set wd $N(em50)
    set aw $N(em60)
    set s $N(serif)
    set h $N(ht_XAcc)

    set ha $N(htAcute)
    set wa $N(wdAcute)

    set t  [expr int(0.2 * $wa)]
    set w  [expr ($wd / 2) - $t]
    set ht [expr $h + $ha]

    if $s {
        S_ c9 $aw 0   0 0 $wd $ht   8 6
    } {
        S_ c9 $aw 0   0 0 $wd $ht   5 5
    }
    $N(prefix)E$N(suffix) $wd
    $N(prefix)acute$N(suffix) $w $h $ha $wa
    E_
}

# Ecircum
proc $N(prefix)@ca$N(suffix) {} {
    global N
    set em $N(emSize)
    set wd $N(em50)
    set aw $N(em60)
    set s $N(serif)
    set h $N(ht_XAcc)

    set ha $N(htCircum)
    set wa $N(wdCircum)

    set w  [expr ($wd - $wa) / 2]
    set ht [expr $h + $ha]

    if $s {
        S_ ca $aw 0   0 0 $wd $ht   9 6
    } {
        S_ ca $aw 0   0 0 $wd $ht   6 5
    }
    $N(prefix)E$N(suffix) $wd
    $N(prefix)circum$N(suffix) $w $h $ha $wa
    E_
}

# Edieresis
proc $N(prefix)@cb$N(suffix) {} {
    global N
    set em $N(emSize)
    set wd $N(em50)
    set aw $N(em60)
    set s $N(serif)
    set h $N(ht_XAcc)
    set r $N(dotRad)

    set ha $N(htDieresis)
    set wa $N(wdDieresis)

    set x1 [expr ($wd - $wa) / 2]
    set h1 [expr $h + $r]
    set ht [expr $h + $ha]

    if $s {
        S_ cb $aw 0   0 0 $wd $ht   15 7
    } {
        S_ cb $aw 0   0 0 $wd $ht   12 6
    }
    $N(prefix)E$N(suffix) $wd
    $N(prefix)dieresis$N(suffix) $x1 $h1 $r $wa
    E_
}

# Igrave
#    q -- fraction of accent left of center
proc $N(prefix)@cc$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set h $N(ht_XAcc)

    set ha $N(htGrave)
    set wa $N(wdGrave)

    set ht [expr $h + $ha]
    set q 0.8
    set la [expr int($q * $wa)]
    set ra [expr int((1.0 - $q) * $wa)]

    if $s {
        if {$s >= $la} {
            set wd $s2; set x1 0; set x2 [expr $s + $ra]
        } elseif {$s >= $ra} {
            set wd [expr $wa + $s - $ra]; set x1 [expr $wd - $s2]
            set x2 $wa
        } {
            set wd $wa; set x1 [expr $la - $s]; set x2 $wd
        }
        set aw [expr $N(em10) + $wd]
        S_ cc $aw 0   0 0 $wd $ht   4 4
        $N(prefix)I$N(suffix) $x1
        $N(prefix)grave$N(suffix) $x2 $h $ha $wa
    } {
        S_ cc $aw 0   0 0 $wa $ht   2 2
        $N(prefix)I$N(suffix) $la
        $N(prefix)grave$N(suffix) $wa $h $ha $wa
    }
    E_
}

# Iacute
proc $N(prefix)@cd$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set h $N(ht_XAcc)

    set ha $N(htAcute)
    set wa $N(wdAcute)

    set ht [expr $h + $ha]

    set q 0.8
    set ra [expr int($q * $wa)]
    set la [expr int((1.0 - $q) * $wa)]

    if $s {
        if {$s >= $ra} {
            set wd $s2; set x1 0; set x2 [expr $s - $la]
        } elseif {$s >= $la} {
            set wd [expr $wa + $s - $la]; set x1 0
            set x2 [expr $s - $la]
        } {
            set wd $wa; set x1 [expr $la - $s]; set x2 0
        }
        set aw [expr $N(em10) + $wd]
        S_ cd $aw 0   0 0 $wd $ht   4 4
        $N(prefix)I$N(suffix) $x1
        $N(prefix)acute$N(suffix) $x2 $h $ha $wa
    } {
        S_ cd $aw 0   0 0 $wa $ht   2 2
        $N(prefix)I$N(suffix) $la
        $N(prefix)acute$N(suffix) 0 $h $ha $wa
    }
    E_
}

# Icircum
proc $N(prefix)@ce$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set h $N(ht_XAcc)

    set ha $N(htCircum)
    set wa $N(wdCircum)

    set ht [expr $h + $ha]

    if $s {
        if {$wa < $s2} {
            set wd $s2; set xa [expr ($wd - $wa) / 2]; set xI 0
        } else {
            set wd $wa; set xa 0; set xI [expr ($wd / 2)  - $s]
        }
        set aw [expr $N(em10) + $wd]
        S_ ce $aw 0   0 0 $wd $ht   5 4
    } {
        set wd $wa; set xI [expr $wd / 2]; set xa 0
        set aw [expr $N(em10) + $wd]
        S_ ce $aw 0   0 0 $wd $ht   3 2
    }
    $N(prefix)I$N(suffix) $xI
    $N(prefix)circum$N(suffix) $xa $h $ha $wa
    E_
}

# Idieresis
proc $N(prefix)@cf$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set h $N(ht_XAcc)
    set r $N(dotRad)

    set ha $N(htDieresis)
    set wa $N(wdDieresis)

    set ht [expr $h + $ha]
    set h1 [expr $h + $r]

    if $s {
        if {$wa < $s2} {
            set wd $s2; set xa [expr $r + ($wd - $wa) / 2]; set xI 0
        } else {
            set wd $wa; set xa $r; set xI [expr ($wd / 2)  - $s]
        }
        set aw [expr $N(em10) + $wd]
        S_ cf $aw 0   0 0 $wd $ht   11 5
    } {
        set wd $wa; set xa $r; set xI [expr $wd / 2]
        set aw [expr $N(em10) + $wd]
        S_ cf $aw 0   0 0 $wd $ht   9 3
    }
    $N(prefix)I$N(suffix) $xI
    $N(prefix)dieresis$N(suffix) $xa $h1 $r [expr $wa - 2 * $r]
    E_
}

# Eth
#   x1 -- left end of D
#   x2 -- left end of dash
proc $N(prefix)@d0$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set h $N(ht_X)
    set w1 $N(em50)

    set w2 [expr int(0.05 * $em)]

    if $s {
        if {$s >= $w2} {
            set x1 0; set wd $w1; set x2 [expr $s - $w2]
        } {
            set x1 [expr $w2 - $s]; set wd [expr $w1 + $x1]; set x2 0
        }
    } {
        set wd [expr $w1 + $w2]; set x1 $w2; set x2 0
    }
    set aw [expr $wd + $N(em10)]
    set y [expr $h / 2]

    S_ d0 $aw 0   0 0 $wd $h   6 3
    $N(prefix)D$N(suffix) $w1 $x1
    moveto $x2 $y; rxlineto [expr 2 * $w2]
    E_
}

# Ntilde
proc $N(prefix)@d1$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set h $N(ht_XAcc)
    set wd $N(em50)
    set aw $N(em60)

    set ha $N(htTilde)
    set wa $N(wdTilde)

    set w  [expr ($wd - $wa) / 2]
    set ht [expr $h + $ha]

    if $s {
        S_ d1 $aw 0   0 0 $wd $ht   10 5
    } {
        S_ d1 $aw 0   0 0 $wd $ht   7 2
    }
    $N(prefix)N$N(suffix) $wd
    $N(prefix)tilde$N(suffix) $w $h $ha $wa
    E_
}

# Ograve
#
proc $N(prefix)@d2$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_XAcc)
    set wd $N(em50)
    set aw $N(em60)

    set ha $N(htGrave)
    set wa $N(wdGrave)

    set t  [expr round($em / 15.0)]
    set w  [expr ($wd / 2) + $t]
    set ht [expr $h + $ha]

    S_ d2 $aw 0   0 $N(ht_baseOver) $wd $ht   5 3
    $N(prefix)O$N(suffix) $wd
    $N(prefix)grave$N(suffix) $w $h $ha $wa
    E_
}

# Oacute
#
proc $N(prefix)@d3$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_XAcc)
    set wd $N(em50)
    set aw $N(em60)

    set ha $N(htAcute)
    set wa $N(wdAcute)

    set t  [expr round($em / 15.0)]
    set w  [expr ($wd / 2) - $t]
    set ht [expr $h + $ha]

    S_ d3 $aw 0   0 $N(ht_baseOver) $wd $ht   5 3
    $N(prefix)O$N(suffix) $wd
    $N(prefix)acute$N(suffix) $w $h $ha $wa
    E_
}

# Ocircum
#
proc $N(prefix)@d4$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_XAcc)
    set wd $N(em50)
    set aw $N(em60)

    set ha $N(htCircum)
    set wa $N(wdCircum)

    set w  [expr ($wd - $wa) / 2]
    set ht [expr $h + $ha]

    S_ d4 $aw 0   0 $N(ht_baseOver) $wd $ht   6 3
    $N(prefix)O$N(suffix) $wd
    $N(prefix)circum$N(suffix) $w $h $ha $wa
    E_
}

# Otilde
#
proc $N(prefix)@d5$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_XAcc)
    set wd $N(em50)
    set aw $N(em60)

    set ha $N(htTilde)
    set wa $N(wdTilde)

    set w  [expr ($wd - $wa) / 2]
    set ht [expr $h + $ha]

    S_ d5 $aw 0   0 $N(ht_baseOver) $wd $ht   8 3
    $N(prefix)O$N(suffix) $wd
    $N(prefix)tilde$N(suffix) $w $h $ha $wa
    E_
}

# Odieresis
#
proc $N(prefix)@d6$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_XAcc)
    set r $N(dotRad)
    set wd $N(em50)
    set aw $N(em60)

    set ha $N(htDieresis)
    set wa $N(wdDieresis)

    set x1 [expr ($wd - $wa) / 2]
    set h1 [expr $h + $r]
    set ht [expr $h + $ha]

    S_ d6 $aw 0   0 $N(ht_baseOver) $wd $ht   12 4
    $N(prefix)O$N(suffix) $wd
    $N(prefix)dieresis$N(suffix) $x1 $h1 $r $wa
    E_
}

# Oslash
#
proc $N(prefix)@d8$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_XOver)
    set hbo $N(ht_baseOver)
    set wd $N(em50)
    set aw $N(em60)

    S_ d8 $aw 0   0 $hbo $wd $h   5 3
    $N(prefix)O$N(suffix) $wd
    moveto 0 $hbo; lineto $wd $h
    E_
}

# Ugrave
#
proc $N(prefix)@d9$N(suffix) {} {
    global N
    set s $N(serif)
    set hbo $N(ht_baseOver)
    set h $N(ht_XAcc)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htGrave)
    set wa $N(wdGrave)

    set t  [expr int(0.2 * $wa)]
    set w  [expr ($wd / 2) + $t]
    set ht [expr $h + $ha]

    if $s {
        S_ d9 $aw 0   0 $hbo $wd $ht   7 4
    } {
        S_ d9 $aw 0   0 $hbo $wd $ht   5 2
    }
    $N(prefix)U$N(suffix) $wd
    $N(prefix)grave$N(suffix) $w $h $ha $wa
    E_
}

# Uacute
#
proc $N(prefix)@da$N(suffix) {} {
    global N
    set s $N(serif)
    set hbo $N(ht_baseOver)
    set h $N(ht_XAcc)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htAcute)
    set wa $N(wdAcute)

    set t  [expr int(0.2 * $wa)]
    set w  [expr ($wd / 2) - $t]
    set ht [expr $h + $ha]

    if $s {
        S_ da $aw 0   0 $hbo $wd $ht   7 4
    } {
        S_ da $aw 0   0 $hbo $wd $ht   5 2
    }
    $N(prefix)U$N(suffix) $wd
    $N(prefix)acute$N(suffix) $w $h $ha $wa
    E_
}

# Ucircum
#
proc $N(prefix)@db$N(suffix) {} {
    global N
    set s $N(serif)
    set hbo $N(ht_baseOver)
    set h $N(ht_XAcc)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htCircum)
    set wa $N(wdCircum)

    set w  [expr ($wd - $wa) / 2]
    set ht [expr $h + $ha]

    if $s {
        S_ db $aw 0   0 $hbo $wd $ht   8 4
    } {
        S_ db $aw 0   0 $hbo $wd $ht   6 2
    }
    $N(prefix)U$N(suffix) $wd
    $N(prefix)circum$N(suffix) $w $h $ha $wa
    E_
}

# Udieresis
#
proc $N(prefix)@dc$N(suffix) {} {
    global N
    set s $N(serif)
    set hbo $N(ht_baseOver)
    set h $N(ht_XAcc)
    set r $N(dotRad)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htDieresis)
    set wa $N(wdDieresis)

    set x1 [expr ($wd - $wa) / 2]
    set h1 [expr $h + $r]
    set ht [expr $h + $ha]

    if $s {
        S_ dc $aw 0   0 $hbo $wd $ht   14 5
    } {
        S_ dc $aw 0   0 $hbo $wd $ht   12 3
    }
    $N(prefix)U$N(suffix) $wd
    $N(prefix)dieresis$N(suffix) $x1 $h1 $r $wa
    E_
}

# Yacute
#
proc $N(prefix)@dd$N(suffix) {} {
    global N
    set h $N(ht_XAcc)
    set wd $N(em40)
    set aw $N(em50)
    set s $N(serif)

    set ha $N(htAcute)
    set wa $N(wdAcute)

    set t  [expr int(0.2 * $wa)]
    set w  [expr ($wd / 2) - $t]
    set ht [expr $h + $ha]

    if $s {
        S_ dd $aw 0   0 0 $wd $ht   7 6
    } {
        S_ dd $aw 0   0 0 $wd $ht   4 3
    }
    $N(prefix)Y$N(suffix) $wd
    $N(prefix)acute$N(suffix) $w $h $ha $wa
    E_
}

# Thorn
#
proc $N(prefix)@de$N(suffix) {} {
    global N
    set h $N(ht_X)
    set s $N(serif)
    set s2 $N(2serif)
    set wd $N(em40)
    set aw $N(em50)

    set y1 [expr $h / 4]
    set y2 [expr $h / 2]
    set y3 [expr $h - $y1]

    S_ de $aw 0   0 0 $wd $h   5 4
    moveto 0 0; xlineto $s2; moveto 0 $h; xlineto $s2
    moveto $s 0; ylineto $h
    moveto $s $y3; xcurveto $wd $y2; ycurveto $s $y1
    E_
}

# germandbls
#
proc $N(prefix)@df$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_XOver)
    set s $N(serif)
    set s2 $N(2serif)
    set wd $N(em50)
    set aw $N(em60)

    set x1 [expr int(0.5 * $wd)]
    set x2 [expr int(0.8 * $wd)]
    set x3 [expr int(0.6 * $wd)]
    set x4 [expr int(0.2 * $wd)]

    set k [expr $em / 12]
    set y0 [expr int(0.1 * $h)]
    set y1 [expr int(0.15 * $h)]
    set y3 [expr int(0.55 * $h)]
    set y2 [expr $y3 - $k]
    set y4 [expr $y3 + $k]
    set y5 [expr int(0.8 * $h)]
    set y6 [expr ($y4 + $h) / 2]

    S_ df $aw 0   0 0 $wd $h   10 3
    moveto 0 0; xlineto $s2; moveto 0 $y5; xlineto $s
    moveto $s 0; ylineto $y5

    ycurveto $x1 $h; xcurveto $x2 $y6;
    curveto $x3 $y4 $x2 [expr ($y4 + $y6) / 2];
    curveto $x3 $y2 $x4 $y3 0.8
    curveto $wd $y1 $wd [expr ($y1 + $y2) / 2] 1.2;
    ycurveto $x3 0; xcurveto $s2 $y1 0.5
    E_
}

# agrave
proc $N(prefix)@e0$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_xAcc)
    set hbo $N(ht_baseOver)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htGrave)
    set wa $N(wdGrave)

    set t1  $N(serif)
    set t2  [expr int(0.2 * $wa)]
    set w   [expr ($wd - $t1) / 2]
    set x1  [expr $w + $t2]
    set ht  [expr $h + $ha]

    S_ e0 $aw 0   0 $hbo $wd $ht   9 3
    $N(prefix)a$N(suffix) $wd $t1 $w
    $N(prefix)grave$N(suffix) $x1 $h $ha $wa
    E_
}

# aacute
proc $N(prefix)@e1$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_xAcc)
    set hbo $N(ht_baseOver)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htAcute)
    set wa $N(wdAcute)

    set t1  $N(serif)
    set t2  [expr int(0.2 * $wa)]
    set w  [expr ($wd - $t1) / 2]
    set x1 [expr $w - $t2]
    set ht [expr $h + $ha]

    S_ e1 $aw 0   0 $hbo $wd $ht   9 3
    $N(prefix)a$N(suffix) $wd $t1 $w
    $N(prefix)acute$N(suffix) $x1 $h $ha $wa
    E_
}

# acircum
proc $N(prefix)@e2$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_xAcc)
    set hbo $N(ht_baseOver)
    set wd $N(em40)
    set aw $N(em50)

    set ha $N(htCircum)
    set wa $N(wdCircum)

    set t  $N(serif)
    set w  [expr ($wd - $t) / 2]
    set ht [expr $h + $ha]

    S_ e2 $aw 0   0 $hbo $wd $ht   10 3
    $N(prefix)a$N(suffix) $wd $t $w
    $N(prefix)circum$N(suffix) [expr $w - ($wa / 2)] $h $ha $wa
    E_
}

# atilde
proc $N(prefix)@e3$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_xAcc)
    set hbo $N(ht_baseOver)
    set ha $N(htTilde)
    set wa $N(wdTilde)
    set aw $N(em50)
    set wd $N(em40)

    set t  $N(serif)
    set w  [expr ($wd - $t) / 2]
    set ht [expr $h + $ha]

    S_ e3 $aw 0   0 $hbo $wd $ht   12 3
    $N(prefix)a$N(suffix) $wd $t $w
    $N(prefix)tilde$N(suffix) [expr $w - ($wa / 2)] $h $ha $wa
    E_
}

# aDieresis
proc $N(prefix)@e4$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_xAcc)
    set hbo $N(ht_baseOver)
    set r $N(dotRad)
    set ha $N(htDieresis)
    set wa $N(wdDieresis)
    set aw $N(em50)
    set wd $N(em40)

    set t  $N(serif)
    set w  [expr ($wd - $t) / 2]
    set ht [expr $h + $ha]
    set y1 [expr $h + $r]

    S_ e4 $aw 0   0 $hbo $wd $ht   16 4
    $N(prefix)a$N(suffix) $wd $t $w
    $N(prefix)dieresis$N(suffix) [expr ($wd - $wa) / 2] $y1 $r $wa
    E_
}

# aRing
proc $N(prefix)@e5$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_xAcc)
    set hbo $N(ht_baseOver)
    set r $N(ringRad)
    set ha $N(htRing)
    set wa $N(wdRing)
    set aw $N(em50)
    set wd $N(em40)

    set t  $N(serif)
    set w  [expr ($wd - $t) / 2]
    set ht [expr $h + $ha]
    set y1 [expr $h + $r]

    S_ e5 $aw 0   0 $hbo $wd $ht   12 3
    $N(prefix)a$N(suffix) $wd $t $w
    $N(prefix)ring$N(suffix) $w $y1 $r
    E_
}

# ae ligature (do later)
# proc $N(prefix)@e6$N(suffix) {} {
# }

# ccedilla
proc $N(prefix)@e7$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set aw $N(em45)
    set wd $N(em35)
    set ha $N(htCedilla)
    set wa $N(wdCedilla)

    set k [expr round($N(emSize) * 0.04)]
    set w [expr int($wd / 2.0) + $k]
    set bo $N(ht_baseOver)
    set b [expr $bo - $ha]

    S_ e7 $aw 0   0 $b $wd $N(ht_xOver)   8 2
    $N(prefix)c$N(suffix) $wd $w $k
    $N(prefix)cedilla$N(suffix) $w $bo $ha $wa
    E_
}

# egrave
proc $N(prefix)@e8$N(suffix) {} {
    global N
    set em $N(emSize)
    set ha $N(htGrave)
    set wa $N(wdGrave)
    set aw $N(em45)
    set wd $N(em35)

    set w [expr $wd / 2]
    set x1 [expr $w + $w / 4]
    set h $N(ht_xAcc)
    set bo $N(ht_baseOver)

    S_ e8 $aw 0   0 $bo $wd [expr $h + $ha]   6 2
    $N(prefix)e$N(suffix) $wd
    $N(prefix)grave$N(suffix) $x1 $h $ha $wa
    E_
}

# eacute
proc $N(prefix)@e9$N(suffix) {} {
    global N
    set em $N(emSize)
    set ha $N(htAcute)
    set wa $N(wdAcute)
    set aw $N(em45)
    set wd $N(em35)

    set w [expr $wd / 2]
    set x1 [expr $w - $w / 4]
    set h $N(ht_xAcc)
    set bo $N(ht_baseOver)

    S_ e9 $aw 0   0 $bo $wd [expr $h + $ha]   6 2
    $N(prefix)e$N(suffix) $wd
    $N(prefix)acute$N(suffix) $x1 $h $ha $wa
    E_
}

# ecircum
proc $N(prefix)@ea$N(suffix) {} {
    global N
    set em $N(emSize)
    set ha $N(htCircum)
    set wa $N(wdCircum)
    set aw $N(em45)
    set wd $N(em35)

    set x1 [expr ($wd - $wa) / 2]
    set h $N(ht_xAcc)
    set bo $N(ht_baseOver)

    S_ ea $aw 0   0 $bo $wd [expr $h + $ha]   7 2
    $N(prefix)e$N(suffix) $wd
    $N(prefix)circum$N(suffix) $x1 $h $ha $wa
    E_
}

# edieresis
proc $N(prefix)@eb$N(suffix) {} {
    global N
    set em $N(emSize)
    set r $N(dotRad)
    set ha $N(htDieresis)
    set wa $N(wdDieresis)
    set aw $N(em45)
    set wd $N(em35)

    set x1 [expr ($wd - $wa) / 2]
    set h $N(ht_xAcc)
    set h1 [expr $h + $r]
    set bo $N(ht_baseOver)

    S_ eb $aw 0   0 $bo $wd [expr $h + $ha]   13 3
    $N(prefix)e$N(suffix) $wd
    $N(prefix)dieresis$N(suffix) $x1 $h1 $r $wa
    E_
}

# igrave
#   xd -- distance from right end to accent (bottom)
#   w1 -- wdth of glyph at accent
#   x1 -- left end of dotless i
#   x2 -- accent position
#
proc $N(prefix)@ec$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set ha $N(htGrave)
    set wa $N(wdGrave)

    set h1 $N(ht_xAcc)
    set ht [expr $h1 + $ha]
    set xd [expr round( $s * 0.75 )]
    set w1 [expr $wa + $xd]
    if {$w1 < $s2} {
        set wd $s2
        set x1 0
    } else {
        set wd $w1
        set x1 [expr $w1 - $s2]
    }
    set aw [expr round(0.1 * $em) + $wd]
    set x2 [expr $wd - $xd]

    S_ ec $aw 0   0 0 $wd $ht   4 3
    $N(prefix)dotlessi$N(suffix) $x1
    $N(prefix)grave$N(suffix) $x2 $h1 $ha $wa
    E_
}

# iacute
#   xd -- distance from left end to accent (bottom)
#   w1 -- wdth of glyph at accent
#
proc $N(prefix)@ed$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set ha $N(htAcute)
    set wa $N(wdAcute)

    set h1 $N(ht_xAcc)
    set ht [expr $h1 + $ha]
    set xd [expr round( $s * 0.75 )]
    set w1 [expr $wa + $xd]
    if {$w1 < $s2} {
        set wd $s2
    } else {
        set wd $w1
    }
    set aw [expr round(0.1 * $em) + $wd]

    S_ ed $aw 0   0 0 $wd $ht   4 3
    $N(prefix)dotlessi$N(suffix) 0
    $N(prefix)acute$N(suffix) $xd $h1 $ha $wa
    E_
}

# icircum
#   x1 -- left end of accent
#   x2 -- left end of base
#
proc $N(prefix)@ee$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set ha $N(htCircum)
    set wa $N(wdCircum)

    set h1 $N(ht_xAcc)
    set ht [expr $h1 + $ha]
    if {$wa < $s2} {
        set wd $s2
        set x1 [expr ($wd - $wa) / 2]
        set x2 0
    } else {
        set wd $wa
        set x1 0
        set x2 [expr ($wd / 2)  - $s]
    }
    set aw [expr round(0.1 * $em) + $wd]

    S_ ee $aw 0   0 0 $wd $ht   5 3
    $N(prefix)dotlessi$N(suffix) $x2
    $N(prefix)circum$N(suffix) $x1 $h1 $ha $wa
    E_
}

# idieresis
#   x1 -- left end of accent
#   x2 -- left end of base
#
proc $N(prefix)@ef$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set r $N(dotRad)
    set ha $N(htDieresis)
    set wa $N(wdDieresis)

    set h1 $N(ht_xAcc)
    set h2 [expr $h1 + $r]
    set ht [expr $h1 + $ha]
    if {$wa < $s2} {
        set wd $s2
        set x1 [expr $r + ($wd - $wa) / 2]
        set x2 0
    } else {
        set wd $wa
        set x1 $r
        set x2 [expr ($wd / 2)  - $s]
    }
    set aw [expr round(0.1 * $em) + $wd]


    S_ ef $aw 0   0 0 $wd $ht   11 4
    $N(prefix)dotlessi$N(suffix) $x2
    $N(prefix)dieresis$N(suffix) $x1 $h2 $r [expr $wa - $r - $r]
    E_
}

# eth
# aw -- advance wdth
# wd -- bbox wdth
# m -- vertical midpoint of bbox
# w -- horizontal midpoint of bbox
#
proc $N(prefix)@f0$N(suffix) {} {
    global N
    set em $N(emSize)
    set aw $N(em45)
    set wd $N(em35)
    set k1 $N(em10)
    set k2 $N(em15)
    set bo $N(ht_baseOver)
    set h $N(ht_xOver)

    set w [expr int($wd / 2.0)]
    set h1 [expr $h + $k1 + $k1]
    set h2 [expr $h1 - $k2]
    set m [expr ($h - $bo) / 2]

    S_ f0 $aw 0   0 $bo $wd $h1   5 3
    moveto $w $bo
    rxcurveto -$w $m
    rycurveto  $w $m
    moveto $w $bo
    rxcurveto  $w $m
    curveto 0 $h1 $wd $h
    moveto $k2 $h1
    lineto 0 $h2
    E_
}

# ntilde
proc $N(prefix)@f1$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_xAcc)
    set s $N(serif)
    set wd $N(em40)
    set aw $N(em50)
    set ha $N(htTilde)
    set wa $N(wdTilde)

    set ht [expr $h + $ha]

    if $s {
        S_ f1 $aw 0   0 0 $wd $ht   11 5
    } {
        S_ f1 $aw 0   0 0 $wd $ht   8 2
    }
    $N(prefix)n$N(suffix) $wd
    $N(prefix)tilde$N(suffix) [expr ($wd - $wa) / 2] $h $ha $wa
    E_
}

# o grave
#
proc $N(prefix)@f2$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_xAcc)
    set ha $N(htGrave)
    set wa $N(wdGrave)
    set aw $N(em45)
    set wd $N(em35)
    set bo $N(ht_baseOver)

    set t  [expr round($em / 15.0)]
    set w  [expr ($wd / 2) + $t]
    set ht [expr $h + $ha]

    S_ f2 $aw 0   0 $bo $wd $ht   5 3
    $N(prefix)o$N(suffix) $wd
    $N(prefix)grave$N(suffix) $w $h $ha $wa
    E_
}

# o acute
#
proc $N(prefix)@f3$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_xAcc)
    set ha $N(htAcute)
    set wa $N(wdAcute)
    set aw $N(em45)
    set wd $N(em35)
    set bo $N(ht_baseOver)

    set t  [expr round($em / 15.0)]
    set w  [expr ($wd / 2) - $t]
    set ht [expr $h + $ha]

    S_ f3 $aw 0   0 $bo $wd $ht   5 3
    $N(prefix)o$N(suffix) $wd
    $N(prefix)acute$N(suffix) $w $h $ha $wa
    E_
}

# o circum
#
proc $N(prefix)@f4$N(suffix) {} {
    global N
    set h $N(ht_xAcc)
    set ha $N(htCircum)
    set wa $N(wdCircum)
    set aw $N(em45)
    set wd $N(em35)
    set bo $N(ht_baseOver)

    set w  [expr ($wd - $wa) / 2]
    set ht [expr $h + $ha]

    S_ f4 $aw 0   0 $bo $wd $ht   6 3
    $N(prefix)o$N(suffix) $wd
    $N(prefix)circum$N(suffix) $w $h $ha $wa
    E_
}

# o tilde
#
proc $N(prefix)@f5$N(suffix) {} {
    global N
    set h $N(ht_xAcc)
    set ha $N(htTilde)
    set wa $N(wdTilde)
    set aw $N(em45)
    set wd $N(em35)
    set bo $N(ht_baseOver)

    set w  [expr ($wd - $wa) / 2]
    set ht [expr $h + $ha]

    S_ f5 $aw 0   0 $bo $wd $ht   8 3
    $N(prefix)o$N(suffix) $wd
    $N(prefix)tilde$N(suffix) $w $h $ha $wa
    E_
}

# o dieresis
#
proc $N(prefix)@f6$N(suffix) {} {
    global N
    set h $N(ht_xAcc)
    set r $N(dotRad)
    set ha $N(htDieresis)
    set wa $N(wdDieresis)
    set aw $N(em45)
    set wd $N(em35)
    set bo $N(ht_baseOver)

    set h1 [expr $h + $r]
    set ht [expr $h + $ha]
    set x1 [expr $r + ($wd - $wa) / 2]
    set ht [expr $h + $ha]

    S_ f6 $aw 0   0 $bo $wd $ht   12 4
    $N(prefix)o$N(suffix) $wd
    $N(prefix)dieresis$N(suffix) $x1 $h1 $r [expr $wa - $r - $r]
    E_
}

# divide (do later)
# proc $N(prefix)@f7$N(suffix) {} {
# }

# oslash
#
proc $N(prefix)@f8$N(suffix) {} {
    global N
    set h $N(ht_xOver)
    set aw $N(em45)
    set wd $N(em35)
    set bo $N(ht_baseOver)

    S_ f8 $aw 0   0 $bo $wd $h   5 3
    $N(prefix)o$N(suffix) $wd
    moveto 0 0; lineto $wd $h
    E_
}

# u grave
proc $N(prefix)@f9$N(suffix) {} {
    global N
    set s $N(serif)
    set h $N(ht_xAcc)
    set wd $N(em40)
    set aw $N(em50)
    set ha $N(htGrave)
    set wa $N(wdGrave)
    set bo $N(ht_baseOver)

    set t  [expr int(0.2 * $wa)]
    set x1  [expr ($wd / 2) + $t]
    set ht [expr $h + $ha]

    if $s {
        S_ f9 $aw 0   0 $bo $wd $ht   8 3
    } {
        S_ f9 $aw 0   0 $bo $wd $ht   5 3
    }
    $N(prefix)u$N(suffix) $wd
    $N(prefix)grave$N(suffix) $x1 $h $ha $wa
    E_
}

# u acute
proc $N(prefix)@fa$N(suffix) {} {
    global N
    set s $N(serif)
    set h $N(ht_xAcc)
    set wd $N(em40)
    set aw $N(em50)
    set ha $N(htAcute)
    set wa $N(wdAcute)
    set bo $N(ht_baseOver)

    set t  [expr int(0.2 * $wa)]
    set x1  [expr ($wd / 2) - $t]
    set ht [expr $h + $ha]

    if $s {
        S_ fa $aw 0   0 $bo $wd $ht   8 3
    } {
        S_ fa $aw 0   0 $bo $wd $ht   5 3
    }
    $N(prefix)u$N(suffix) $wd
    $N(prefix)acute$N(suffix) $x1 $h $ha $wa
    E_
}

# u circum
proc $N(prefix)@fb$N(suffix) {} {
    global N
    set s $N(serif)
    set h $N(ht_xAcc)
    set wd $N(em40)
    set aw $N(em50)
    set ha $N(htCircum)
    set wa $N(wdCircum)
    set bo $N(ht_baseOver)

    set x1  [expr ($wd - $wa) / 2]
    set ht [expr $h + $ha]

    if $s {
        S_ fb $aw 0   0 $bo $wd $ht   9 3
    } {
        S_ fb $aw 0   0 $bo $wd $ht   6 3
    }
    $N(prefix)u$N(suffix) $wd
    $N(prefix)circum$N(suffix) $x1 $h $ha $wa
    E_
}

# u dieresis
proc $N(prefix)@fc$N(suffix) {} {
    global N
    set s $N(serif)
    set h $N(ht_xAcc)
    set r $N(dotRad)
    set wd $N(em40)
    set aw $N(em50)
    set ha $N(htDieresis)
    set wa $N(wdDieresis)
    set bo $N(ht_baseOver)

    set h1 [expr $h + $r]
    set ht [expr $h + $ha]
    set x1 [expr $r + ($wd - $wa) / 2]

    if $s {
        S_ fc $aw 0   0 $bo $wd $ht   15 4
    } {
        S_ fc $aw 0   0 $bo $wd $ht   12 4
    }
    $N(prefix)u$N(suffix) $wd
    $N(prefix)dieresis$N(suffix) $x1 $h1 $r [expr $wa - $r - $r]
    E_
}

# y acute
proc $N(prefix)@fd$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set h $N(ht_xAcc)
    set wd $N(em45)
    set aw $N(em55)
    set ha $N(htAcute)
    set wa $N(wdAcute)

    set t  [expr int(0.2 * $wa)]
    set x1  [expr ($wd / 2) - $t]
    set ht [expr $h + $ha]

    if $s {
        S_ fd $aw 0   0 $N(ht_dsc) $wd $ht   6 5
    } {
        S_ fd $aw 0   0 $N(ht_dsc) $wd $ht   4 3
    }
    $N(prefix)y$N(suffix) $wd
    $N(prefix)acute$N(suffix) $x1 $h $ha $wa
    E_
}

# thorn
#
proc $N(prefix)@fe$N(suffix) {} {
    global N
    set em $N(emSize)
    set s $N(serif)
    set s2 $N(2serif)
    set aw $N(em50)
    set wd $N(em40)
    set q  $N(em15)
    set h1 $N(ht_xOver)
    set h2 $N(ht_x)
    set h3 $N(ht_X)
    set b  $N(ht_dsc)
    set bo $N(ht_baseOver)

    set w [expr $wd / 2]
    set m [expr ($h1 - $bo) / 2]

    set y1 [expr $h2 - $q]

    S_ fe $aw 0   0 $b $wd $h3   7 3
    moveto 0 $h3; xlineto $s; ylineto $b
    moveto 0 $b; xlineto $s2
    moveto $s $q; ycurveto $w $bo; xcurveto $wd $m
    ycurveto $w $h1
    xcurveto $s $y1
    E_
}

# y dieresis
proc $N(prefix)@ff$N(suffix) {} {
    global N
    set em $N(emSize)
    set h $N(ht_xAcc)
    set r $N(dotRad)
    set ha $N(htDieresis)
    set wa $N(wdDieresis)
    set aw $N(em55)
    set wd $N(em45)

    set h1 [expr $h + $r]
    set ht [expr $h + $ha]
    set x1 [expr $r + ($wd - $wa) / 2]
    set ht [expr $h + $ha]

    S_ ff $aw 0   0 $N(ht_dsc) $wd $ht   13 6
    $N(prefix)y$N(suffix) $wd
    $N(prefix)dieresis$N(suffix) $x1 $h1 $r [expr $wa - $r - $r]
    E_
}

# Wrapup (mode specific)
WrapupSf
