#ifndef POINT_
#define POINT_

//======================================================================
//
// Point class
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.9 $
//
//======================================================================

#ifndef _FSTREAM_H
#include <fstream.h>
#endif

// Need M_PI
#if !defined(_MATH_H) && !defined(__math_h)
extern "C" {
#include <math.h>
}
#endif

// Need ApproxEq()
#ifndef GLOBALDEFS_
#include "globaldefs.H"
#endif

//====================  EXTERNS  =======================================

class Point ;

extern ostream&
operator<<( ostream &os, Point const &p ) ;

//===============================================================  Point
//
// A single point or vector depending on usage
//
class Point {
public:
    COORD x, y ;

    // ctors
    Point() {}
    Point( Point const &p ) : x( p.x ), y( p.y ) { }
    Point( COORD xx, COORD yy ) : x( xx ), y( yy ) { }

    // used in IntersectArgs template
    explicit Point( COORD xx ) : x( xx ), y( xx ) { }

    // comparison operators

    // exact equality
    bool
    operator==( Point const &q ) const {
        return x == q.x && y == q.y ;
    }

    bool
    operator!=( Point const &q ) const {
        return !(*this == q) ;
    }

    // approximate equality
    bool
    operator|( Point const &q ) const {
        return ApproxEq( x, q.x, 0.01 /* G::tolPtOnLine */ )
            && ApproxEq( y, q.y, 0.01 /* G::tolPtOnLine */ ) ;
    }

    // lexicographic ordering
    bool
    operator<( Point const &q ) const {
        return x < q.x || (x == q.x && y < q.y) ;
    }

    // assignment operators

    Point const &
    operator=( Point const &p ) {
        if ( this != &p ) {
            x = p.x, y = p.y ;
        }
        return *this ;
    }
    Point const &
    operator+=( Point const &p ) {
        x += p.x, y += p.y ;
        return *this ;
    }
    Point const &
    operator-=( Point const &p ) {
        x -= p.x, y -= p.y ;
        return *this ;
    }
    // This allows "v /= w /= k ;" to divide both v and w by k
    COORD
    operator/=( COORD f ) {
        x /= f, y /= f ;
        return f ;
    }
    // This allows "v *= w *= k ;" to multiply both v and w by k
    COORD
    operator*=( COORD f ) {
        x *= f, y *= f ;
        return f ;
    }

    // arithmetic operators
    Point
    operator+( Point const &p ) const {
        return Point( x + p.x, y + p.y ) ;
    }
    Point
    operator-( Point const &p ) const {
        return Point( x - p.x, y - p.y ) ;
    }
    Point
    operator*( COORD f ) const {
        return Point( x * f, y * f ) ;
    }
    Point
    operator/( COORD f ) const {
        return Point( x / f, y / f ) ;
    }
    // negation and unary minus
    Point
    operator-( ) const {
        return Point( -x, -y ) ;
    }
    void
    Negate( ) {
        x = -x, y = -y ;
    }

    // inner product; can be used to determine angle between non-zero
    // vectors since cos( angle ) = (inn. prod.) / (product of lengths):
    //   0   ==>  angle is 90 degrees (perpendicular)
    //   > 0 ==>  angle is acute
    //   < 0 ==>  angle is obtuse
    // Collinearity of vectors can be determined by examining the
    // outer product (see below). If they are collinear, we can
    // distinguish between cases where the angle is 0 or 180 degrees
    // by examining sign of the inner product.
    //
    COORD
    operator*( Point const &p ) const {
        return x * p.x + y * p.y ;
    }

    // Outer product: gives the signed area of the parallelogram.
    // Assuming that axes are oriented in the usual way with x
    // increasing to the right and y to the top, the outer product is:
    //   = 0 ==>  vectors are collinear
    //   > 0 ==>  p is anticlockwise from 'this'
    //   < 0 ==>  p is clockwise from 'this'
    // Perpendicularity of vectors can be determined by examining the
    // inner product (see above). If they are perpendicular, we can
    // distinguish between cases where the angle is 90 or 270 degrees
    // by examining the sign of the outer product.
    //
    COORD
    operator%( Point const &p ) const {
        return x * p.y - y * p.x ;
    }

    // other member functions

    // length
    COORD Len() const { return hypot( x, y ) ; }

    // square of length; sometimes we can avoid the square root
    // computation involved in Len()
    //
    COORD LenSq() const { return (x * x) + (y * y) ; }

    // vector perpendicular in the positive direction. Assuming ordinary
    // right-handed coordinate system, this will be anticlockwise; most
    // screens however have a left-handed coordinate system since the
    // origin is at the top left and the y-axis goes down; in such cases
    // this will be clockwise.
    //
    Point Perp() const { return Point( -y, x ) ; }

    // angle of inclination in radians in the range (0 <= angle < 2pi)
    // of a *UNIT* vector
    //
    COORD AngleUnit() const {
        if ( ApproxZero( x ) ) {
            return y > 0.0 ? G::half_pi : G::half_3pi ;
        }
        if ( ApproxZero( y ) ) {
            return x > 0.0 ? 0.0 : G::pi ;
        }

        // both non-zero
        COORD f = acos( x ) ;
        if ( y > 0.0 ) {
            return f ;
        }
        return x > 0.0 ? (G::two_pi - f) : (G::pi + f) ;
    }

    // angle of inclination in radians in the range (0 <= angle < 2pi)
    // of a non-zero vector. Caller *MUST* ensure that the length of the
    // vector is not very small.
    //
    COORD Angle() const {
        COORD len = Len() ;
        return Point( x / len, y / len ).Angle() ;
    }

} ;                                                             // Point

//-----------------------------------------------------------  operator*
//
// product of a scaler and vector (cannot be a member function since the
// left operand is a COORD and not a Point
//
inline Point
operator*( COORD f, Point const &p )
{
    return Point( p.x * f, p.y * f ) ;
}                                                           // operator*

#endif  // POINT_
