//======================================================================
//
// Stroke font rasterizer
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.5 $
//
//======================================================================

// Need strlen(), strchr() etc.
#if !defined(__string_h) && !defined(_STRING_H)
extern "C" {
#include <string.h>
}
#endif

// Need sort(), lower_bound() etc.
#ifndef ALGO_H
#include <algo.h>
#endif

#ifdef DEBUG
#include "Debug.H"
#define DBGFLAG dbg.Flag( Debug::eStrokeFont )
#define DBGFILE dbg.File( Debug::eStrokeFont )
#endif /* DEBUG */

#ifndef GEOMETRY_
#include "Geometry.H"
#endif

#ifndef STROKEFONT_
#include "StrokeFont.H"
#endif

//====================  GLOBALS/STATICS  ==============================

template< class T, INT32 aBits > T const Bitmap< T, aBits>::
    alignMask = Bitmap< T, aBits>::alignSize - 1 ;
template< class T, INT32 aBits > T const Bitmap< T, aBits>::
    fullMask  = ~static_cast< T >( 0 ) ;

// instantiate bitmap class
template BitmapX11 ;

// precomputed pens; these never change after initialization
//   pens[ i ] --> diameter i+2
//
static const INT32 MAXPEN = 8 ;
static BitmapX11  *pens[ MAXPEN ] ;

#ifdef DEBUG
//----------------------------------------------------------  operator<<
//
// Output a Ipoint
//
ostream&
operator<<( ostream &os, Ipoint const &pt )
{
    return os << '(' << pt.x << ',' << pt.y << ") " ;
}                                                          // operator<<

//-----------------------------------------------------------  dumpGlyph
//
// Dumps glyph data; if doHead is false, everything other than the
// strokes and segments is suppressed; if seg is not NIL, it should
// point to an array of segments (usually the array of transformed
// segments) whose number is consistent with the rest of the data in
// gPtr.
//
static void
dumpGlyph( SFGlyph const *const gPtr,
           SFSeg const   *const seg = NIL,
           bool          const  doHead = true )
{
    // dump the transformed strokes
    INT32 i = 0,
          nStrokes = gPtr->nStrokes,
          nSegs ;
    SFSeg *p, *pEnd ;

    p = NIL == seg ? gPtr->seg : seg ;

    if ( doHead ) {
        DBGFILE
            << "gCode = " << gPtr->gCode << ", adv = " << gPtr->adv
            << ", bbox = " << gPtr->bbox << endl
            << "nStrokes = " << nStrokes << ", nSegs = " << gPtr->nSegs
            << endl ;
    }
    for ( ; i < nStrokes ; i++ ) {
        nSegs = gPtr->nSS[ i ] ;
        pEnd = p + nSegs ;
        DBGFILE << "stroke " << i << ", " << nSegs << " segments:"
                << endl ;
        do {
            DBGFILE << "    " << p->pt ;
            if ( NIL != p->curve ) {    // straight
                DBGFILE << p->curve->apex << "("
                        << p->curve->bulge << ")" ;
            }
            DBGFILE<< endl ;
        } while ( ++p <= pEnd ) ;
    }
}                                                           // dumpGlyph
#endif  /* DEBUG */

//------------------------------------------------------------  XformSeg
//
// Transforms a single segment and adjusts the bbox
//
inline static void
XformSeg( Matrix const &mtx,
          SFSeg const  &in,
          bool const   initBbox,
          SFSeg        &out,       // output
          Bbox         &bbox )     // output
{
    mtx.XformPt( in.pt, out.pt ) ;
    if ( initBbox ) {
        bbox.Set( out.pt ) ;
    } else {
        bbox.Adjust( out.pt ) ;
    }
    if ( NIL == in.curve ) {
        out.curve = NIL ;
    } else {
        out.curve = new SFConic( in.curve->bulge ) ;
        mtx.XformPt( in.curve->apex, out.curve->apex ) ;
        if ( initBbox ) {
            bbox.Set( out.curve->apex ) ;
        } else {
            bbox.Adjust( out.curve->apex ) ;
        }
    }
}                                                            // XformSeg

//----------------------------------------------------------------  Line
//
// Draws a straight line from p1 to p2 on the bitmap
//
inline static void
Line( Point     const &pt1,
      Point     const &pt2,
      INT32     const iminx,
      INT32     const iminy,
      BitmapX11 const *const pen,  // may be NIL
      BitmapX11       &bm )
{
    Ipoint p1( Round( pt1.x ) - iminx , Round( pt1.y ) - iminy ),
           p2( Round( pt2.x ) - iminx, Round( pt2.y ) - iminy ) ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "    p1 = " << p1 << ", p2 = " << p2 << endl ;
    }
#endif

    if ( p1 != p2 ) {
        bm.Line( pen, p1, p2 ) ;
    }
}                                                                // Line

//---------------------------------------------------------  SFGlyphLess
//
// glyph code comparison; used for sorting array of glyphs by code
//
inline static bool
SFGlyphLess( SFGlyph const &g1, SFGlyph const &g2 )
{
    return g1.gCode < g2.gCode ;
}                                                         // SFGlyphLess

//-----------------------------------------------------------  InitSFont
//
// Initialize StrokeFont module
//
void
InitSFont()
{
    INT32 i ;
    for ( i = 0 ; i < MAXPEN ; i++ ) {
        pens[ i ] = BitmapX11::MakePen( i + 2 ) ;
        // pens[ i ]->Dump() ;
    }
}                                                           // InitSFont

//------------------------------------------------------------  GetPoint
//
// Get x and y coordinates and check them against bbox
//
static void
GetPoint( char const *strx,    // x source
          char const *stry,    // y source
          Bbox const &bbox,    // bounds for x and y
          char const *errmsg,  // error string
          COORD      &x,       // return x and y
          COORD      &y )
{
    if ( 1 != sscanf( strx, "%f", &x ) ) {
        cerr << "ERROR(" << errmsg << "): " ;
        Fatal( "sscanf failed (x)" ) ;
    }
    if ( ! bbox.InsideX( x ) ) {
        cerr << "x = " << x << endl ;
        cerr << "ERROR(" << errmsg << "): " ;
        Fatal( "Bad point (x)" ) ;
    }

    if ( 1 != sscanf( stry, "%f", &y ) ) {
        cerr << "ERROR(" << errmsg << "): " ;
        Fatal( "sscanf failed (y)" ) ;
    }
    if ( ! bbox.InsideY( y ) ) {
        cerr << "y = " << y << endl ;
        cerr << "ERROR(" << errmsg << "): " ;
        Fatal( "Bad point (y)" ) ;
    }
}                                                            // GetPoint

//----------------------------------------------------------  GlyphBegin
//
// Bound to the "S_" tcl proc
//
static int
GlyphBegin( ClientData  cd,
            Tcl_Interp  *,
            int         argc,
            char        *argv[] )
{
    if ( 10 != argc ) {
        Fatal( "(GlyphBegin): Too few args" ) ;
    }

    // get and check the glyph code
    int i ;
    if ( 1 != sscanf( argv[ 1 ], "%x", &i ) ) {
        Fatal( "(GlyphBegin): Bad glyph code" ) ;
    }

    StrokeFont *font = static_cast< StrokeFont * >( cd ) ;
    INT32 gCode = i ;
    SFGlyph tmpGlyph ;
    tmpGlyph.gCode = gCode ;

    SFGlyph *g    = font->mGlyph,
            *endg = g + font->mNumGlyphs,
            *gPtr = lower_bound( g, endg, tmpGlyph, SFGlyphLess ) ;

    if ( gPtr->gCode != gCode ) {
        cerr << "gCode = " << gCode << endl ;
        Fatal( "(GlyphBegin): glyph code not in array" ) ;
    }

    // error if we have already seen a "S_"
    if ( gPtr->nStrokes >= 0 ) {
        cerr << "gCode = " << gCode << endl ;
        Fatal( "(GlyphBegin): multiple S_ commands" ) ;
    }

    // get the width and bbox and check them
    //
    //COORD emSize     = static_cast< COORD >( font->mEmSize ) ;
    Bbox const &bbox = font->mBbox ;
    char *nm         = "GlyphBegin" ;

    float wx, wy, llx, lly, urx, ury ;
    int   nSegs, nStrokes ;

    // width
    GetPoint( argv[ 2 ], argv[ 3 ], bbox, nm, wx, wy ) ;

    // remove this check for fonts with a vertical writing direction
    if ( 0.0 != wy ) {
        cerr << "wy = " << wy << endl ;
        Fatal( "(GlyphBegin): Bad width (y)" ) ;
    }

    // glyph bbox
    GetPoint( argv[ 4 ], argv[ 5 ], bbox, nm, llx, lly ) ;
    GetPoint( argv[ 6 ], argv[ 7 ], bbox, nm, urx, ury ) ;

    // no. of segments and strokes (will be 0 for space chars)
    if ( 1 != sscanf( argv[ 8 ], "%i", &nSegs ) ) {
        Fatal( "(GlyphBegin): sscanf failed (nSegs)" ) ;
    }
    if ( nSegs < 0 || nSegs > 256 ) {
        cerr << "nSegs = " << nSegs << endl ;
        Fatal( "(GlyphBegin): Bad segment count" ) ;
    }

    if ( 1 != sscanf( argv[ 9 ], "%i", &nStrokes ) ) {
        Fatal( "(GlyphBegin): sscanf failed (nStrokes)" ) ;
    }
    if ( nStrokes < 0 || nStrokes > 256 ) {
        cerr << "nStrokes = " << nStrokes << endl ;
        Fatal( "(GlyphBegin): Bad stroke count" ) ;
    }

    gPtr->adv.x    = wx ;
    gPtr->adv.y    = wy ;
    gPtr->bbox.llx = llx ;
    gPtr->bbox.lly = lly ;
    gPtr->bbox.urx = urx ;
    gPtr->bbox.ury = ury ;
    gPtr->nSegs    = nSegs ;
    gPtr->nStrokes = nStrokes ;

    // allocate space for segments if any
    if ( nStrokes > 0 ) {
        gPtr->seg  = new SFSeg[ nSegs + nStrokes ] ;
        gPtr->nSS  = new INT32[ nStrokes ] ;
    }

    if ( NIL != font->mCurGlyph ) {
        Fatal( "(GlyphBegin): mCurGlyph is not NIL" ) ;
    }
    font->mCurGlyph = gPtr ;
    font->mCurSegCnt = font->mCurSegTot = font->mCurStrCnt = 0 ;
    font->mNewStr = false ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "Begin(), gCode = " << gCode
                << ", bbox = ["   << llx << ", " << lly
                << ", " << urx << ", " << ury << "]" << endl
                << "    wx = " << wx << ", wy = " << wy
                << ", nSegs = "    << nSegs
                << ", nStrokes = " << nStrokes
                << endl ;
    }
#endif /* DEBUG */

    return TCL_OK ;
}                                                          // GlyphBegin

//---------------------------------------------------------  GlyphMoveto
//
// Bound to the "M_" tcl proc
//
static int
GlyphMoveto( ClientData  cd,
             Tcl_Interp  *,
             int         argc,
             char        *argv[] )
{
    if ( 3 != argc ) {
        Fatal( "(GlyphMoveto): Wrong no. of args" ) ;
    }

    // get the width and bbox and check them. The range checks can be
    // modified if necessary to increase the limits.
    //
    StrokeFont *font = static_cast< StrokeFont * >( cd ) ;
    SFGlyph *glyph   = font->mCurGlyph ;
    Bbox const &bbox = glyph->bbox ;
    char const *nm   = "GlyphMoveto" ;
    float x, y ;

    // error if space char
    if ( 0 == glyph->nStrokes ) {
        Fatal( "(GlyphMoveto): space char" ) ;
    }

    // Each moveto starts a new stroke and a stroke MUST have at least
    // one segment (lineto or conicto) so two consecutive moveto's are
    // not allowed
    //
    if ( font->mNewStr ) {
        Fatal( "(GlyphMoveto): Empty stroke" ) ;
    }
    font->mNewStr = true ;

    // get the point coordinates and check them
    GetPoint( argv[ 1 ], argv[ 2 ], bbox, nm, x, y ) ;

    // if this is not the first stroke, set the segment count of the
    // previous stroke
    //
    if ( font->mCurStrCnt > 0 ) {
        if ( font->mCurStrCnt + 1 > glyph->nStrokes ) {
            Fatal( "(GlyphMoveto): stroke count exceeded" ) ;
        }
        glyph->nSS[ font->mCurStrCnt - 1 ] = font->mCurSegCnt ;
        glyph->seg[ font->mCurSegTot++ ].curve = NIL ;
        font->mCurSegCnt = 0 ;
    }
    if ( font->mCurSegTot >= glyph->nSegs + glyph->nStrokes ) {
        Fatal( "(GlyphMoveto): seg count exceeded" ) ;
    }
    glyph->seg[ font->mCurSegTot ].pt.x = x ;
    glyph->seg[ font->mCurSegTot ].pt.y = y ;

    font->mCurStrCnt++ ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "Moveto(), x = " << x
                << ", y = " << y
                << ", mCurSegTot = " << font->mCurSegTot
                << ", mCurSegCnt = " << font->mCurSegCnt
                << endl ;
    }
#endif /* DEBUG */

    return TCL_OK ;
}                                                         // GlyphMoveto

//---------------------------------------------------------  GlyphLineto
//
// Bound to the "L_" tcl proc
//
static int
GlyphLineto( ClientData  cd,
             Tcl_Interp  *,
             int         argc,
             char        *argv[] )
{
    if ( 2 > argc ) {
        Fatal( "(Canvas::NewCanvas): Too few args" ) ;
    }
    StrokeFont *font = static_cast< StrokeFont * >( cd ) ;
    SFGlyph *glyph   = font->mCurGlyph ;
    Bbox const &bbox = glyph->bbox ;
    char const *nm   = "GlyphLineto" ;
    float x, y ;

    font->mNewStr = false ;

    // error if space char
    if ( 0 == glyph->nStrokes ) {
        Fatal( "(GlyphLineto): space char" ) ;
    }

    // get the point coordinates and check them
    GetPoint( argv[ 1 ], argv[ 2 ], bbox, nm, x, y ) ;

    glyph->seg[ font->mCurSegTot++ ].curve = NIL ;
    if ( font->mCurSegTot >= glyph->nSegs + glyph->nStrokes ) {
        Fatal( "(GlyphLineto): seg count exceeded" ) ;
    }
    glyph->seg[ font->mCurSegTot ].pt.x = x ;
    glyph->seg[ font->mCurSegTot ].pt.y = y ;

    font->mCurSegCnt++ ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "Lineto(), x = " << x << ", y = " << y
                << ", mCurSegTot = " << font->mCurSegTot
                << ", mCurSegCnt = " << font->mCurSegCnt
                << endl ;
    }
#endif /* DEBUG */

    return TCL_OK ;
}                                                         // GlyphLineto

//--------------------------------------------------------  GlyphConicto
//
// Bound to the "C_" tcl proc
//
static int
GlyphConicto( ClientData  cd,
              Tcl_Interp  *,
              int         argc,
              char        *argv[] )
{
    if ( 2 > argc ) {
        Fatal( "(GlyphConicto): Too few args" ) ;
    }
    StrokeFont *font = static_cast< StrokeFont * >( cd ) ;
    SFGlyph *glyph   = font->mCurGlyph ;
    Bbox const &bbox = glyph->bbox ;
    char const *nm   = "GlyphConicto" ;
    float x, y, x1, y1, bulge ;

    font->mNewStr = false ;

    // error if space char
    if ( 0 == glyph->nStrokes ) {
        Fatal( "(GlyphConicto): space char" ) ;
    }

    // get point coordinates
    GetPoint( argv[ 1 ], argv[ 2 ], bbox, nm, x, y ) ;
    GetPoint( argv[ 3 ], argv[ 4 ], bbox, nm, x1, y1 ) ;

    // get bulge
    if ( 1 != sscanf( argv[ 5 ], "%f", &bulge ) ) {
        Fatal( "(GlyphConicto): sscanf failed (bulge)" ) ;
    }
    if ( bulge <= -0.9 || bulge > 9.0 ) {
        cerr << "bulge = " << bulge << endl ;
        Fatal( "(GlyphConicto): Bad bulge" ) ;
    }

    glyph->seg[ font->mCurSegTot++ ].curve
        = new SFConic( x1, y1, bulge ) ;
    if ( font->mCurSegTot >= glyph->nSegs + glyph->nStrokes ) {
        Fatal( "(GlyphConicto): seg count exceeded" ) ;
    }
    glyph->seg[ font->mCurSegTot ].pt.x = x ;
    glyph->seg[ font->mCurSegTot ].pt.y = y ;

    font->mCurSegCnt++ ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "Conicto(), x = " << x
                << ", y = " << y
                << ", x1 = " << x1 << ", y1 = " << y1
                << ", bulge = " << bulge << endl
                << "    mCurSegTot = " << font->mCurSegTot
                << ", mCurSegCnt = " << font->mCurSegCnt
                << endl ;
    }
#endif /* DEBUG */

    return TCL_OK ;
}                                                        // GlyphConicto

//------------------------------------------------------------  GlyphEnd
//
// Bound to the "E_" tcl proc
//
static int
GlyphEnd( ClientData  cd,
          Tcl_Interp  *,
          int         argc,
          char        *[] )
{
    if ( 1 != argc ) {
        Fatal( "(GlyphEnd): Too few args" ) ;
    }
    StrokeFont *font = static_cast< StrokeFont * >( cd ) ;
    SFGlyph *glyph   = font->mCurGlyph ;

    // if this is not the first stroke, set the segment count of the
    // previous stroke
    //
    if ( font->mCurStrCnt > 0 ) {
        if ( font->mCurStrCnt > glyph->nStrokes ) {
            Fatal( "(GlyphEnd): stroke count exceeded" ) ;
        }
        glyph->nSS[ font->mCurStrCnt - 1 ] = font->mCurSegCnt ;
        glyph->seg[ font->mCurSegTot ].curve = NIL ;
    }
    if ( 0 != glyph->nStrokes && font->mCurSegTot
                 != (glyph->nSegs + glyph->nStrokes - 1) ) {
        Fatal( "(GlyphEnd): seg count exceeded" ) ;
    }

    // These are done in GlyphStart as well so that if either S_ or
    // E_ is missing, the error is caught
    //
    font->mCurGlyph = NIL ;
    font->mCurSegCnt = font->mCurSegTot = font->mCurStrCnt = 0 ;
    font->mNewStr = false ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "End()"
                << endl ;
    }
#endif /* DEBUG */

    return TCL_OK ;
}                                                            // GlyphEnd

//--------------------------------------------------------------  MatMul
//
// Multiply two matrices and return the result; C may be the same as
// as A or B
//
void
MatMul( const Matrix &matA, const Matrix &matB, Matrix &matC )
{
    // need locals since C may be the same as A or B
    COORD a = matA.a * matB.a + matA.b * matB.c,
          b = matA.a * matB.b + matA.b * matB.d,
          c = matA.c * matB.a + matA.d * matB.c,
          d = matA.c * matB.b + matA.d * matB.d,
          x, y ;

    // do the translation
    if ( matA.tran ) {
        x = matA.x * matB.a + matA.y * matB.c ;
        y = matA.x * matB.b + matA.y * matB.d ;
        if ( matB.tran ) {
            x += matB.x ;
            y += matB.y ;
        }
        if ( ApproxZero( x ) &&  ApproxZero( y ) ) {
            matC.x = matC.y = 0.0 ;
            matC.tran = false ;
        } else {
            matC.x = x, matC.y = y ;
            matC.tran = true ;
        }
    } else {
        if ( matB.tran ) {
            matC.x = matB.x, matC.y = matB.y ;
            matC.tran = true ;
        } else {
            matC.x = matC.y = 0.0 ;
            matC.tran = false ;
        }
    }
    matC.a = a, matC.b = b, matC.c = c, matC.d = d ;
}                                                              // MatMul

//----------------------------------------------------------  StrokeFont
//
StrokeFont::
StrokeFont( char *fileName, Tcl_Interp *interp )
{
    // Eval the file, find the number of char procs, allocate glyph
    // array, initialize it with the glyph codes, sort it

    // (Tcl scribbles into its arguments so this should not be const)
    static char *cmd = "set mode scaler" ;

    if ( TCL_OK != Tcl_Eval( interp, cmd ) ) {
        cerr << "cmd = '" << cmd << "'" << endl ;
        Fatal( "(StrokeFont): Tcl_Eval() failed" ) ;
    }

    if ( TCL_OK != Tcl_EvalFile( interp, fileName ) ) {
        cerr << "fileName = '" << fileName << "'" << endl ;
        Fatal( "(StrokeFont): Tcl_EvalFile failed" ) ;
    }

    INT32 len ;
    char *tmp ;

    mInterp = interp ;
    mAName = GetTclVar( interp, "aName" ) ;

    // Number of glyphs in font
    mNumGlyphs = GetTclVarInt2( interp, mAName, "nGlyphs" ) ;

    // body size and matrix
    mEmSize = GetTclVarInt2( interp, mAName, "emSize" ) ;
    mMatrix.a = mMatrix.d = 1.0 / mEmSize ;
    mMatrix.b = mMatrix.c =
    mMatrix.x = mMatrix.y = 0.0 ;
    mMatrix.tran = false ;

    // bbox
    mBbox.llx = GetTclVarFloat2( interp, mAName, "llx" ) ;
    mBbox.lly = GetTclVarFloat2( interp, mAName, "lly" ) ;
    mBbox.urx = GetTclVarFloat2( interp, mAName, "urx" ) ;
    mBbox.ury = GetTclVarFloat2( interp, mAName, "ury" ) ;

    // font name
    tmp = GetTclVar2( interp, mAName, "fontName" ) ;
    len = strlen( tmp ) ;
    mName = new char[ len + 1 ] ;
    strcpy( mName, tmp ) ;

    // glyph proc prefix and suffix
    tmp        = GetTclVar2( interp, mAName, "prefix" ) ;
    mPrefixLen = strlen( tmp ) ;
    mPrefix    = new char[ mPrefixLen + 1 ] ;
    strcpy( mPrefix, tmp ) ;
    tmp        = GetTclVar2( interp, mAName, "suffix" ) ;
    mSuffixLen = strlen( tmp ) ;
    mSuffix    = new char[ mSuffixLen + 1 ] ;
    strcpy( mSuffix, tmp ) ;

    // allocate space for proc name (the additional 2 bytes are for the
    // '@' following the prefix and the terminating null byte)
    //
    INT32 num  = mPrefixLen + mSuffixLen + (2 * sizeof( INT32 )) + 2 ;
    mProc      = new char[ num ] ;

    // copy prefix to it (cannot copy suffix since the length of the
    // glyph code can take a variable amount of space (1-8 bytes)
    //
    strcpy( mProc, mPrefix ) ;
    mProc[ mPrefixLen ] = '@' ;
    mProc[ mPrefixLen + 1 ] = 0 ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "In StrokeFont(), numGlyphs = " << mNumGlyphs
                << endl ;
    }
#endif /* DEBUG */

    // glyph array
    mGlyph = new SFGlyph[ mNumGlyphs ] ;
    SFGlyph *g    = mGlyph,
            *endg = g + mNumGlyphs ;
    char *gList = GetTclVar2( interp, mAName, "gList" ),
         *gPtr = gList ;
    for ( ; g < endg ; g++ ) {
        g->seg = NIL ;
        g->nSS = NIL ;
        g->nStrokes = g->nSegs = -1 ;
        g->adv.x = g->adv.y = 0.0 ;

        // extract glyphcode (hex).
        // Note that glyphcodes MUST be positive but the check to limit
        // them to 16-bits is not essential and may be removed if
        // necessary ; our code should work even for 32-bit codes
        //
        int gcode ;
        if ( 1 != sscanf( gPtr, "%x", &gcode ) ) {
            Fatal( "(StrokeFont): sscanf failed" ) ;
        }
        if ( gcode <= 0 || gcode >= 65536 ) {
            Fatal( "(StrokeFont): sscanf failed" ) ;
        }
        g->gCode = gcode ;

        // find and skip separating blank
        gPtr = strchr( gPtr, ' ' ) ;
        if ( NULL == gPtr ) {
            Fatal( "(StrokeFont): No blank in gList" ) ;
        }
        gPtr++ ;
    }

    // sort the glyph array
    g     = mGlyph ;
    endg  = g + mNumGlyphs ;
    sort( g, endg, SFGlyphLess ) ;

    Tcl_CreateCommand( interp, "S_", GlyphBegin,
                       static_cast< ClientData >( this ),
                       (Tcl_CmdDeleteProc *)NIL ) ;
    Tcl_CreateCommand( interp, "M_", GlyphMoveto,
                       static_cast< ClientData >( this ),
                       (Tcl_CmdDeleteProc *)NIL ) ;
    Tcl_CreateCommand( interp, "L_", GlyphLineto,
                       static_cast< ClientData >( this ),
                       (Tcl_CmdDeleteProc *)NIL ) ;
    Tcl_CreateCommand( interp, "C_", GlyphConicto,
                       static_cast< ClientData >( this ),
                       (Tcl_CmdDeleteProc *)NIL ) ;
    Tcl_CreateCommand( interp, "E_", GlyphEnd,
                       static_cast< ClientData >( this ),
                       (Tcl_CmdDeleteProc *)NIL ) ;
#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "sorted gCodes = "
                << endl ;
        for ( ; g < endg ; g++ ) {
            DBGFILE << g->gCode << " " ;
        }
        DBGFILE << endl ;
    }
#endif

    mCurGlyph = NIL ;
    mCurSegCnt = mCurSegTot = mCurStrCnt = 0 ;
    mNewStr = false ;
}                                                          // StrokeFont

//-----------------------------------------------------------  GetBitmap
//
// Rasterizes a single glyph
//
SF_EStatus StrokeFont::
GetBitmap( SFInput *in, SFOutput *out )
{
    INT32 gCode    = in->gCode ;
    Matrix matrix ;
    MatMul( mMatrix, in->xform->mMtx, matrix ) ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "matrix: " << matrix ;
    }
#endif /* DEBUG */

    SFGlyph tmpGlyph ;
    tmpGlyph.gCode = gCode ;

    SFGlyph *g     = mGlyph,
            *gLast = mGlyph + mNumGlyphs,
            *gPtr  = lower_bound( g, gLast, tmpGlyph, SFGlyphLess ) ;

    if ( gPtr->gCode != gCode ) {
        return eStatNoChar ;
    }

    if ( -1 == gPtr->nStrokes ) {    // not yet initialized
        // Create the proc name
        char *p = mProc + mPrefixLen + 1 ;  // position of glyph code

        sprintf( p, "%x", gCode ) ;
        p += strlen( p ) ;
        strcpy( p, mSuffix ) ;

        // execute the proc
        if ( TCL_OK != Tcl_Eval( mInterp, mProc ) ) {
            cerr << "mProc = '" << mProc << "'" << endl ;
            Fatal( "(GetBitmap): Tcl_Eval() failed" ) ;
        }
#ifdef DEBUG
        if ( DBGFLAG ) {
            DBGFILE << "Original glyph: " << endl ;
            dumpGlyph( gPtr ) ;
        }
#endif /* DEBUG */

    }
    if ( gPtr->nStrokes < 0 ) {    // error
        Fatal( "(GetBitmap): Bad nStrokes" ) ;
    }

    // scale advance width
    matrix.XformPt( gPtr->adv, out->adv ) ;

    INT32 nStrokes = gPtr->nStrokes ;
    if ( 0 == nStrokes ) {    // space character, no bitmap
        matrix.XformPt( gPtr->adv, out->adv ) ;
        out->off.x = out->off.y = 0.0 ;
        out->bm = NIL ;
        return eStatOk ;
    }

    // scale points and find bbox; first point done separately to
    // initialize bbox
    //
    INT32 segSz = gPtr->nSegs + nStrokes ;
    SFSeg *xseg = new SFSeg[ segSz ],
          *xp   = xseg,
          *xEnd = xseg + segSz,
          *seg  = gPtr->seg ;

    Bbox xbbox ;
    XformSeg( matrix, *seg++, true, *xp++, xbbox ) ;

    // now transform remaining segments
    for ( ; xp != xEnd ; seg++, xp++ ) {
        XformSeg( matrix, *seg, false, *xp, xbbox ) ;
    }

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "Transformed strokes: " << endl ;
        dumpGlyph( gPtr, xseg, false ) ;
    }
#endif /* DEBUG */

    // allocate bitmaps
    INT32
        diam  = in->diam,
        rad   = (diam - 1) >> 1,      // offset for all points
        iminx = Round( xbbox.llx ) - rad,
        iminy = Round( xbbox.lly ) - rad,
        wd    = Round( xbbox.urx ) - iminx + diam,
        ht    = Round( xbbox.ury ) - iminy + diam,
        i     = 0 ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "xbbox = " << xbbox
                << ", wd = " << wd << ", ht = " << ht
                << endl ;
    }
#endif /* DEBUG */

    BitmapX11 *bm = new BitmapX11( wd, ht ) ;
    BitmapX11 *pen = NIL ;

    if ( diam > 1 ) {
        i = diam - 2 ;
        pen = i < MAXPEN ? pens[ i ] : BitmapX11::MakePen( diam ) ;
        // if ( diam >= 8 ) pen->Dump() ;
    }

    // draw the strokes
    i = 0 ;
    xp = xseg ;
    for ( ; i < nStrokes ; i++, xp++ ) {

#ifdef DEBUG
        if ( DBGFLAG ) {
            DBGFILE << "Stroke " << i << ", nSegs = " << gPtr->nSS[ i ]
                    << endl ;
        }
#endif

        xEnd = xp + gPtr->nSS[ i ] ;

        Ipoint p1, p2 ;
        p1.x = Round( xp->pt.x ) - iminx ;
        p1.y = Round( xp->pt.y ) - iminy ;
        do {
            if ( NIL == xp->curve ) {    // straight
                xp++ ;
                p2.x = Round( xp->pt.x ) - iminx ;
                p2.y = Round( xp->pt.y ) - iminy ;
#ifdef DEBUG
                if ( DBGFLAG ) {
                    DBGFILE << "    line: " << p1 << p2 << endl ;
                }
#endif
                bm->Line( pen, p1, p2 ) ;
            } else {                     // conic
                // flatten conic and draw it
                Point   &pt1  = xp->pt,
                        &apex = xp->curve->apex ;
                COORD   bulge = xp->curve->bulge ;

                xp++ ;

                Point      &pt2  = xp->pt ;
                ConicData  conicData ;
                Bbox       tmpBbox ;

                tmpBbox.Set( pt1, pt2 ) ;
                Geometry::Conic(
                    pt1, pt2, apex, bulge, tmpBbox, conicData ) ;

                // if flat, draw straight line
                p2.x = Round( xp->pt.x ) - iminx ;
                p2.y = Round( xp->pt.y ) - iminy ;
                if ( conicData.isFlat ) {
#ifdef DEBUG
                    if ( DBGFLAG ) {
                        DBGFILE << "    flat conic: " << p1 << p2
                                << endl ;
                    }
#endif
                    bm->Line( pen, p1, p2 ) ;
                } else {
                    Geometry::Flatten(
                        pt1, pt2, apex, bulge, tmpBbox, conicData ) ;
                    if ( NIL == conicData.flats
                         || ! conicData.flats->Valid() ) {
                        Fatal( "(GetBitmap): No flats" ) ;
                    }

                    // draw flattened segments
                    Point const
                        *pt    = conicData.flats->mPts,
                        *ptEnd = pt + conicData.flats->mCnt - 1 ;
#ifdef DEBUG
                    if ( DBGFLAG ) {
                        DBGFILE << "    conic: " << p1 << p2 << apex
                                << "(" << bulge << "), numFlatPts = "
                                << conicData.flats->mCnt
                                << endl ;
                    }
#endif
                    for ( ; pt < ptEnd ; pt++ ) {
                        Line( *pt, *(pt + 1), iminx, iminy, pen, *bm ) ;
                    }
                }
            }  // straight/curve test
            p1 = p2 ;
        } while ( xp < xEnd ) ;
    }

    // set offset vector and return
    out->off.x = static_cast< COORD >( iminx ) ;
    out->off.y = static_cast< COORD >( iminy ) ;
    out->bm = bm ;

    return eStatOk ;
}                                                           // GetBitmap
