#ifndef STROKEFONT_
#define STROKEFONT_

//======================================================================
//
// Stroke font rasterizer
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.3 $
//
//======================================================================

#ifndef _TCL
#include <tcl.h>
#endif

#ifndef BITMAP_
#include "Bitmap.H"
#endif

#ifndef XFORM_
#include "Xform.H"
#endif

#ifndef BBOX_
#include "Bbox.H"
#endif

// Input arguments for scaling a single character
//
// The offset is the vector from the origin to the (minx,miny) point of
// the bitmap and indicates where the bitmap should be placed w.r.t the
// current position.
//
struct SFInput {
    Xform  *xform ;    // transformation matrix
    INT32  gCode,      // glyph code
           diam ;      // pen diameter
} ;

struct SFOutput {
    Point      adv,      // advance vector
               off ;     // bitmap offset vector
    BitmapX11  *bm ;     // bitmap data
    // StrokePath *path
} ;

// apex and bulge of conic segment
struct SFConic {
    Point apex ;
    COORD bulge ;

    // ctors
    SFConic( COORD b ) : bulge( b ) {}
    SFConic( COORD x, COORD y, COORD b ) : apex( x, y ), bulge( b ) {}
} ;


// A single segment (pt is its initial endpoint)
struct SFSeg {
    Point   pt ;
    SFConic *curve ;  // NIL if straight
} ;

// function return values: add more as needed
enum SF_EStatus {
    eStatOk,         // No problem
    eStatBadFont,    // Malformed font
    eStatNoChar,     // Requested char missing
    eStatNoMem,      // Memory allocation failure
    eStatTooBig,     // scaled character is too large
    eStatError       // misc. error
} ;

// A single strokefont glyph
//
// NOTE: If we have n segments, we need (n+1) SFSeg items since a
//       segment needs two SFSegs to store it.
//
// -- each glyph is a collection of strokes; the number of strokes is
//    'nStrokes'
//
// -- nSS[ i ] is the number of segments in stroke i; so the segments
//    of stroke i are:
//        segs[ first(i) ] to segs[first(i) + nSS[i] - 1] where
//    first(i) is 0 if i == 0, and sum(nSS[ j ] + 1) for j in 0..(i-1)
//    if i > 0.
//
// -- the segments of stroke i go from segs[ j ].pt to segs[ j+1 ].pt
//    with the exception of the very last segment which goes to
//    segs[ first(i) ].
//
// -- each segment is a straight line if 'curve' is NIL, and a conic
//    with apex curve->pt and bulge curve->bulge otherwise.
//
// We want to initialize the gCode so that the array can be sorted
// but we want to defer full initialization to the first time the glyph
// is actually used (this avoids slow startup if there are a large
// number of glyphs). So we use nStrokes to distinguish space chars,
// which have no strokes/segments from uninitialized glyphs:
//      0  --> space glyph
//     -1  --> uninitialized glyph
//
struct SFGlyph {
    SFSeg *seg ;       // NIL for space, (nSegs + 1) elements otherwise
    INT32 *nSS,        // NIL for space, nStrokes elements otherwise
          nSegs,       // total number of segments
          nStrokes,    // -1 means not initialized
          gCode ;      // glyph code
    Point adv ;        // advance width
    Bbox  bbox ;
} ;                                                           // SFGlyph

// Initialization routine: must be be called exactly once before any
// functions in this module are invoked.
//
extern void
InitSFont( void ) ;

class StrokeFont {
public:
    SFGlyph   *mGlyph ;     // glyph array sorted by gCode
    INT32     mNumGlyphs,
              mEmSize,      // body size
              mPrefixLen,
              mSuffixLen ;
    char      *mName,       // font name
              *mAName,      // name of array containing font-wide info
              *mPrefix,     // glyph proc prefix
              *mSuffix ;    // glyph proc suffix
    Matrix    mMatrix ;     // font matrix
    Bbox      mBbox ;       // font bbox (used to check pt. coords.)

    // These are used to keep state information during a glyph
    // definition
    SFGlyph   *mCurGlyph ;    // current glyph (points into mGlyph)
    INT32     mCurSegCnt,     // seg count of current stroke
              mCurSegTot,     // cumulative seg count of current glyph
              mCurStrCnt ;    // stroke count  of current glyph
    char      *mProc ;        // glyph proc name
    bool      mNewStr ;       // started new stroke with moveto

    Tcl_Interp  *mInterp ;

    // ctor
    StrokeFont( char *fileName, Tcl_Interp *interp ) ;

    // other methods
    SF_EStatus
    GetBitmap( SFInput *in, SFOutput *out ) ;

    SF_EStatus
    GetStrokes( SFInput *in, SFOutput *out ) ;

} ;                                                        // StrokeFont

// These must be provided by the environment
extern char *
GetTclVar( Tcl_Interp *const interp,
           char const *const name,
           bool const mustExist = true ) ;

extern char *
GetTclVar2( Tcl_Interp *interp,
            char const *const name1,
            char const *const name2,
            bool const mustExist = true ) ;

extern INT32
GetTclVarInt( Tcl_Interp *const interp, char const *const name ) ;

extern INT32
GetTclVarInt2( Tcl_Interp *const interp,
               char const *const name1,
               char const *const name2 ) ;

extern COORD
GetTclVarFloat( Tcl_Interp *const interp,
                char const *const name ) ;

extern COORD
GetTclVarFloat2( Tcl_Interp *const interp,
                 char const *const name1,
                 char const *const name2 ) ;

#endif  // STROKEFONT_
