//======================================================================
//
// Transform Matrices
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.9 $
//
//======================================================================

#ifndef _FSTREAM_H
#include <fstream.h>
#endif

#ifdef DEBUG
#ifndef DEBUG_
#include "Debug.H"
#endif
#endif

#ifndef GEOMETRY_
#include "Geometry.H"
#endif

#ifndef XFORM_
#include "Xform.H"
#endif

//====================  FUNCTIONS  =====================================

//----------------------------------------------------------  operator<<
//
// Output a matrix
//
ostream&
operator<<( ostream &os, const Matrix &m )
{
    return os << '[' << m.a << ',' << m.b << ',' << m.c << ','
              << m.d << ',' << m.x << ',' << m.y << ']'
              << endl ;
}                                                          // operator<<

//----------------------------------------------------------  operator<<
//
// Output an xform
//
ostream&
operator<<( ostream &os, const Xform &xform )
{
    return os << "type=" << xform.mType << ", " << xform.mMtx ;
}                                                          // operator<<

//-------------------------------------------------------------  XformPt
//
// Transforms a point ; src and dst may be the same
//
void Matrix::
XformPt( Point const &src, Point &dst ) const
{
    // need temporaries since src and dst may be the same
    COORD xx = a * src.x + c * src.y,
          yy = b * src.x + d * src.y ;
    if ( tran ) {
        xx += x ;
        yy += y ;
    }
    dst.x = xx ;
    dst.y = yy ;
}                                                             // XformPt

//--------------------------------------------------------  Xform::Check
//
// Check for very small or very large determinant values. Should throw
// exception but for now just terminates if values are out of range.
//
inline void
Xform::Check()
{
    // Check for very small or very large scales
    COORD absdet = fabs( mDet ), absdetr = fabs( mDetr ) ;

    if ( isinf( mDet ) || isnan( mDet ) ) {
        Fatal( "(Xform::Xform): Bad determinant" ) ;
    }
    if ( isinf( mDetr ) || isnan( mDetr ) ) {
        Fatal( "(Xform::Xform): Bad inverse determinant" ) ;
    }

    if ( absdet < verySmall ) {
        Fatal( "(Xform::Xform): Singular matrix (det zero)" ) ;
    }
    if ( absdet > veryLarge ) {
        Fatal( "(Xform::Xform): Determinant too large" ) ;
    }
    if ( absdetr < verySmall ) {
        Fatal( "(Xform::Xform): Determinant of inv. too small" ) ;
    }
    if ( absdetr > veryLarge ) {
        Fatal( "Error(Xform::Xform): Determinant of inv. too large" ) ;
    }
}                                                        // Xform::Check

//---------------------------------------------------------------  Xform
//
// General matrix
//
Xform::Xform( COORD sx,    COORD sy,
              COORD skew,  COORD rotate,
              COORD xx,    COORD yy )
{
    mSkewX = ApproxEq( skew, 0.0 ) ? 0.0 : skew ;
    if ( fabs( rotate ) >= 360.0 ) {
        mRot =  Geometry::Fmod( rotate ) ;
    } else if ( rotate < 0.0 ) {
        mRot = rotate + 360 ;
    } else {
        mRot = rotate ;
    }

    COORD sinRot, cosRot, tanSkew = tan( Geometry::d2r * mSkewX ) ;

    if ( ApproxEq( mRot, 0.0 ) ) {
        mRot = 0.0, sinRot = 0.0, cosRot = 1.0 ;
    } else if ( ApproxEq( mRot, 90.0 ) ) {
        mRot = 90.0, sinRot = 1.0, cosRot = 0.0 ;
    } else if ( ApproxEq( mRot, 180.0 ) ) {
        mRot = 180.0, sinRot = 0.0, cosRot = -1.0 ;
    } else if ( ApproxEq( mRot, 270.0 ) ) {
        mRot = 270.0, sinRot = -1.0, cosRot = 0.0 ;
    } else {
        COORD tmp = Geometry::d2r * mRot ;
        sinRot = sin( tmp ), cosRot = cos( tmp ) ;
    }

    mMtx.a = sx * cosRot ;
    mMtx.b = sx * sinRot ;
    mMtx.c = sy * (tanSkew * cosRot - sinRot) ;
    mMtx.d = sy * (tanSkew * sinRot + cosRot) ;
    mMtx.x = ApproxEq( xx, 0.0 ) ? 0.0 : xx ;
    mMtx.y = ApproxEq( yy, 0.0 ) ? 0.0 : yy ;

    mDet  = mMtx.a * mMtx.d - mMtx.b * mMtx.c ;
    mDetr = 1.0 / mDet ;

    // Check for very small or very large determinant values
    Check() ;

    // Check for some special cases ; add more later
    if ( 0.0 == mMtx.b ) {
        if ( 0.0 == mMtx.c ) {                     // scale/identity
            if ( 1.0 == mMtx.a ) {
                mType = (1.0 == mMtx.d) ? eIdent
                  : (-1.0 == mMtx.d) ? eFlip270 : eScale ;
            } else if ( -1.0 == mMtx.a ) {
                mType = (1.0 == mMtx.d) ? eFlip90
                  : (-1.0 == mMtx.d) ? eRot180 : eScale ;
            }
        } else if ( 1.0 == mMtx.a && 1.0 == mMtx.d ) {  // x-skew
            mType   = eSkewX ;
        }
    } else if ( 0.0 == mMtx.c ) {
        if ( 1.0 == mMtx.a && 1.0 == mMtx.d ) {         // y-skew
            mType   = eSkewY ;
            mSkewY = Geometry::Fmod( atan( mMtx.b ) ) * Geometry::r2d ;
        }
    } else if ( 0.0 == mMtx.a && 0.0 == mMtx.d ) {
        if ( 1.0 == mMtx.b ) {
            mType = (1.0 == mMtx.c) ? eFlip
              : (-1.0 == mMtx.c) ? eRot270 : eScale ;
        } else if ( -1.0 == mMtx.b ) {
            mType = (1.0 == mMtx.c) ? eRot90
              : (-1.0 == mMtx.c) ? eFlip180 : eScale ;
        }
    }

    mMtxInv.a = mMtx.d * mDetr ;    mMtxInv.b = -mMtx.b * mDetr ;
    mMtxInv.d = mMtx.a * mDetr ;    mMtxInv.c  = -mMtx.c * mDetr ;
    mMtx.tran = (0.0 == mMtx.x && 0.0 == mMtx.y) ? false : true ;
    mMtxInv.tran = mMtx.tran ;
}
