#ifndef XFORM_
#define XFORM_

//======================================================================
//
// Transformation matrices
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.6 $
//
//======================================================================

// Need fabs()
#if !defined(_MATH_H) && !defined(__math_h)
extern "C" {
#include <math.h>
}
#endif

#ifndef POINT_
#include "Point.H"
#endif

//==============================================================  Matrix
//
// Transformation matrix as in PostScript: [a b c d x y]
// This is a lighweight class, so most methods, sanity checks etc. are
// in Xform
//
class Matrix {

    friend ostream& operator<<( ostream &os, const Matrix &m ) ;

public:

    COORD a,  b,  c,  d,  x,  y ;

    // true if there is translation, i.e. x != 0 || y != 0
    bool  tran ;

    // ctors
    Matrix( COORD aA, COORD aB, COORD aC, COORD aD, COORD aX, COORD aY )
        : a( aA ), b( aB ), c( aC ), d( aD ), x( aX ), y( aY ),
          tran( (0.0 == aX && 0.0 == aY) ? false : true )
    {}
    Matrix( COORD aA, COORD aB, COORD aC, COORD aD )
        : a( aA ), b( aB ), c( aC ), d( aD ), x( 0.0 ), y( 0.0 ),
          tran( false )
    {}
    Matrix() { }

    // Transforms a point; src and dst may be the same
    void
    XformPt( Point const &src, Point &dst ) const ;
} ;                                                            // Matrix

//===============================================================  Xform
//
// 2D transformation as in PostScript: [a b c d x y]
//
// Client specifies scalex, scaley, angSkew, angRot, x and y; the other
// members are computed. The intent is to manipulate these components
// individually rather than the matrix as a whole.
//
// There seemed to be no strong reasons for deriving from Matrix; since
// we store both the matrix and its inverse, we decided to make both
// members.
//
// MUST be invertible
//
class Xform {

    friend ostream& operator<<( ostream &os, const Xform &xform ) ;

public:

    Matrix mMtx, mMtxInv ;          // matrix and its inverse

    // skewY meaninful iff typ == eSkewY
    COORD mScaleX, mScaleY,        // scale factors
          mSkewX, mSkewY,          // skew (degrees)
          mRot,                    // rotation (degrees)
          mDet, mDetr ;            // determinant and reciprocal thereof

    // type of Xform
    //
    // The matrices corresponding to these types are:
    // eIdent   = [1  0 0  1]    eRot90   = [0 -1  1  0]
    // eRot180  = [-1 0 0 -1]    eRot270  = [0  1 -1  0]
    // eFlip    = [0  1 1  0]    eFlip90  = [-1 0  0  1]
    // eFlip180 = [0 -1 -1 0]    eFlip270 = [1  0  0 -1]
    //
    // eScale = [Sx 0 0   Sy]    eRot   = [cos -sin sin cos]
    // eSkewX = [1  0 tan  1]    eSkewY = [1   tan  0     1]
    enum XformType {
        eIdent,  eRot90,   eRot180,   eRot270,
        eFlip,   eFlip90,  eFlip180,  eFlip270,
        eScale,  eRot,     eSkewX,    eSkewY,    eGeneral
    } ;

    // ctors
    Xform( COORD sx, COORD sy, COORD skew, COORD rot,
           COORD xx, COORD yy ) ;

private:
    XformType mType ;

    // Bounds on the determinant for reasonable transformations
    static const COORD
        verySmall,    // = 1.0e-6
        veryLarge ;   // = 1.0e+6

    // check reasonableness; should throw exception but for now just
    // terminates if matrix values are too large or too small
    //
    inline void Check() ;
} ;                                                             // Xform

#endif  // XFORM_
