#ifndef GLOBALDEFS_
#define GLOBALDEFS_

//======================================================================
//
// Global definitions
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.10 $
//
//======================================================================

// for M_PI, fabs() etc.
#ifndef __math_h
extern "C" {
#include <math.h>
}
#endif

//====================  DEFINES  =======================================

// syntactic sugar for nil pointer
#define NIL 0

// point coordinates
#define COORD   float

// NOTE: These denote _MINIMUM_ bit counts, and *NOT* exact bit counts.
//       So the code should work even if, say, INT16 is a 32 or 64 bit
//       type. If you need an exact number of bits, use explict
//       masking operations
//
#define INT8    signed char
#define INT16   signed short
#define INT32   signed int
#define UINT8   unsigned char
#define UINT16  unsigned short
#define UINT32  unsigned int

//====================  EXTERNS  =======================================

// from main.C
extern void
Fatal( const char *const msg ) ;

//====================  TYPES  =========================================

//-------------------------------------------------------------------  G
//
// A minimal set of global values -- mostly constants and tolerances --
// needed by StrokeFont is encapsulated here; all other globals should
// be placed in derived classes. This is a singleton class.
//
class G {
protected:
    G( ) { } ;  // enforce singleton semantics (sort of)

public:
    static COORD const
        pi,        // = M_PI
        half_pi,   // = M_PI_2
        half_3pi,  // = 3.0 * M_PI_2
        two_pi ;   // = M_PI + M_PI

    //--------------------------------------------------------------
    // tolerances, thresholds, limits of various sorts; all of these
    // members are prefixed with 'tol'. They may be reset from values
    // read from external data files or tcl variables, so they are not
    // made 'const'. The intent however, is that once appropriately
    // initialized, they will not change.
    //--------------------------------------------------------------

    // threshold distance of shoulder point from chord for conic to be
    // considered flat; should be sufficiently small that for allowed
    // zoom factors, we do not have to recompute 'isFlat' flags.
    // Something like 0.2 should work.
    //
    static COORD const tolFlatConic ;

    // segments whose ends are closer than this value in both dimensions
    // and are flat will be considered 'trivial' and discarded.
    // Something like 0.1 is reasonable.
    //
    static COORD const tolTrivSeg ;

    // when comparing floats, values that differ by less than this will
    // be considered equal.  This value is used in contexts where no
    // other more specific tolerance is applicable. Use something like
    // 1.0e-6.
    //
    static COORD const tolZero ;
} ;                                                                 // G

//====================  FUNCTIONS  =====================================

//==========================================================  ApproxZero
//
// Returns true iff argument is close to zero. 'eps' is the tolerance
// and MUST be positive.
//
inline bool
ApproxZero( COORD v1, COORD eps = G::tolZero )
{
    // alternatively: return fabs(v1) <= eps;
    return -eps <= v1 && v1 <= eps ;
}                                                          // ApproxZero

//============================================================  ApproxEq
//
// Returns true iff arguments are approximately equal; 'eps' is the
// tolerance and MUST be positive.
//
inline bool
ApproxEq( COORD v1, COORD v2, COORD eps = G::tolZero )
{
    // alternatively: return fabs(v1 - v2) <= eps;
    return (v1 - eps <= v2) && (v2 <= v1 + eps) ;
}                                                            // ApproxEq

//---------------------------------------------------------------  Round
//
// Rounds a floating point value to an integral value
//
inline INT32
Round( COORD x )
{
    return (INT32)(x < 0.0 ? (-0.5 + x) : (0.5 + x)) ;
}                                                               // Round

#endif  // GLOBALDEFS_
