//======================================================================
//
// Main program for stroke font test application
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.5 $
//
//======================================================================

// Need atoi()
#if !(defined(_STDLIB_H) || defined(__stdlib_h))
extern "C" {
#include <stdlib.h>
}
#endif

// Need fabs()
#if !defined(_MATH_H) && !defined(__math_h)
extern "C" {
#include <math.h>
}
#endif

#ifndef _FSTREAM_H
#include <fstream.h>
#endif

using namespace std ;

#ifndef _TK
#include <tk.h>
#endif

#ifndef STROKEFONT_
#include "StrokeFont.H"
#endif

#ifndef GEOMETRY_
#include "Geometry.H"
#endif

#ifdef DEBUG
#include "Debug.H"
#define DBGFLAG dbg.Flag( Debug::eStrokeFont )
#define DBGFILE dbg.File( Debug::eStrokeFont )
#endif /* DEBUG */

//====================  GLOBALS/STATICS  ==============================
//
// Some globals/statics also in StrokeFont.C
//

COORD const
    G::pi           = M_PI,
    G::half_pi      = M_PI_2,
    G::half_3pi     = 3.0 * M_PI_2,
    G::two_pi       = M_PI + M_PI,
    G::tolFlatConic = 0.5,
    G::tolTrivSeg   = 0.1,
    G::tolZero      = 1.0e-5 ;

COORD const
    Geometry::d2r  = M_PI  / 180.0,   // degrees to radians
    Geometry::r2d  = 180.0 / M_PI ;   // radians to degrees

COORD const             // keep conic bulge in a reasonable range
    Geometry::bulgeMin = -0.99,
    Geometry::bulgeMax = 1000 ;

COORD const    // Bounds on the determinant for reasonable xforms
    Xform::verySmall = 1.0e-6,
    Xform::veryLarge = 1.0e+6 ;

#ifdef DEBUG
Debug dbg( "dbg.dat" ) ;
#endif

//---------------------------------------------------------------  Fatal
//
// Print fatal error message and exit
//
void
Fatal( const char *const msg )
{
    cerr << "ERROR" << msg << endl ;
    exit( 1 ) ;
}                                                               // Fatal

//---------------------------------------------------------------  Usage
//
// Print fatal error message, usage message and exit
//
//   xscale, yscale  --  scale factors (points)
//   skew, rotate    --  skew and rotate
//   xpos, ypos      --  position on canvas to display bitmap
//   diam, res       --  pen diameter and resolution (dpi)
//   glyph           --  glyph code
//
void
Usage( const char *const progname,
       const char *const msg )
{
    cerr << "Usage : " << progname << " [-xscale f] [-yscale f] "
         << "[-skew f] [-rotate f] [-diam i] [-res i] "
         << "[-xpos i] [-ypos i] [-glyph i]"
         << endl ;
    Fatal( msg ) ;
}                                                               // Usage

//------------------------------------------------------------ GetTclVar
//
// Returns the value of a tcl *GLOBAL* variable if it exists; if it does
// not exist, either NIL is returned or a fatal error occurs according
// as the second argument is false or true.
//
// The return value is volatile and should be copied if it needs to be
// long-lived.
//
char *
GetTclVar( Tcl_Interp *interp,
           char const *const name,
           bool mustExist )
{
    if ( NIL == name || '\0' == *name ) {
        Fatal( "(GetTclVar): Empty name" ) ;
    }

    // since the argument may be in readonly memory and tcl scribbles
    // into its arguments, we need to copy the argument
    //
    char *nm = new char[ strlen( name ) + 1 ] ;
    strcpy( nm, name ) ;
    char *tmp = Tcl_GetVar( interp, nm, TCL_GLOBAL_ONLY ) ;
    delete [] nm ;

    if ( mustExist && NULL == tmp ) {
        cerr << name << endl ;
        Fatal( "(GetTclVar): var not found" ) ;
    }
    return tmp ;
}                                                           // GetTclVar

//----------------------------------------------------------- GetTclVar2
//
// Returns the value of a tcl *GLOBAL* variable if it exists; if it does
// not exist, either NIL is returned or a fatal error occurs according
// as the second argument is false or true.
//
// The return value is volatile and should be copied if it needs to be
// long-lived.
//
char *
GetTclVar2( Tcl_Interp *const interp,
            char const *const name1,
            char const *const name2,
            bool const mustExist )
{
    if ( NIL == name1 || '\0' == *name1 ) {
        Fatal( "(GetTclVar2): Empty name" ) ;
    }
    if ( NIL == name2 || '\0' == *name2 ) {
        Fatal( "(GetTclVar2): Empty index" ) ;
    }

    // since the argument may be in readonly memory and tcl scribbles
    // into its arguments, we need to copy the argument
    //
    char *nm1 = new char[ strlen( name1 ) + 1 ],
         *nm2 = new char[ strlen( name2 ) + 1 ] ;
    strcpy( nm1, name1 ) ; strcpy( nm2, name2 ) ;

    char *tmp = Tcl_GetVar2( interp, nm1, nm2, TCL_GLOBAL_ONLY ) ;
    delete [] nm1 ;
    delete [] nm2 ;

    if ( mustExist && NULL == tmp ) {
        cerr << name1 << "(" << name2 << ")" << endl ;
        Fatal( "(GetTclVar2): var not found" ) ;
    }
    return tmp ;
}                                                          // GetTclVar2

//--------------------------------------------------------- GetTclVarInt
//
// Returns the value of a tcl *GLOBAL* variable as an integer if it
// exists. If it does not exist, a fatal error occurs.
//
INT32
GetTclVarInt( Tcl_Interp *const interp, char const *const name )
{
    if ( NIL == name || '\0' == *name ) {
        Fatal( "(GetTclVarInt): Empty name" ) ;
    }

    // since the argument may be in readonly memory and tcl scribbles
    // into its arguments, we need to copy the argument
    //
    char *nm = new char[ strlen( name ) + 1 ] ;
    strcpy( nm, name ) ;
    char *tmp = Tcl_GetVar( interp, nm, TCL_GLOBAL_ONLY ) ;
    delete [] nm ;

    if ( NULL == tmp ) {
        cerr << name << endl ;
        Fatal( "(GetTclVarInt): var not found" ) ;
    }
    return atoi( tmp ) ;
}                                                        // GetTclVarInt

//-------------------------------------------------------- GetTclVarInt2
//
// Returns the value of a tcl *GLOBAL* variable as an integer if it
// exists. If it does not exist, a fatal error occurs.
//
INT32
GetTclVarInt2( Tcl_Interp *const interp,
               char const *const name1,
               char const *const name2 )
{
    if ( NIL == name1 || '\0' == *name1 ) {
        Fatal( "(GetTclVarInt2): Empty name" ) ;
    }
    if ( NIL == name2 || '\0' == *name2 ) {
        Fatal( "(GetTclVarInt2): Empty index" ) ;
    }

    // since the argument may be in readonly memory and tcl scribbles
    // into its arguments, we need to copy the argument
    //
    char *nm1 = new char[ strlen( name1 ) + 1 ],
         *nm2 = new char[ strlen( name2 ) + 1 ] ;
    strcpy( nm1, name1 ) ; strcpy( nm2, name2 ) ;

    char *tmp = Tcl_GetVar2( interp, nm1, nm2, TCL_GLOBAL_ONLY ) ;
    delete [] nm1 ;
    delete [] nm2 ;

    if ( NULL == tmp ) {
        cerr << name1 << "(" << name2 << ")" << endl ;
        Fatal( "(GetTclVarInt2): var not found" ) ;
    }
    return atoi( tmp ) ;
}                                                       // GetTclVarInt2

//------------------------------------------------------- GetTclVarFloat
//
// Returns the value of a tcl *GLOBAL* variable as a float/double if it
// exists. If it does not exist, a fatal error occurs.
//
COORD
GetTclVarFloat( Tcl_Interp *const interp, char const *const name )
{
    if ( NIL == name || '\0' == *name ) {
        Fatal( "(GetTclVarFloat2): Empty name" ) ;
    }

    // since the argument may be in readonly memory and tcl scribbles
    // into its arguments, we need to copy the argument
    //
    char *nm = new char[ strlen( name ) + 1 ] ;
    strcpy( nm, name ) ;
    char *tmp = Tcl_GetVar( interp, nm, TCL_GLOBAL_ONLY ) ;
    delete [] nm ;

    if ( NULL == tmp ) {
        cerr << name << endl ;
        Fatal( "(GetTclVarFloat2): var not found" ) ;
    }
    return atof( tmp ) ;
}                                                      // GetTclVarFloat

//------------------------------------------------------ GetTclVarFloat2
//
// Returns the value of a tcl *GLOBAL* variable as a float/double if it
// exists. If it does not exist, a fatal error occurs.
//
COORD
GetTclVarFloat2( Tcl_Interp *const interp,
                 char const *const name1,
                 char const *const name2 )
{
    if ( NIL == name1 || '\0' == *name1 ) {
        Fatal( "(GetTclVarFloat2): Empty name" ) ;
    }
    if ( NIL == name2 || '\0' == *name2 ) {
        Fatal( "(GetTclVarFloat2): Empty index" ) ;
    }

    // since the argument may be in readonly memory and tcl scribbles
    // into its arguments, we need to copy the argument
    //
    char *nm1 = new char[ strlen( name1 ) + 1 ],
         *nm2 = new char[ strlen( name2 ) + 1 ] ;
    strcpy( nm1, name1 ) ; strcpy( nm2, name2 ) ;

    char *tmp = Tcl_GetVar2( interp, nm1, nm2, TCL_GLOBAL_ONLY ) ;
    delete [] nm1 ;
    delete [] nm2 ;

    if ( NULL == tmp ) {
        cerr << name1 << "(" << name2 << ")" << endl ;
        Fatal( "(GetTclVarFloat2): var not found" ) ;
    }
    return atof( tmp ) ;
}                                                     // GetTclVarFloat2

//------------------------------------------------------------  GetFloat
//
// Get a float argument
//
static void
GetFloat( char const *const progname,
          char const *const argname,
          char const *const src,
          int        const  numArgs,
          float      *const arg,
          bool       &flag )
{
    if ( flag ) {
        cerr << "Multiple " << argname << " flags" << endl ;
        Usage( progname, "Duplicate flags" ) ;
    }
    flag = true ;
    if ( 0 == numArgs ) {
        cerr << "Missing value after " << argname << endl ;
        Usage( progname, "Missing argument value" ) ;
    }
    if ( 1 != sscanf( src, "%f", arg ) ) {
        cerr << "Bad argument value for " << argname << endl ;
        Usage( progname, "Bad argument value" ) ;
    }
}                                                            // GetFloat

//--------------------------------------------------------------  GetInt
//
// Get an int argument
//
static void
GetInt( char const *const progname,
        char const *const argname,
        char const *const src,
        int        const numArgs,
        int  const *const arg,
        bool       &flag )
{
    if ( flag ) {
        cerr << "Multiple " << argname << " flags" << endl ;
        Usage( progname, "Duplicate flags" ) ;
    }
    flag = true ;
    if ( 0 == numArgs ) {
        cerr << "Missing value after " << argname << endl ;
        Usage( progname, "Missing argument value" ) ;
    }
    if ( 1 != sscanf( src, "%d", arg ) ) {
        cerr << "Bad argument value for " << argname << endl ;
        Usage( progname, "Bad argument value" ) ;
    }
}                                                              // GetInt

//-------------------------------------------------------------  DoGlyph
//
// command to rasterize a single glyph
//
// Invoked when the tcl command "g" is executed.
//
int
DoGlyph( ClientData  cd,
         Tcl_Interp  *interp,
         int         argc,
         char        *argv[] )
{
#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "Into DoGlyph, argc = " << argc << endl ;
    }
#endif /* DEBUG */

    if ( 1 > argc ) {
        Fatal( "(Canvas::DoGlyph): Bad argc" ) ;
    }

    StrokeFont *sfont = reinterpret_cast< StrokeFont * >( cd ) ;

    bool     xscaleflag = false,
             yscaleflag = false,
             skewflag   = false,
             rotateflag = false,
             glyphflag  = false,
             xposflag   = false,
             yposflag   = false,
             resflag    = false,
             diamflag   = false ;

    // Though typical screen resolution (dpi) is in the range 72-100,
    // the 10 point bitmaps look better if the resolution is set to 120
    //
    COORD    xscale = 20.0, yscale = 20.0,  // in points
             skew = 0.0, rotate = 0.0,      // in degrees
             res = 120.0 ;
    INT32    diam = 1, glyph = 97 ;
    char     *progname = *argv++ ;

    // we want these to persist from call to call
    static COORD xpos = 50.0, ypos = 50.0 ;

    /* process command line arguments */
    for ( ; --argc > 0 ; argv++ ) {
        if ( 0 == strcmp( *argv, "-xscale" ) ) {
            GetFloat( progname, "-xscale", *++argv, --argc, &xscale,
                      xscaleflag ) ;
        } else if ( 0 == strcmp( *argv, "-yscale" ) ) {
            GetFloat( progname, "-yscale", *++argv, --argc, &yscale,
                      yscaleflag ) ;
        } else if ( 0 == strcmp( *argv, "-skew" ) ) {
            GetFloat( progname, "-skew", *++argv, --argc, &skew,
                      skewflag ) ;
        } else if ( 0 == strcmp( *argv, "-rotate" ) ) {
            GetFloat( progname, "-rotate", *++argv, --argc, &rotate,
                      rotateflag ) ;
        } else if ( 0 == strcmp( *argv, "-diam" ) ) {
            GetInt( progname, "-diam", *++argv, --argc, &diam,
                    diamflag ) ;
        } else if ( 0 == strcmp( *argv, "-res" ) ) {
            GetFloat( progname, "-res", *++argv, --argc, &res,
                    resflag ) ;
        } else if ( 0 == strcmp( *argv, "-glyph" ) ) {
            GetInt( progname, "-glyph", *++argv, --argc, &glyph,
                    glyphflag ) ;
        } else if ( 0 == strcmp( *argv, "-xpos" ) ) {
            GetFloat( progname, "-xpos", *++argv, --argc, &xpos,
                    xposflag ) ;
        } else if ( 0 == strcmp( *argv, "-ypos" ) ) {
            GetFloat( progname, "-ypos", *++argv, --argc, &ypos,
                    yposflag ) ;
        } else {
            Usage( progname, "Unknown option" ) ;
        }
    }    // command line argument loop

    // some sanity checks
    if ( fabs( xscale ) > 50.0 || fabs( yscale ) > 50.0 ) {
        Fatal( "Scale factors too large" ) ;
    }
    if ( fabs( skew ) > 80.0 ) {
        Fatal( "Skew too large" ) ;
    }
    if ( diam < 1 || diam > 40  ) {
        Fatal( "Bad diameter" ) ;
    }
    if ( res < 50 || res > 1200 ) {
        Fatal( "Bad resolution" ) ;
    }
    if ( xpos < 5.0 || xpos > 1600.0 ) {
        Fatal( "Bad xpos" ) ;
    }
    if ( ypos < 5.0 || ypos > 1600.0 ) {
        Fatal( "Bad ypos" ) ;
    }

    SFOutput out ;
    SFInput  in ;

    // flip yscale since y-axis goes top to bottom
    res /= 72.0 ;    // pixels per point
    in.xform = new Xform( res * xscale, res * -yscale, skew, rotate,
                          0.0, 0.0 ) ;
    in.gCode = glyph ;
    in.diam = diam ;
    out.bm = NIL ;

#ifdef DEBUG
    if ( DBGFLAG ) {
        DBGFILE << "Before calling GetBitmap(), gCode = " << glyph
                << ", xscale = " << xscale
                << ", yscale = " << yscale
                << ", skew = "   << skew
                << ", rotate = " << rotate
                << ", diam = "   << diam
                << endl ;
    }
#endif /* DEBUG */

    SF_EStatus status = sfont->GetBitmap( &in, &out ) ;
    if ( eStatOk != status ) {
#ifdef DEBUG
        if ( eStatNoChar == status ) {
            cerr << "WARNING: Glyph not found" << endl ;
        }
#endif /* DEBUG */
        return TCL_OK ;
    }


    // ASCII dump
    if ( 1 == GetTclVarInt( interp, "ascii" ) ) {
        cout << "advance = " << out.adv << ", offset = " << out.off
             << endl ;
        if ( NIL != out.bm ) {
            out.bm->Dump() ;
        }
    }
    if ( NIL == out.bm ) {
        xpos += out.adv.x ;
        ypos += out.adv.y ;
        return TCL_OK ;
    }

    static int tag ;
    static int const cmdMax = 200 ;
    char cmd[ cmdMax ],
         *cmdPtr = cmd ;

    sprintf( cmd, "U%x", ++tag ) ;
    Tk_Uid uid = Tk_GetUid( cmd ) ;
    if ( TCL_OK != Tk_DefineBitmap(
            interp,
            uid,
            reinterpret_cast< char * >( out.bm->mData ),
            out.bm->mWd,
            out.bm->mHt ) ) {
        Fatal( "(GetTextChar): Tk_DefineBitmap() failed" ) ;
    }

    // clear area
    memset( cmd, static_cast<int>( ' ' ), cmdMax ) ;

    strcpy( cmdPtr, ".c create bitmap " ) ;
    cmdPtr += strlen( cmdPtr ) ;

    // coordinates
    sprintf( cmdPtr, "%d %d ",
             Round( xpos + out.off.x ), Round( ypos + out.off.y ) ) ;
    cmdPtr += strlen( cmdPtr ) ;

    // -anchor, -foreground, -tags, and -bitmap options
    strcpy( cmdPtr,
            "-anchor nw -foreground black -tags char -bitmap " ) ;
    cmdPtr += strlen( cmdPtr ) ;
    strcpy( cmdPtr, uid ) ;
    cmdPtr += strlen( cmdPtr ) ;

    if ( cmdPtr >= cmd + cmdMax ) {
        Fatal( "(DoGlyph): buffer overflow" ) ;
    }

    // Actually paint the bitmap
    if ( TCL_OK != Tcl_Eval( interp, cmd ) ) {
        cerr << "cmd = " << cmd << endl ;
        Fatal( "(DoGlyph): Tcl_Eval() failed" ) ;
    }

    xpos += out.adv.x ;
    ypos += out.adv.y ;

    // since tk owns the bitmap data, prevent its deletion
    out.bm->Zero() ;
    delete out.bm ;
    out.bm = NIL ;

    return TCL_OK ;
}                                                             // DoGlyph

//----------------------------------------------------------------  Init
//
int
Init(Tcl_Interp *interp)
{
#define MAIN_SCRIPT_FILE  "testfont.tcl"
#define DEF_FONT_FILE  "Pencil.tcl"

    if ( Tcl_Init(interp) == TCL_ERROR ) {
        return TCL_ERROR ;
    }
    if ( Tk_Init(interp) == TCL_ERROR ) {
        return TCL_ERROR ;
    }

    // create a separate interpreter for stroke fonts
    Tcl_Interp *fInterp = Tcl_CreateInterp() ;

    // initialize stroke font module
    InitSFont() ;

    // extract font file from command line args if any
    if ( TCL_OK != Tcl_EvalFile( interp, "testfont1.tcl" ) ) {
        Fatal( "(Init): Tcl_EvalFile() failed" ) ;
    }

    char *fontFile ;
    if ( NIL == interp->result || 0 == strlen( interp->result ) ) {
        fontFile = DEF_FONT_FILE ;
    } else {
        fontFile = interp->result ;
    }

    // read the font
    //
    StrokeFont *sfont = new StrokeFont( fontFile, fInterp ) ;

    // command for rasterizing a single glyph
    Tcl_CreateCommand( interp, "g", DoGlyph, (ClientData)sfont,
                       (Tcl_CmdDeleteProc *)NIL ) ;

    /*
     * Specify a user-specific startup file to invoke if the application
     * is run interactively.  Typically the startup file is "~/.apprc"
     * where "app" is the name of the application.  If this line is
     * deleted then no user-specific startup file will be run under any
     * conditions.
     */
#if (TCL_MAJOR_VERSION <= 7 && TCL_MINOR_VERSION <= 4)
    tcl_RcFileName = MAIN_SCRIPT_FILE ;
#endif
    Tcl_SetVar( interp, "tcl_rcFileName", MAIN_SCRIPT_FILE,
                TCL_GLOBAL_ONLY ) ;

    return TCL_OK ;

#undef MAIN_SCRIPT_FILE
}                                                                // Init

//----------------------------------------------------------------  main
//
int
main( int const argc, char **argv )
{
    // Initialize tcl
    Tk_Main( argc, argv, Init ) ;

    exit( 0 ) ;
}
