#=======================================================================
#
# Some common procs for 'Pencil' stroke font
#
# Copyright (C 1999 CurveSoft, Inc. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.
#  
# $Revision: 1.13 $
#
#=======================================================================

# Auxiliary procedures called by the character definitions
#
# In principle, only moveto, lineto, and curveto are needed; the others
# are for convenience
#
# The core commands are the following. They are
#   -- defined as tcl procs to draw to the screen (mode=display) or
#   -- defined as tcl procs to output PS commands (mode=ps)      or
#   -- bound to C/C++ font scaler functions       (mode=scaler)
#
# Arguments of the "S_" command are:
#   g:                glyph code in hex
#   wx,wy:            width vector
#   llx,lly,urx,ury:  bounding box
#   nSeg:             no. of segments (L_ and C_)
#   nStr:             no. of strokes (M_)
#
#   S_  g wx wy llx lly urx ury nSeg nStr (start glyph definition)
#   M_  x y                               (moveto)
#   L_  x y                               (lineto)
#   C_  x y x1 y1 b                       (curveto)
#   E_                                    (end glyph definition)
#


#---------------------------------------------------------------  moveto
#
proc moveto {newx newy} {
    global aName; upvar #0 $aName G
    M_ $newx $newy
    set G(curx) $newx
    set G(cury) $newy
}

#---------------------------------------------------------------  lineto
#
proc lineto {newx newy} {
    global aName; upvar #0 $aName G
    L_ $newx $newy
    set G(curx) $newx
    set G(cury) $newy
}

#--------------------------------------------------------------  curveto
#
proc curveto {xB yB xC yC {bulge 0.9}} {
    global aName; upvar #0 $aName G
    C_ $xB $yB $xC $yC $bulge
    set G(curx) $xB
    set G(cury) $yB
}

#-----------------------------------------------------------------------
# Other moveto variants
#-----------------------------------------------------------------------

#--------------------------------------------------------------  rmoveto
#
proc rmoveto {rx ry} {
    global aName; upvar #0 $aName G
    moveto [expr $rx+$G(curx)] [expr $ry+$G(cury)]
}

#--------------------------------------------------------------  xmoveto
#
# Horizontal move
#
proc xmoveto {newx} {
    global aName; upvar #0 $aName G
    moveto $newx $G(cury)
}

#--------------------------------------------------------------  ymoveto
#
# Vertical move
#
proc ymoveto {newy} {
    global aName; upvar #0 $aName G
    moveto $G(curx) $newy
}

#-----------------------------------------------------------------------
# Other lineto variants
#-----------------------------------------------------------------------

#--------------------------------------------------------------  rlineto
#
proc rlineto {rx ry} {
    global aName; upvar #0 $aName G
    lineto [expr $rx+$G(curx)] [expr $ry+$G(cury)]
}

#-------------------------------------------------------------  rxlineto
#
proc rxlineto {rx} {
    global aName; upvar #0 $aName G
    lineto [expr $rx+$G(curx)] $G(cury)
}

#-------------------------------------------------------------  rylineto
#
proc rylineto {ry} {
    global aName; upvar #0 $aName G
    lineto $G(curx) [expr $ry+$G(cury)]
}

#--------------------------------------------------------------  xlineto
#
# Horizontal line
#
proc xlineto {newx} {
    global aName; upvar #0 $aName G
    lineto $newx $G(cury)
}

#--------------------------------------------------------------  ylineto
#
# Vertical line
#
proc ylineto {newy} {
    global aName; upvar #0 $aName G
    lineto $G(curx) $newy
}

#-----------------------------------------------------------------------
# Other curveto variants
#-----------------------------------------------------------------------

#-------------------------------------------------------------  rcurveto
#
# B and C are relative to current point
#
proc rcurveto {rxB ryB rxC ryC {bulge 0.9}} {
    global aName; upvar #0 $aName G
    curveto [expr $rxB+$G(curx)] [expr $ryB+$G(cury)] \
        [expr $rxC+$G(curx)] [expr $ryC+$G(cury)] $bulge
}

#-------------------------------------------------------------  xcurveto
#
# AC is horizontal, CB is vertical, i.e. xC = xB and yC = cury
#
proc xcurveto {xB yB {bulge 0.9}} {
    global aName; upvar #0 $aName G
    curveto $xB $yB $xB $G(cury) $bulge
}

#-------------------------------------------------------------  ycurveto
#
# AC is vertical, CB is horizontal, i.e. xC = curx and yC = yB
#
proc ycurveto {xB yB {bulge 0.9}} {
    global aName; upvar #0 $aName G
    curveto $xB $yB $G(curx) $yB $bulge
}

#------------------------------------------------------------  rxcurveto
#
# relative xcurveto: AC horizontal, CB vertical
#
proc rxcurveto {rxB ryB {bulge 0.9}} {
    global aName; upvar #0 $aName G
    set xB [expr $rxB + $G(curx)]
    set cury $G(cury)
    curveto $xB [expr $ryB + $cury] $xB $cury $bulge
}

#------------------------------------------------------------  rycurveto
#
# relative ycurveto: AC vertical, CB horizontal
#
proc rycurveto {rxB ryB {bulge 0.9}} {
    global aName; upvar #0 $aName G
    set curx $G(curx)
    set yB [expr $ryB + $G(cury)]
    curveto [expr $rxB + $curx] $yB $curx $yB $bulge
}

