#=======================================================================
#
# Data and procedures used for screen display of strokefonts. Can be
# shared across fonts.
#
# Copyright (C 1999 CurveSoft, Inc. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.
#  
# $Revision: 1.13 $
#
#=======================================================================

#-------------------------------------------------------------  WrapupSf
#
# Creates a canvas and draws axes and a grid
#
proc WrapupSf {} {
    global aName; upvar #0 $aName G

    # display window parameters
    set G(canvasWd) [expr int(0.8 * [winfo screenwidth .])]
    set G(canvasHt) [expr int(0.8 * [winfo screenheight .])]

    # location of origin on canvas
    set G(origx) [expr int(0.08 * $G(canvasWd))]
    set G(origy) [expr 0.78 * $G(canvasHt)]

    # stroke width and color
    set G(sw) 5
    set G(color) magenta

    # Scale factor to go from font coordinates to canvas coordinates
    set G(scale) [expr double($G(canvasHt)) * 0.95 / $G(emSize)]

    wm title . $G(fontName)
    wm geometry . +5+15
    canvas .c -height $G(canvasHt) -width $G(canvasWd) \
        -background black
    pack .c

    set aCol red;      # axis color
    set gCol blue;     # grid color
    set lCol yellow;   # grid label color
    set hCol green;    # heights color

    set cWd  $G(canvasWd)
    set cHt  $G(canvasHt)
    set ox   $G(origx)
    set oy   $G(origy)
    set em   $G(emSize)
    set sc   $G(scale)

    # axes
    .c create line 0   $oy  $cWd $oy -fill $aCol
    .c create line $ox $cHt $ox  0   -fill $aCol

    # grid label locations
    set yt [expr $oy + 20]
    set xt [expr $ox - 20]

    # grid
    for {set i 500} {$i < $em} {incr i 500} {
        set t [expr $i * $sc]
        set x [expr $t + $ox]
        set y [expr $oy - $t]

        .c create line $x $cHt $x 0 -fill $gCol
        .c create text $x $yt -fill $lCol -text $i

        .c create line $ox $y $cWd $y -fill $gCol
        .c create text $xt $y -fill $lCol -text $i
    }

    # heights -- add more later if necessary
    set ytmp [expr $oy - $sc * $G(ht_X)]
    .c create line 0 $ytmp $cWd $ytmp -fill $hCol

    set tmp [expr $oy - $sc * $G(ht_asc)]
    if {$tmp != $ytmp} {
        .c create line 0 $tmp $cWd $tmp -fill $hCol
    }

    set ytmp [expr $oy - $sc * $G(ht_x)]
    .c create line 0 $ytmp $cWd $ytmp -fill $hCol

    set ytmp [expr $oy - $sc * $G(ht_dsc)]
    .c create line 0 $ytmp $cWd $ytmp -fill $hCol
}

#----------------------------------------------------------------  clear
#
proc clear {} {.c delete all}

#------------------------------------------------------------------  dot
#
# Draws a filled dot of the given color and radius centered at (x,y)
#
proc dot {x y {rad 3} {color cyan}} {
    .c create oval [expr $x-$rad] [expr $y-$rad] \
        [expr $x+$rad] [expr $y+$rad] -fill $color -tags char
    update idletasks
}

#------------------------------------------------------------  subdivide
#
# Subdivides an ABC conic (endpoints AB, apex C, bulge b) at the
# shoulder point P and returns the coordinates of P and of the
# intersections of the tangent at P with AC and BC as the points D and E
# respectively. The new sharpness of the two pieces is returned in newb
#
proc subdivide {xA yA xB yB xC yC b xd yd xp yp xe ye newb } {

    upvar $xd xD $yd yD $xp xP $yp yP $xe xE $ye yE $newb bN
    set u [expr $b/(1.0+$b)]

    set xM [expr (double($xA)+$xB)/2.0]
    set yM [expr (double($yA)+$yB)/2.0]

    set xD [expr $xA+(double($xC)-$xA)*$u]
    set yD [expr $yA+(double($yC)-$yA)*$u]

    set xP [expr $xM+(double($xC)-$xM)*$u]
    set yP [expr $yM+(double($yC)-$yM)*$u]

    set xE [expr $xB+(double($xC)-$xB)*$u]
    set yE [expr $yB+(double($yC)-$yB)*$u]

    set bN [expr sqrt((1.0+$b)/2.0)]
}

#-------------------------------------------------------------------  c3
#
proc c3 {xA yA xB yB xC yC bulge color wid} {
    subdivide $xA $yA $xB $yB $xC $yC $bulge xD yD xP yP xE yE bulge
    .c create line $xA $yA $xD $yD $xE $yE $xB $yB -fill $color \
        -smooth 1 -width $wid -capstyle round -joinstyle round \
        -tags char
}

#------------------------------------------------------------  tcl-conic
#
# Draws a conic bounded by triangle ABC passing through A and B; drawn
# by approximating it with a tk spline.  Negative bulge is handled by
# doing an extra subdivision.
#
proc tcl-conic {xA yA xB yB xC yC {bulge 1.0} {color green} {wid 0}} {
    if {$bulge >= 0.0} {
        c3 $xA $yA $xB $yB $xC $yC $bulge $color $wid
    } else {
        subdivide $xA $yA $xB $yB $xC $yC $bulge xD yD xP yP xE yE bulge
        c3 $xA $yA $xP $yP $xD $yD $bulge $color $wid
        c3 $xP $yP $xB $yB $xE $yE $bulge $color $wid
    }
}

#-------------------------------------------------------------  tocanvas
#
# Converts font coordinates to canvas coordinates
#
proc tocanvas {xx yy} {
    global aName; upvar #0 $aName G
    upvar $xx x $yy y
    set x [expr $G(origx) + $G(scale) * $x]
    set y [expr $G(origy) - $G(scale) * $y]
}

#--------------------------------------------------------------  curInit
#
# These variables are used to detect errors such as:
# -- two S_ commands in a char
# -- incorrect segment or stroke counts
# -- points outside the declared bbox
#
# This code will not be running embedded in an application so there is
# no need to worry about namespace conflicts here
#
proc curInit {} {
    global cur
    set cur(llx)  0;     set cur(lly)  0
    set cur(urx)  0;     set cur(ury)  0
    set cur(nSeg) 0;     set cur(nStr) 0
    set cur(cntSeg) 0;   set cur(cntStr) 0
    set cur(start)  0;   set cur(code)   0
}

#-------------------------------------------------------------------  S_
#
# Start char definition
#
# Deletes any previous displayed char and displays a dot at the end of
# the advance width vector.
#
proc S_ {g wx wy llx lly urx ury nSeg nStr} {
    global cur aName; upvar #0 $aName G
    set i [lindex [info level [expr [info level] - 1]] 0]

    if ![regexp (^$G(prefix)@)(\[0-9a-f\]+)($G(suffix)$) \
            $i z x gg y] {
        puts stderr "ERROR(S_): Bad char proc ($i)"; exit 1
    }
    if {$g != $gg} {
        puts stderr "ERROR(S_): Bad code ($gg != $g)"; exit 1
    }
    if $cur(start) {
        curInit; error "ERROR(S_,$cur(code)): Missing E_"
    }
    set cur(start) 1;        set cur(code) $g
    set cur(nSeg)  $nSeg;    set cur(nStr) $nStr
    set cur(llx)   $llx;     set cur(lly)  $lly
    set cur(urx)   $urx;     set cur(ury)  $ury

    .c delete char
    .c create text 15 [expr $G(canvasHt) - 30]  -fill red -text $g \
        -tags char
    tocanvas wx wy; dot $wx $wy; update idletasks
}

#-------------------------------------------------------------------  E_
#
# Error check
#
proc E_ {} {
    global cur
    if !$cur(start) {
        curInit
        error "ERROR(E_,$cur(code)): Missing S_"
    }
    if {$cur(cntStr) != $cur(nStr)} {
        curInit
        error "ERROR(E_,$cur(code)): wrong stroke count. \
            (cntStr=$cur(cntStr), nStr=$cur(nStr))"
    }
    if {$cur(cntSeg) != $cur(nSeg)} {
        curInit
        error "ERROR(E_,$cur(code)): wrong segment count. \
            (cntSeg=$cur(cntSeg), nSeg=$cur(nSeg))"
    }
    curInit
}

#------------------------------------------------------------------  Chk
#
# check a point; z is name of caller
#
proc Chk {x y z} {
    global cur
    if !$cur(start) {
        curInit; error "ERROR($z,$cur(code)): Missing S_"
    }
    if {$x < $cur(llx) || $cur(urx) < $x } {
        curInit; error "ERROR($z,$cur(code)): x=$x outside bbox"
    }
    if {$y < $cur(lly) || $cur(ury) < $y } {
        curInit; error "ERROR($z,$cur(code)): y=$y outside bbox"
    }
    return 1
}

#-------------------------------------------------------------------  M_
#
# moveto (error check)
#
proc M_ {x y} {
    global cur
    Chk $x $y M_
    incr cur(cntStr)
    if {$cur(cntStr) > $cur(nStr) } {
        curInit
        error "ERROR(M_,$cur(code)): too many strokes"
    }
}

#-------------------------------------------------------------------  L_
#
# lineto (error check)
#
proc L_ {x y} {
    global cur aName; upvar #0 $aName G
    Chk $x $y L_
    incr cur(cntSeg)
    if {$cur(cntSeg) > $cur(nSeg)} {
        curInit
        error "ERROR(L_,$cur(code)): too many segments"
    }

    set curx $G(curx);    set cury $G(cury)
    tocanvas curx cury;   tocanvas x y
    .c create line $curx $cury $x $y -fill $G(color) \
        -width $G(sw) -capstyle round -joinstyle round -tags char
    update idletasks
}

#-------------------------------------------------------------------  C_
#
# conicto
#
proc C_ {x y x1 y1 b} {
    global cur aName; upvar #0 $aName G
    Chk $x $y C_
    incr cur(cntSeg)
    if {$cur(cntSeg) > $cur(nSeg) } {
        curInit
        error "ERROR(C_,$cur(code)): too many segments"
    }

    set curx $G(curx);   set cury $G(cury)
    tocanvas curx cury;  tocanvas x y;        tocanvas x1 y1
    tcl-conic $curx $cury $x $y $x1 $y1 $b $G(color) $G(sw)
    update idletasks
}

curInit
